/**** bitrev(ix,iy,n,nb,isgn): reverse bits in a word *********
*     fortran call is  call bitrev(ix,iy,n,nb,isgn)           *
*     arguments are: ix: input array			      *
*		     iy: output array                         *
*		      n: number of 32 bit words to convert    *
*		     nb: number of bottom bits to reverse     *
*		   isgn: if nonzero then sign-extend result,  *
*        		 else don't                           *
*							      *
* bitrev takes n words from array ix and puts them into array *
* iy with the bottom nb bits reversed end for end. the higher *
* bits are not saved. if isgn is nonzero sign extension is    *
* done on the result.					      *
*							      *
* no return value.					      *
**************************************************************/

/****** this is the actual FORTRAN-called routine       *****/
bitrev_(ix,iy,n,nb,isgn)
int ix[],iy[],*n,*nb,*isgn;
{

bitrevc(ix,iy,*n,*nb,*isgn);		/* let 'c' function do the work */

} /*** end of bitrev_() ***/

/*************************************************************/

/****** 		'C'-called routine             *******/
#define MAXBITS 32	 			/* vax 'word' size in bits */
#define MASK    0xffffffff			/* vax */

bitrevc(ix,iy,n,nb,isgn)
int ix[],iy[],n,nb,isgn;
{
register unsigned temp,iwrd,mask;		/* keep these handy */
unsigned msk;
int i,j,mb;

if (!nb || nb > MAXBITS) {			/* nb=0 is nonsense */
	for (i=0; i < n; i++) iy[i] = ix[i];
	return;
	};

msk = ~(MASK << nb);

mb = nb-1;
for (i=0; i < n; i++) {				/* loop for n words */	
	mask = msk;
	iwrd = ix[i] & mask;		       /* zero all but nb bottom bits */
	temp = 0;

	for (j=0; j < nb; j++) {		/* loop for nb bits */
		
		temp += (iwrd >> (mb-j)) << j;  /* store running reversal */
		mask >>= 1;
		iwrd &= mask;			/* clear bit just obtained */
		};

	/** check for sign extension by using new high bit (old low bit)  **/
	if (isgn && (ix[i] & 1)) iy[i] = temp | ~msk;
	else iy[i] = temp;
	};

} /* end of bitrev() */

/*** end of file bitrev.c ***/

