      function iscnsp(it1,it2,isamp,mode)
c  finds the number of scans (terms, data points, . . .) separating times
c  specified in the arrays it1 and it2, for a sample interval of isamp
c  seconds. it2 is taken to be the later time; if it is actually earlier,
c  the function will return an appropriately negative value.  if the
c  dates are the same, 0 is returned (note that this value therefore
c  differs from the usual scan count)
c   mode sets the way in which the dates are given; if it is 'd' the dates
c  are year, day, hour, minute, second; if it is 'm' the dates are year,
c  month, day, hour, minute, second.
c
c  $$$$calls juldat,toymd,muldiv
c
      character*1 mode
      dimension it1(*),it2(*)
      dimension il1(6),il2(6)
      common /scanrem/irc
c convert times to Julian Date, and seconds of day
      if(mode.eq.'d') then
        call toymd(it1,il1)
        call toymd(it2,il2)
c  since seconds not used, do not need this copying
c       do i=3,5
c         il1(i+1)=it1(i)
c         il2(i+1)=it2(i)
c       enddo
        jd1 = juldat(il1)
        jd2 = juldat(il2)
        js1=60*(60*it1(3)+it1(4))+it1(5)
        js2=60*(60*it2(3)+it2(4))+it2(5)
      endif
      if(mode.ne.'d') then
        jd1 = juldat(it1)
        jd2 = juldat(it2)
        js1=60*(60*it1(4)+it1(5))+it1(6)
        js2=60*(60*it2(4)+it2(5))+it2(6)
      endif
      idir=1
c     if time-2 earlier than time-1, note this and swap the times
      if(jd2.lt.jd1.or.jd2.eq.jd1.and.is2.lt.is1) then
        j=jd2
        jd2=jd1
        jd1=j
        j=js2
        js2=js1
        js1=j
        idir=-1
      endif
c If necessary adjust the end date to a day earlier, to make the number of
c seconds for the later day greater than the the number on the start day
      if(js2.lt.js1) then
        js2=js2+86400
        jd2=jd2-1
      endif
      js=js2-js1
      jd=jd2-jd1
c Time span is jd days plus js seconds. Convert this to
c  scans, plus a remainder if it is not an even number.
      call muldiv(jd,86400,isamp,iscnsp,irc)
c add in the seconds part
      iscnsp = iscnsp + js/isamp
c and combine the two remainders and add them in as well
      irc = irc + mod(js,isamp)
      iscnsp = iscnsp + irc/isamp
      irc = mod(irc,isamp)
c scans are < 0 if time-2 is before time-1
      iscnsp = idir*iscnsp
      return
      end
      subroutine timfnd(it1,iscans,isamp,it2,mode)
c  given a start date (in it1), the number of separating scans iscans, the
c  sample interval in seconds isamp, finds the end date it2.
c   mode sets the way in which the dates aregiven; if it is 'd' the dates
c  are year, day, hour, minute, second; if it is 'm' the dates are year,
c  month, day, hour, minute, second
c   note that iscansp is the number of separating scans, not the scan count;
c  if it is zero, it2 will be set to it1
c
c  $$$$calls juldat,toymd,muldiv,datjul,toyd
c
      dimension it1(6),it2(6)
      dimension il1(3),il2(3)
      character*1 mode
c convert start time to ymd
      if(mode.eq.'d') call toymd(it1,il1)
      if(mode.ne.'d') then
        do i=1,3
          il1(i)=it1(i)
        enddo
      endif
c convert start time to Julian date, and seconds of that day
      jd1= juldat(il1)
      if(mode.eq.'d') js1=60*(60*it1(3)+it1(4))+it1(5)
      if(mode.ne.'d') js1=60*(60*it1(4)+it1(5))+it1(6)
c check for negative scan count
      lscans=iabs(iscans)
c find the series length in half-days and seconds. Half-days are used to
c  avoid overflow with 32-bit integers
      call muldiv(lscans,isamp,43200,ihd,is)
c now convert to days, and add seconds to get any additional days
      id=ihd/2
      is=is+43200*mod(ihd,2)
c 2 later than 1
      if(iscans.ge.0) then
        jd2=jd1+id
        js2=js1+is
        jd2=jd2+js2/86400
        js2=mod(js2,86400)
      endif
c 1 later than 2 (negative scans)
      if(iscans.lt.0) then
        jd2=jd1-id
        js2=js1-is
        if(js2.lt.0) then
          jd2=jd2-1
          js2=js2+86400
        endif
      endif
c now have end time in Julian date and seconds of that day. Convert back
c  to calendar date
      call datjul(jd2,il2)
      its=mod(js2,60)
      itm=mod(js2,3600)/60
      ith=js2/3600
      if(mode.ne.'d') then
        do i=1,3
          it2(i)=il2(i)
        enddo
        it2(6)=its
        it2(5)=itm
        it2(4)=ith
      endif
      if(mode.eq.'d') then
        call toyd(il2,it2)
        it2(5)=its
        it2(4)=itm
        it2(3)=ith
      endif
      return
      end
      subroutine toymd(it1,it2)
      dimension it1(*),it2(*)
c
c  converts times in it1 given as year and day to year-month-day in it2
c
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      mon(jj,m) = (12*(jj-29-m))/367 + 2 + (jj-200)/169
      it2(1)=it1(1)
      it2(2)=mon(it1(2),leap(it1(1)))
      it2(3) = it1(2) - idn(it2(2)) - leap(it2(1))*((9+it2(2))/12)
      return
      end
      subroutine toyd(it1,it2)
      dimension it1(*),it2(*)
c
c  converts times in it1 given as year-month-day to year and day in it2
c
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      it2(1)=it1(1)
      it2(2) = it1(3) + idn(it1(2)) + leap(it1(1))*((9+it1(2))/12)
      return
      end
      function leap(iy)
c
c returns 1 if year is a leap year, by Clavian (Gregorian) rule
c
      leap = 1 - (mod(iy,4)+3)/4
      if(mod(iy,100).eq.0.and.mod(iy,400).ne.0) leap=0
      return
      end
      function juldat(it)
      dimension it(*)
c
c  Julian Date from Gregorian date, Algorithm from p. 604, Explanatory
c    Supplement Amer Ephemeris & Nautical Almanac (cf Comm CACM, 11, 657 (1968)
c    and 15, 918 (1972)) Valid for all positive values of Julian Date
c
      juldat=(1461*(it(1)+4800+(it(2)-14)/12))/4
     1     + (367*(it(2)-2-12*((it(2)-14)/12)))/12
     2     - (3*((it(1)+4900+(it(2)-14)/12)/100))/4+it(3)-32075
      return
      end
      subroutine datjul(jd,it)
c
c  Converts Julian Date jd into year-month-day (Gregorian calendar) in
c    array it. Algorithm from p. 6??, Explanatory Supplement Amer Ephemeris
c    & Nautical Almanac
c
      dimension it(*)
      k = jd+68569
      n = 4*k/146097
      k = k - (146097*n+3)/4
      it(1) = 4000*(k+1)/1461001
      k = k - 1461*it(1)/4 + 31
      it(2) = 80*k/2447
      it(3) = k - 2447*it(2)/80
      k = it(2)/11
      it(2) = it(2) + 2 - 12*k
      it(1) = it(1) + 100*(n-49) + k 
      return
      end
      subroutine muldiv(n1,n2,n3,np,nr)
c
c   finds the integer part and remainder of the quantity (n1*n2)/n3
c    without forming the product n1*n2, to minimize the chance of overflow
c      Algorithm by R. L. Parker, modified by D. C. Agnew
c
c    Basic idea is that we can decompose an integer n2 to be j1 mutliples
c       of n3, plus a remainder k1
c            n2 = j1*n3 + k1   whence in real arithmetic
c          n2/n3 = j1 + k1/n3  and this means that
c          n1*n2/n3 = n1*j1 + n1*k1/n3 and we can take the integer part
c    and remainder of n1*k1/n3 to get the integer part and remainder of
c    n1*n2/n3, with k1 being bounded as n2 is not. A second application  
c    gives the algorithm below.
c
      j2=n2/n3
      k2=mod(n2,n3)
      j1=n1/n3
      k1=mod(n1,n3)
c product nn has max value of n3*n3, so for n3*n3 < max value, will not
c  overflow unless another product is actually too large
      nn=k1*k2
      np=n1*j2 + j1*k2 + nn/n3
      nr=mod(nn,n3)
      return
      end
      function ndays(it1,it2,mode)
c  finds the number of days between two dates, given in arrays it1 and it2.
c  it2 is taken to be later. the number of days is 0 if the dates are the
c  same, 1 if it2 is one day later, -1 if it is one day earlier, and so on.
c   mode sets the way in which the date is given; if it is 'd' the date
c  is y,d; if it is 'm' the date is y,m,d.
c
c  $$$$calls juldat
c
      character*1 mode
      dimension it1(*),it2(*)
      dimension il1(3),il2(3)
      if(mode.ne.'d') then
        jd1 = juldat(it1)
        jd2 = juldat(it2)
      endif
      if(mode.eq.'d') then
        call toymd(it1,il1)
        call toymd(it2,il2)
        jd1 = juldat(il1)
        jd2 = juldat(il2)
      endif
      ndays=jd2-jd1
      return
      end
      subroutine weekday(it,dayname,mode)
c
c   returns the name of the day of the week for time given in it, either
c   as y-m-d (mode m) or y-d (mode d)
      character*1 mode
      character*3 dayname
      character*3 names
      dimension it(*)
      dimension names(7),il(3)
      data names/'Mon','Tue','Wed','Thu','Fri','Sat','Sun'/
      if(mode.ne.'d') jd = juldat(it)
      if(mode.eq.'d') then
        call toymd(it,il)
        jd = juldat(il)
      endif
      dayname=names(mod(jd,7)+1)
      return
      end
      subroutine monnam(it,month,mode)
c
c   returns the name of the month week for time given in it, either
c   as y-m-d (mode m) or y-d (mode d)
      character*1 mode
      character*3 month
      character*3 names
      dimension it(*)
      dimension names(12),il(3)
      data names/'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep',
     1  'Oct','Nov','Dec'/
      if(mode.ne.'d') then
         month=names(it(2))
      else
         call toymd(it,il)
         month=names(il(2))
      endif
      return
      end
      subroutine julmon(it1,it2)
c  converts from year-day-h-m-s in array it1, to y-month-day-h-m-s in it2
c  assumes all leap years to be those divisible by 4
      dimension it1(5),it2(6)
      mon(jj,m) = (12*(jj-29-m))/367 + 2 + (jj-200)/169
      leap(jj) = 1 - (mod(jj,4)+3)/4
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      it2(1) = it1(1)
      it2(2) = mon(it1(2),leap(it2(1)))
      it2(3) = it1(2) - idn(it2(2)) - leap(it2(1))*((9+it2(2))/12)
      it2(4) = it1(3)
      it2(5) = it1(4)
      it2(6) = it1(5)
      return
      end
      function iweek(iy,im,id)
c  finds day of week (1 is Monday) for given date
      jd = jcscal(iy,im,id)
      iweek = mod(jd,7) + 1
      return
      end
      function jcscal(iy,m,id)
c  computes the Julian date starting at noon on the date given
c  by year iy, month m, and day id; this is also the Modifed Julian
c  Date for that whole day, if 2400001 is subtracted
      jcscal = 367*iy - 7*(iy+(m+9)/12)/4 + 275*m/9 + id + 1721029
     1   - 3*((iy+(m-9)/7)/100+1)/4
      return
      end
      function jgwk(iy,m,id)
c  returns the GPS week for a given year month and day (must be after
c  1980 6 Jan, which is the zero point)
      jgwk = (jcscal(iy,m,id) - 2444245)/7
      return
      end
