      subroutine qrseq(a,x,esq,m,msub,n,mode)
c$$$$$ calls h12,rows   
c  performs stable solution of linear least-squares problem by up-
c  dated form of golub q-r method.   see lawson+hanson - solving least   
c  squares problems - prentice-hall 1974(pp 208-211)
c  calls a user-provided subroutine  rows  to fetch into memory a block  
c  of  msub  data and associated rows of coeff matrix.  msub  may be 1,  
c  but routine is up to twice as fast if msub is larger.   
c  a  is a working array with at leas t (n+1+msub)*(n+1)  elements.  if  
c     msub=m, only (n+1)*m  elements need be reserved for  a .    
c  x  is an  n-vector containing the solution on exit.     
c     inconsistent problem,  the routine will return  esq=-1.     
c  m  total number of data in least-squares system. 
c  msub  size of data block to be fetched into memory.  all blocks
c     will be  msub  in length, except perhaps the last one which may be 
c     smaller.   
c  n  number of parameters to be fitted.     
c  mode if 1, solves a new system. if 2, continues solving a system      
c     already begun with mode=1.      
c  must have m.ge.msub,  m.ge.n
      dimension a(*),x(*)      
      data l/0/  
c  
      n1=n +1    
      msub=min0(msub,m) 
      ma=msub + n1      
      if (msub.eq.m) ma=m      
      if (mode.eq.1) l=0
c  forms  right-triangle matrix from data blocks    
      do 1500 it=1,m,msub      
      mt=min0(msub,m-it+1)     
      ip=l + mt  
c  inside subroutine rows,  a  must be dimensioned  a(ma,n1).  only the  
c  1st mt  rows must be filled as follows,  a(i,n1), i=1,mt  hold the    
c  next block of data. a(i,1),a(i,2) ... a(i,n1-1),i=1,mt  hold the      
c  corresponding rows of the coeff matrix.   
      call rows(a(l+1),ma,mt,n1)      
c  
      min=min0(n1,ip-1) 
      ij=1
      do 1200 i=1,min   
      call h12(1,i,max0(i+1,l+1),ip,a(ij),1,rho,a(ij+ma),1,ma,n1-i)   
 1200 ij=ma + ij 
 1500 l=min0(n1,ip)     
c  
c  solve triangular system in  a  for  x, and copy res from bottom of a. 
      do 2200 ii=1,n
      i=n-ii+1
      ij=ma*(n1-1)+i
      sum=a(ij)
      if (ii.eq.1) go to 2110
      i1=i+1
      if (i1.gt.n) go to 2110
      ij=ma*(i1-2) + i
      do 2100 j=i1,n
      ij=ma + ij
 2100 sum=sum - a(ij)*x(j)
 2110 ij=ma*(i-1)+i
      if (a(ij).eq.0.0) go to 3000
 2200 x(i)=sum/a(ij)
c
      nn=n1 + n*ma
      esq=a(nn)**2      
      return     
c  error exit - singular system
 3000 esq=-1.0   
      return     
      end   
      subroutine h12(mode,lpivot,l1,m,u,iue,up,c,ice,icv,ncv)     
c$$$$$ calls no other routines 
c  construction and application of householder transformation for nnlsq  
c  modified   from  solving least squares problems.  no double precision 
      dimension u(iue,m),c(1)  
      if (0.ge.lpivot .or. lpivot.ge.l1 .or. l1.gt.m) return      
      cl=abs(u(1,lpivot))      
      if (mode.eq.2) go to 60  
c  construct transformation    
      do 10 j=l1,m      
 10   cl=amax1(abs(u(1,j)),cl) 
      if (cl) 130,130,20
 20   clinv=1.0/cl      
      sm=(u(1,lpivot)*clinv)**2
      do 30 j=l1,m      
 30   sm=(u(1,j)*clinv)**2 + sm
      sm1=sm     
      cl=sqrt(sm1)*cl   
      if (u(1,lpivot)) 50,50,40
 40   cl=-cl     
 50   up=u(1,lpivot) - cl      
      u(1,lpivot)=cl    
      go to 70   
c  aplly transformation 
 60   if (cl) 130,130,70
 70   if (ncv.le.0) return     
      b=up*u(1,lpivot)  
      if (b) 80,130,130 
 80   b=1.0/b    
      i2=1-icv+ice*(lpivot-1)  
      incr=ice*(l1-lpivot)     
      do 120 j=1,ncv    
      i2=icv + i2
      i3=i2 + incr      
      i4=i3      
      sm=c(i2)*up
      do 90 i=l1,m      
      sm=c(i3)*u(1,i) + sm     
 90   i3=ice + i3
      if (sm) 100,120,100      
 100  sm=b*sm    
      c(i2)=sm*up + c(i2)      
      do 110 i=l1,m     
      c(i4)=sm*u(1,i) + c(i4)  
 110  i4=ice + i4
 120  continue   
 130  return     
      end     
