      subroutine svd(m,n,a,u,v,q,index)      
c$$$$$ calls no other routines 
c  singular value decomposition)  for algo program see wilkinson+reinsch 
c  handbook for automatic computation vol 2 - linear algebra, pp140-144  
c  translated from algol by r.l.parker
c  the matrix a(m,n) is decomposed.  singular values in q, pre-matrix in u,     
c  post-matrix in v.   index may be 1,2,3 or 4.  if 1, find u,v. if 2, find     
c  only u. if 3, find only v. if 4, find neither. in all cases, the array  u    
c  must be supplied as it is used as working space for the routine.      
      dimension a(m,n),u(m,n),v(n,n),q(n)    
      dimension e(200)  
c  
      eps=1.0e-10
      tol=1.0e-38
      do 1100 i=1,m     
      do 1100 j=1,n     
 1100 u(i,j)=a(i,j)     
c  householder reduction to bi-diagonal form 
      g=0.0      
      x=0.0      
      do 2900 i=1,n     
      e(i)=g     
      s=0.0      
      l=i+1      
      do 2100 j=i,m     
 2100 s=u(j,i)**2 + s   
      if (s .lt. tol) go to 2500      
      f=u(i,i)   
      g=-sign(sqrt(s),f)
      h=f*g - s  
      u(i,i)=f - g      
      if (l.gt.n) go to 2501   
      do 2400 j=l,n     
      s=0.0      
      do 2200 k=i,m     
 2200 s=u(k,i)*u(k,j) + s      
      f=s/h      
      do 2300 k=i,m     
 2300 u(k,j)=u(k,j) + f*u(k,i) 
 2400 continue   
      go to 2501 
 2500 g=0.0      
c  
 2501 continue   
      q(i)=g     
      s=0.0      
      if (l.gt.n) go to 2601   
      do 2600 j=l,n     
 2600 s=u(i,j)**2 + s   
 2601 if (s.lt.tol) go to 2800 
      f=u(i,i+1) 
      g=-sign(sqrt(s),f)
      h=f*g - s  
      u(i,i+1)=f - g    
      if (l.gt.n) go to 2651   
      do 2650 j=l,n     
 2650 e(j)=u(i,j)/h     
 2651 continue   
      if (l.gt.m) go to 2850   
      do 2700 j=l,m     
      s=0.0      
      if (l.gt.n) go to 2700   
      do 2670 k=l,n     
 2670 s=u(j,k)*u(i,k) + s      
      do 2690 k=l,n     
 2690 u(j,k)=u(j,k) + s*e(k)   
 2700 continue   
      go to 2850 
 2800 g=0.0      
 2850 y=abs(q(i)) + abs(e(i))  
      if (y .gt. x) x=y 
 2900 continue   
c  
c  accumulation of right-hand transforms (v) 
c  
      go to (3000,3701,3000,3701),index      
 3000 continue   
      do 3700 iback=1,n 
      i=n+1-iback
      if (g .eq. 0.0) go to 3500      
      h=u(i,i+1)*g      
      if (l.gt.n) go to 3500   
      do 3100 j=l,n     
 3100 v(j,i)=u(i,j)/h   
      do 3400 j=l,n     
      s=0.0      
      do 3200 k=l,n     
 3200 s=u(i,k)*v(k,j) + s      
      do 3300 k=l,n     
 3300 v(k,j)=v(k,j) + s*v(k,i) 
 3400 continue   
 3500 continue   
      if (l.gt.n) go to 3601   
      do 3600 j=l,n     
      v(j,i)=0.0 
 3600 v(i,j)=0.0 
 3601 v(i,i)=1.0 
      g=e(i)     
      l=i 
 3700 continue   
 3701 continue   
c  
c  accumulation of left-hand transforms      
      go to (4000,4000,4701,4701),index      
 4000 continue   
      do 4700 iback=1,n 
      i=n+1-iback
      l=i+1      
      g=q(i)     
      if (l.gt.n) go to 4101   
      do 4100 j=l,n     
 4100 u(i,j)=0.0 
 4101 if (g.eq. 0.0) go to  4500      
      h=u(i,i)*g 
      if (l.gt.n) go to 4401   
      do 4400 j=l,n     
      s=0.0      
      do 4200 k=l,m     
 4200 s=u(k,i)*u(k,j) + s      
      f=s/h      
      do 4300 k=i,m     
 4300 u(k,j)=u(k,j) + f*u(k,i) 
 4400 continue   
 4401 continue   
      do 4550 j=i,m     
 4550 u(j,i)=u(j,i)/g   
      go to 4700 
 4500 continue   
      do 4600 j=i,m     
 4600 u(j,i)=0.0 
 4700 u(i,i)=u(i,i) + 1.0      
c  
c  diagonalization of bi-diagonal form
 4701 eps=eps*x  
      do 9000 kback=1,n 
      k=n+1-kback
c  test f-splitting     
 5000 continue   
      do 5100 lback=1,k 
      l=k+1-lback
      if (abs(e(l)).le. eps) go to 6500      
      if (abs(q(l-1)) .le. eps) go to 6000   
 5100 continue   
c  cancellation of e(l), if l.gt. 1   
 6000 c=0.0      
      s=1 
      l1=l - 1   
      do 6200 i=l,k     
      f=s*e(i)   
      e(i)=c*e(i) 
      if (abs(f) .le. eps) go to 6500 
      g=q(i)     
      q(i)=sqrt(f*f + g*g)     
      h=q(i)     
      c=g/h      
      s=-f/h     
      go to (6050,6050,6200,6200),index      
 6050 continue   
      do 6100 j=1,m     
      y=u(j,l1)  
      z=u(j,i)   
      u(j,l1)=y*c + z*s 
      u(j,i)=-y*s + z*c 
 6100 continue   
 6200 continue   
c  test f-convergence   
 6500 z=q(k)     
      if (l .eq. k) go to  8000
c  shift from bottom 2 x 2 minor      
      x=q(l)     
      y=q(k-1)   
      g=e(k-1)   
      h=e(k)     
      f=((y-z)*(y+z) + (g-h)*(g+h))/(2.0*h*y)
      g=sqrt(f*f + 1.0) 
      f=((x-z)*(x+z) + h*(y/(f + sign(g,f))-h))/x   
c  next q-r transformation     
      c=1.0      
      s=1.0      
      lplus=l + 1
      do 7500 i=lplus,k 
      g=e(i)     
      y=q(i)     
      h=s*g      
      g=c*g      
      z=sqrt(f*f + h*h) 
      e(i-1)=z   
      c=f/z      
      s=h/z      
      f=x*c + g*s
      g=-x*s + g*c      
      h=y*s      
      y=y*c      
      go to (7100,7201,7100,7201),index      
 7100 do 7200 j=1,n     
      x=v(j,i-1) 
      z=v(j,i)   
      v(j,i-1)=x*c + z*s
      v(j,i)=-x*s + z*c 
 7200 continue   
 7201 z=sqrt(f*f + h*h) 
      q(i-1)=z   
      c=f/z      
      s=h/z      
      f=c*g + s*y
      x=-s*g + c*y      
      go to (7300,7300,7500,7500),index      
 7300 do 7400 j=1,m     
      y=u(j,i-1) 
      z=u(j,i)   
      u(j,i-1)=y*c + z*s
      u(j,i)=-y*s + z*c 
 7400 continue   
 7500 continue   
      e(l)=0.0   
      e(k)=f     
      q(k)=x     
      go to  5000
c  convergence   
 8000 if (z .ge. 0.0) go to 9000      
c  q is made non-negative      
      q(k)=-z    
      go to (8100,9000,8100,9000),index      
 8100 do 8200 j=1,n     
 8200 v(j,k)=-v(j,k)    
 9000 continue   
      return     
      end     

