#include "niostf.h"
#include <errno.h> /* For errno: Walter Szeliga*/
#define PM 0644		/* Mode r&w for owner, r for everyone else */
static int jfile[MAXOPN];
static int maxno = 0, nopen = 0;
static int fd[MAXOPN];
extern long lbt[];	/* last byte handled (zeroed when file returned) */

getfd(number,iact,iloc)
/* For a filenumber number, returns the fd attached to it. Numbers of files
   previously accessed are contained in the jfile table. If a file has
   not been previously accessed (number not in the jfiletable), it is
   opened (if iact<=0) or created (if iact>0) and placed in the table.
   The location of the file in the jfile table is returned by iloc,
   which is thus available for indexing purposes in the calling program.
   An error message is printed and the program aborted if:
        1. A file to be opened cannot be.
        2. A file to be created cannot be.
        3. More than MAXOPN files are to be accessed at once (this is
           the number of spaces in the jfile table; files may be removed
           from the table using subroutine return).

   Original version written at CIRES by D. Agnew March 1980
   Revised by D. Agnew November 1980
   and converted to use file numbers and run under 4.2 BSD
   by D. Agnew July 1985 */
int number;
int iact,*iloc;
{
   extern int errno;
   struct stat finfo;
   int i,ity,ier;
   char filnam[NAMLEN];
   for (i=0; i<maxno; i++){
      if(number==jfile[i]){
         *iloc = i;
         return(fd[i]);
      }
   }
/* file is not in jfile table. Check if there is room for another */
/* (note that nopen is the number of files currently open, while */
/* maxno points just after the last filled place in the */
/* jfile table), and if so, open or create it, and put it in the first */
/* empty place */
   if (nopen==MAXOPN-1){
      fprintf(stderr,"No more than %d files may be open at once.\n",MAXOPN);
      fprintf(stderr,"File %d is the last allowed without a call to return.\n",number);
   }
   if (nopen==MAXOPN){
      fprintf(stderr,"No more than %d files may be open at once.\n",MAXOPN);
      fprintf(stderr,"File %d is over this limit: program aborts.\n",number);
      exit(15);
   }
   munman(number,filnam);
   if(iact<=0){				/*file to be read or overwritten*/
      ier = stat(filnam,&finfo);	/*so check status of it*/
      if(ier==-1){
         fprintf(stderr,"Status of file %d unknown.\n",number);
         if((errno==2 || errno==20) && abs(number)<10000)
            fprintf(stderr,"File does not exist -");
         if((errno==2 || errno==20) && abs(number)>10000)
            fprintf(stderr,"File does not exist or pathname bad -");
         if(errno==13)
            fprintf(stderr,"Not permitted to examine it -");
         fprintf(stderr," program aborts...\n");
         exit(15);
      }
      if(finfo.st_uid==getuid()){	/*see if the file has the same uid as the process*/
         ity=open(filnam,O_RDWR,PM);		/*if so, open for read and write, since we may want both*/
         if(ity==-1){		/*error off on failed open*/
            fprintf(stderr,"File number %d could not be opened.\n",number);
            fprintf(stderr,"Program aborts...\n");
            exit(15);
         }
      }
      else{					/*if not*/
         ity=open(filnam,O_RDONLY,PM);		/*open for read only*/
         if(ity==-1){		/*error off on failed open*/
            fprintf(stderr,"File number %d could not be opened.\n",number);
            fprintf(stderr,"Read permission not available - program aborts...\n");
            exit(15);
         }
      }
      lseek(ity,0L,0);		/*rewind to start of file*/
   }
   else{		/*create file, truncating to zero in case an old one exists*/
      ity=open(filnam,O_RDWR|O_CREAT|O_TRUNC,PM);
      if(ity==-1){
         fprintf(stderr,"Could not create file %d. Program aborts.\n",number);
         exit(15);
      }
      close(ity);			/*have to return and then open file to*/
      ity=open(filnam,O_RDWR,PM);		/*be able to read it in same program call*/
      lseek(ity,0L,0);
   }
   nopen++;
   for(i=0; i<maxno; i++){
      if(jfile[i]==0)
         break;
   }
/* if we fall off the end of this loop, put new filenumber at the end, */
/* otherwise it goes within the table, in a space made blank using  */
/* return_ */
   jfile[i]=number;
   fd[i] = ity;
   if(i==maxno)
      maxno++;
   *iloc = i;
   return(ity);
}

retnio_(number)
/* Fortran-callable routine for closing the file number, and removing it */
/* from the jfile table */
int *number;
{
   int i;
   for(i=0; i<maxno; i++){
      if(*number==jfile[i]){
         jfile[i]=0;
         close(fd[i]);
         lbt[i]=0L;
         fd[i]= -1;
         nopen--;
         if(i==maxno-1)
            maxno--;
      }
   }
   return;
}
