#include "niostf.h"
#define BUFS   512	/*size (in words) of integer buffers (short & long)*/
#define LBUFBY 2048 	/*size (in bytes) of integer buffer (long)*/
#define SBUFBY 1024	/*size (in bytes) of integer buffer (short)*/
#include <sys/uio.h>
#include <unistd.h>
#include <errno.h> /* Needed for errno: Walter Szeliga*/
#include <string.h> /* Needed for strerror: Walter Szeliga */
long lbt[MAXOPN];	/*array holding position last reached, for each channel*/

realio_(list,rlx,rhx,numx,nrecx,dum1)
/* Fortran callable routine for reading or writing diskfiles, from */
/* data in array list to positions rlx through rhx in file number numx */
/* For nrec 0 read from file; positive, create file and write to it; */
/* negative, write on file already in existence. (After first call in */
/* a program, nrec>0 is equivalent to nrec<0, unless a call has been */
/* made to return). Files may be short or long integer or real; */
/* integer files are read-only for realio; they may be written or */
/* read with intio (below) */
/*   calls routines getfd, munman, filnumchk */
long *numx;		/*file number*/
long *rlx,*rhx;		/*positions in file to read or write*/
float list[];		/*array of numbers to read or write*/
int *nrecx;		/*variable giving course of action*/
long dum1;		/*UNIX dummy variable*/
{
/*   extern int errno;  */
/*   extern int sys_nerr;  */
/*   extern char *sys_errlist[]; */
   int ity,iloc,lun,nb,nbr,nrec,i,j,nbuf,nbl,num;
   short ibuf[BUFS];	/*buffer to hold integers when reading an integer file*/
   long  lbuf[BUFS];	/*buffer to hold integers when reading an integer file*/
   long lastp,rl,rh,sp,offset,lep;
   long ltmp;
   
   num = *numx;
   rl = *rlx;
   rh = *rhx;
   nrec = *nrecx;
   if(num==0)			/*exit on filenumber of zero*/
      exit(15);
   ity = filnumchk(num);	/*check type of filename (-1 real, 1 or 2 integer, 0 illegal)*/
   if(ity==0){
      fprintf(stderr,"File number %d is illegal. Realio aborts.\n",num);
      exit(15);
   }
   if(ity>0 && nrec!=0){	/*cannot write to integer file*/
      fprintf(stderr,"File %d is integer, and realio cannot write to it. Program aborts.\n",num);
      exit(15);
   }
   lun = getfd(num,nrec,&iloc);   /* get file descriptor; iloc is position of name in array lbt*/
   lep = lbt[iloc];		/*last byte read or written*/
   if(ity==1){
      sp = 2*(rl-1) + OFFSET;	/*start byte in short integer file*/
      nb = 2*(rh-rl+1);		/*number of bytes to read*/
      nbuf = (nb-1)/SBUFBY;		/*number of buffer loads*/
      nbl = nb - SBUFBY*nbuf;		/*size of last load*/
   }
   else if(ity==2){
      sp = 4*(rl-1) + OFFSET;	/*start byte in long integer file*/
      nb = 4*(rh-rl+1);		/*number of bytes to read*/
      nbuf = (nb-1)/LBUFBY;		/*number of buffer loads*/
      nbl = nb - LBUFBY*nbuf;	/*size of last load*/
   }
   else{
      sp = 4*(rl-1) + OFFSET;	/*start byte in real file*/
      nb = 4*(rh-rl+1);		/*number of bytes to read*/
   }
   offset = sp-lep;		/*number of bytes to move from last place in file*/
   if(offset< -lep/2)
      ltmp=lseek(lun,sp,SEEK_SET);		/*seek from start of file, */
   else
      lseek(lun,offset,SEEK_CUR);		/*or last position, whichever is closer */
   if(nrec!=0){
      nbr = write(lun,list,nb);		/*write reals out*/
      if(nb!=nbr){
         lastp = (sp-OFFSET+nbr)/4;
         fprintf(stderr,"Error in writing to real file %d\n",num);
         fprintf(stderr,"Terms to be written were %ld through %ld, or %d bytes.\n",rl,rh,nb);
         if(nbr>0){
            fprintf(stderr,"%d bytes actually written. Last point %d\n",nbr,lastp);
            fprintf(stderr,"Program aborts...\n");
         }
         else{
            fprintf(stderr,"Bad write - %d bytes written. Program aborts...\n",nbr);
         }
         exit(15);
      }
   }
   else if(ity<0){
      nbr = read(lun,list,nb);		/*read reals in */
      if(nb!=nbr){
         lastp = (sp-OFFSET+nbr)/4;
         ioreder(num,rl,rh,nbr,lastp);
         exit(15);
      }
   }
   else if (ity==1) {
      for(i=0; i<=nbuf; i++){		/*read short integers in*/
         if(i!=nbuf){
            nbr = read(lun,ibuf,SBUFBY);	/*read full buffer*/
            if(nbr!=SBUFBY){
               lastp = (sp-OFFSET+nbr+SBUFBY*i)/2;
               ioreder(num,rl,rh,nbr,lastp);
               exit(15);
            }
            for(j=0;j<=BUFS-1;j++)			/*transfer data*/
               list[BUFS*i+j] = ibuf[j];
         }
         else{
            nbr = read(lun,ibuf,nbl);	/*read last (partial) buffer*/
            if(nbr!=nbl){
               lastp = (sp-OFFSET+nbr+SBUFBY*nbuf)/2;
               ioreder(num,rl,rh,nbr,lastp);
               exit(15);
            }
            for(j=0; j<nbl/2; j++)
               list[BUFS*nbuf+j] = ibuf[j];	/*transfer data*/
         }
      }
   }
   else if (ity==2) {
      for(i=0; i<=nbuf; i++){		/*read long integers in*/
         if(i!=nbuf){
            nbr = read(lun,lbuf,LBUFBY);	/*read full buffer*/
            if(nbr!=LBUFBY){
               lastp = (sp-OFFSET+nbr+LBUFBY*i)/4;
               ioreder(num,rl,rh,nbr,lastp);
               exit(15);
            }
            for(j=0;j<=BUFS-1;j++)			/*transfer data*/
               list[BUFS*i+j] = lbuf[j];
         }
         else{
            nbr = read(lun,lbuf,nbl);	/*read last (partial) buffer*/
            if(nbr!=nbl){
               lastp = (sp-OFFSET+nbr+LBUFBY*nbuf)/4;
               ioreder(num,rl,rh,nbr,lastp);
               exit(15);
            }
            for(j=0; j<nbl/4; j++)
               list[BUFS*nbuf+j] = lbuf[j];	/*transfer data*/
         }
      }
   }
   lbt[iloc] = sp+nb;		/*store place stopped at for later use*/
   return;
}

intio_(list,rlx,rhx,numx,nrecx,dum1)
/* Fortran callable routine for reading and writing integers on disk */
/* arguments as for realio (above), except that list is integer*4 rather */
/* than real, and the file number must be positive (for 100-999 the */
/* file written will be short integer, for 1100-1999 long integer) */
long *numx;
long list[];
long *rlx,*rhx;
int *nrecx;
long dum1;
{
   int ity,iloc,lun,nb,nbr,nrec,i,j,nbuf,nbl,num;
   long lastp,rl,rh,sp,offset,lep;
   short ibuf[BUFS];	/*buffer for reading a short integer file*/
   
   num = *numx;
   rl = *rlx;
   rh = *rhx;
   nrec = *nrecx;
   if(num==0)
      exit(15);
   ity = filnumchk(num);
   if(ity==0){
      fprintf(stderr,"File number %d is illegal. Intio aborts.\n",num);
      exit(15);
   }
   if(ity<0){	/*Cannot read or write to real file*/
      fprintf(stderr,"File %d is real, and intio cannot access it. Program aborts.\n",num);
      exit(15);
   }
   lun = getfd(num,nrec,&iloc);
   /* find place to start, and seek to there, either from start of file */
   /* or last place touched within it */
   lep = lbt[iloc];
   if(ity==1){
      sp = 2*(rl-1) + OFFSET;	/*start byte in short integer file*/
      nb = 2*(rh-rl+1);		/*number of bytes to read*/
      nbuf = (nb-1)/SBUFBY;		/*number of buffer loads*/
      nbl = nb - SBUFBY*nbuf;		/*size of last load*/
   }
   else{
      sp = 4*(rl-1) + OFFSET;	/*start byte in long integer file*/
      nb = 4*(rh-rl+1);		/*number of bytes to read*/
   }
   offset = sp-lep;
   if(offset< -lep/2)
      lseek(lun,sp,SEEK_SET);
   else
      lseek(lun,offset,SEEK_CUR);
   if(nrec!=0 && ity==2){
      nbr = write(lun,list,nb);		/*write long integers*/
      if(nb!=nbr){
         fprintf(stderr,"Error in intio in writing to file %d\n",num);
         fprintf(stderr,"Terms to be written were %ld through %ld, or %d bytes.\n",rl,rh,nb);
         fprintf(stderr,"%d bytes actually written. Program aborts.\n",nbr);
         exit(15);
      }
   }
   else if(nrec==0 && ity==2) {
      nbr = read(lun,list,nb);		/*read long integers*/
      if(nb!=nbr){
         lastp = (sp-OFFSET+nbr)/4;
         ioreder(num,rl,rh,nbr,lastp);
         exit(15);
      }
   }
   else if (nrec==0 && ity==1) {
      for(i=0; i<=nbuf; i++){		/*read short integers in*/
         if(i!=nbuf){
            nbr = read(lun,ibuf,SBUFBY);	/*read full buffer*/
            if(nbr!=SBUFBY){
               lastp = (sp-OFFSET+nbr+SBUFBY*i)/2;
               ioreder(num,rl,rh,nbr,lastp);
               exit(15);
            }
            for(j=0;j<=BUFS-1;j++)		/*transfer data*/
               list[BUFS*i+j] = ibuf[j];
         }
         else{
            nbr = read(lun,ibuf,nbl);	/*read last (partial) buffer*/
            if(nbr!=nbl){
               lastp = (sp-OFFSET+nbr+SBUFBY*nbuf)/2;
               ioreder(num,rl,rh,nbr,lastp);
               exit(15);
            }
            for(j=0; j<nbl/2; j++)
               list[BUFS*nbuf+j] = ibuf[j];		/*transfer data*/
         }
      }
   }
   else if (nrec!=0 && ity==1) {
      for(i=0; i<=nbuf; i++){		/*write short integers out*/
         if(i!=nbuf){
            for(j=0;j<=BUFS-1;j++)		/*transfer data*/
               ibuf[j] = list[BUFS*i+j];
            nbr = write(lun,ibuf,SBUFBY);	/*write full buffer*/
            if(nbr!=SBUFBY){
               fprintf(stderr,"Error in intio in writing to file %d\n",num);
               fprintf(stderr,"Terms to be written were %ld through %ld.\n",rl,rh);
               fprintf(stderr,"Buffer load %d should have contained %d bytes\n",i+1,SBUFBY);
               fprintf(stderr,"%d bytes actually written. Program aborts.\n",nbr);
               exit(15);
            }
         }
         else{
            for(j=0; j<nbl/2; j++)
               ibuf[j] = list[BUFS*nbuf+j];		/*transfer data*/
            nbr = write(lun,ibuf,nbl);	/*write last (partial) buffer*/
            if(nbr!=nbl){
               fprintf(stderr,"Error in intio in writing to file %d\n",num);
               fprintf(stderr,"Terms to be written were %ld through %ld.\n",rl,rh);
               fprintf(stderr,"Final buffer load (no. %d)",nbuf+1);
               fprintf(stderr," should have contained %d bytes\n",nbl);
               fprintf(stderr,"%d bytes actually written. Program aborts.\n",nbr);
               exit(15);
            }
         }
      }
   }
   lbt[iloc] = sp+nb;		/*store place last stopped at*/
   return;
}

ioreder(num,rl,rh,nbr,lastp)
/*  Prints out error message on read; if >0 bytes are read, gives the  */
/*  probable last term of the file, otherwise prints out most likely   */
/*  source of error */
int num,nbr;
long rl,rh,lastp;
{
   extern int errno;
/*   extern char *sys_errlist[]; */
   fprintf(stderr,"Error in reading from file %d\n",num);
   fprintf(stderr,"Terms to be read were %ld through %ld.\n",rl,rh);
   if(nbr>0){
      fprintf(stderr,"Partial read - last point read was %d\n",lastp);
      fprintf(stderr,"Program aborts...\n");
   }
   else if (nbr==0)
      fprintf(stderr,"Attempt to read after EOF. Program aborts...\n");
   else {
      fprintf(stderr,"Bad read: ( %s ), program aborts...\n",strerror(errno));
/* sys_errlist is deprecated      fprintf(stderr,"Bad read: ( %s ), program aborts...\n",sys_errlist[errno]);*/
   }
}

shrtio_(list,rlx,rhx,numx,nrecx,dum1)
/* Fortran callable routine for reading and writing short integers on disk */
/* arguments as for realio (above), except that list is integer*2 rather */
/* than real, and the file number must be positive and between 100 and 999*/
long *numx;
short list[];
long *rlx,*rhx;
int *nrecx;
long dum1;
{
   int ity,iloc,lun,nb,nbr,nrec,num;
   long lastp,rl,rh,sp,offset,lep;
   
   num = *numx;
   rl = *rlx;
   rh = *rhx;
   nrec = *nrecx;
   if(num==0)
      exit(15);
   ity = filnumchk(num);
   if(ity==0){
      fprintf(stderr,"File number %d is illegal. Shrtio aborts.\n",num);
      exit(15);
   }
   if(ity<0){	/*Cannot read or write to real file*/
      fprintf(stderr,"File %d is real, and shrtio cannot access it. Program aborts.\n",num);
      exit(15);
   }
   if(ity==2){	/*Cannot read or write to long integer file*/
      fprintf(stderr,"File %d is long, and shrtio cannot access it. Program aborts.\n",num);
      exit(15);
   }
   lun = getfd(num,nrec,&iloc);
   /* find place to start, and seek to there, either from start of file */
   /* or last place touched within it */
   lep = lbt[iloc];
   sp = 2*(rl-1) + OFFSET;	/*start byte in short integer file*/
   nb = 2*(rh-rl+1);		/*number of bytes to read*/
   offset = sp-lep;
   if(offset< -lep/2)
      lseek(lun,sp,SEEK_SET);
   else
      lseek(lun,offset,SEEK_CUR);
   if(nrec!=0){
      nbr = write(lun,list,nb);		/*write short integers*/
      if(nb!=nbr){
         fprintf(stderr,"Error in shrtio in writing to file %d\n",num);
         fprintf(stderr,"Terms to be written were %ld through %ld, or %d bytes.\n",rl,rh,nb);
         fprintf(stderr,"%d bytes actually written. Program aborts.\n",nbr);
         exit(15);
      }
   }
   else {
      nbr = read(lun,list,nb);		/*read short integers*/
      if(nb!=nbr){
         lastp = (sp-OFFSET+nbr)/2;
         ioreder(num,rl,rh,nbr,lastp);
         exit(15);
      }
   }
   lbt[iloc] = sp+nb;		/*store place stopped at for later use*/
   return;
}
