      subroutine findpz(np,nz,poles,zeroes,coeff)
c  given a transfer function in the form of the ratio of two
c  polynomials (through the common block polys), this subroutine
c  finds the poles and zeroes and the leading coefficient such
c  that the transfer function may be written instead as
c
c    coeff*(prod[s+zero(i)]/prod[s+pole(k)])
c
c  where the products are for i=1,nz (for zeroes) and k=1,np
c  (for poles). the subroutine will return the correct values
c  if there are zeros at zero frequency but not if there are poles.
c
c     $$$calls rooter
c
      complex poles,zeroes,tp,bp,rlsw
      double precision cnum,cden
      dimension poles(*),zeroes(*)
      common/polys/nnum,nden,cnum(100),cden(100)
      rlsw(tp) = cmplx(-real(tp),aimag(tp))
      if(nnum.gt.0) call rooter(nnum,cnum,zeroes,iok)
      nz = nnum
      if(nden.gt.0) call rooter(nden,cden,poles,iok)
      np = nden
      tp = (1.,0.)
      bp = tp
c  the poles and zeroes found for a stable system will have negative real
c  parts when found by the algorithm used above (see the comments to rooter).
c  the sign of the real part needs to be changed to agree with the convention
c  used here.
      if(nz.eq.0) go to 3
      do 1 i=1,nz
      zeroes(i) = rlsw(zeroes(i))
 1    if(zeroes(i).ne.(0.,0.)) tp=tp*zeroes(i)
 3    if(np.eq.0) go to 7
      do 5 i=1,np
      poles(i) = rlsw(poles(i))
 5    bp=bp*poles(i)
c scan to find first nonzero numerator coefficient
 7    do 9 i=1,nnum+1
 9    if(cnum(nnum-i+2).ne.0.) coeff=cnum(nnum-i+2)
      coeff = coeff/cden(1)
      coeff = coeff/(real(tp)/real(bp))
      return
      end

