      subroutine pzresp(kind,fw,fn,np,pole,nz,zero,cf,res,gd)
c  computes the response at a given frequency when the total response
c  is given by poles and zeros
c
c    input variables:
c       kind   filter type; if a is analog (s-plane), if d, is digital
c                (z-plane, with z-transform in + powers of z)
c              special case: if n, assumed analog, no complex conjugates
c       fw     frequency at which response is wanted (hz)
c       fn     for kind = a, the normalizing frequency (hz)
c                for kind = d, the Nyquist frequency (hz)
c       np     number of poles in list
c       pole   list of poles (complex). real part of poles must be
c                 positive. if a pole is not purely imaginary (or zero)
c                 the complex conjugate pole will automatically be assumed
c                 to be present. the frequencies of the poles and zeroes
c                 are given relative to the normalizing frequency.
c       nz     number of zeroes in list.
c       zero   list of zeroes (complex). as with poles, real part must be
c                 positive, and complex conjugates are assumed for complex
c                 values.
c       cf     overall amplitude coefficient
c
c    output variables:
c       res    the response (complex)
c       gd     the group delay, in seconds (- for delay)
c
c calls only system routines
c
      character*1 kind
      complex res,pole,zero,s
      double precision a,a2,b,c,c2
      dimension pole(1),zero(1)
      res = cmplx(cf,0.)
      if(kind.eq.'a'.or.kind.eq.'n') then
         f = fw/fn
         s = cmplx(0.,f)
         s2 = f*f
      elseif(kind.eq.'d') then
         f = 3.14159*fw/fn
         s = cmplx(cos(f),-sin(f))
         c = dcos(dble(f))
         c2 = dcos(2.0d0*f)
      endif
      if(nz.ne.0) then
         do 1 i=1,nz
         if(aimag(zero(i)).eq.0) res=res*(s+zero(i))
         if(aimag(zero(i)).ne.0.and.kind.ne.'n')
     1                res=res*(s+zero(i))*(s+conjg(zero(i)))
         if(aimag(zero(i)).ne.0.and.kind.eq.'n')
     1                res=res*(s+zero(i))
 1       continue
      endif
      if(np.ne.0) then
         do 3 i=1,np
         if(aimag(pole(i)).eq.0) res=res/(s+pole(i))
         if(aimag(pole(i)).ne.0.and.kind.ne.'n')
     1                res=res/((s+pole(i))*(s+conjg(pole(i))))
         if(aimag(pole(i)).ne.0.and.kind.eq.'n')
     1                res=res/(s+pole(i))
 3       continue
      endif
c
c   compute group delay analytically
c
      gd = 0.
      if(kind.eq.'a') then
         if(nz.ne.0) then
            do 5 i = 1,nz
            a = real(zero(i))
            a2 = a*a
            b = (aimag(zero(i)))**2
            if(a.eq.0.and.b.eq.0.and.f.eq.0) go to 5
            if(b.eq.0) gd = gd + a/(s2+a2)
            if(b.ne.0) gd = gd + (2*a*(s2+a2+b))/
     1        ((a2+b)**2 + 2*s2*(a2-b) + s2**2)
 5          continue
         endif
         if(np.ne.0) then
            do 7 i = 1,np
            a = real(pole(i))
            a2 = a*a
            b = (aimag(pole(i)))**2
            if(b.eq.0) gd = gd - a/(s2+a2)
            if(b.ne.0) gd = gd - (2*a*(s2+a2+b))/
     1        ((a2+b)**2 + 2*s2*(a2-b) + s2**2)
 7          continue
         endif
         gd = gd/(6.2831853*fn)
      elseif(kind.eq.'d') then
         if(nz.ne.0) then
            do 9 i = 1,nz
            a = real(zero(i))
            a2 = a*a
            b = cabs(zero(i))
            b = b*b
            if(aimag(zero(i)).eq.0) gd = gd - (1+a*c)/(1+a*a+2*a*c)
            if(aimag(zero(i)).ne.0) gd = gd -
     1                        (2 + 4*a2 +6*a*c + 2*b*(c2 + a*c))/
     2                        (cabs((s+zero(i))*(s+conjg(zero(i))))**2)
 9          continue
         endif
         if(np.ne.0) then
            do 11 i = 1,nz
            a = real(pole(i))
            a2 = a*a
            b = cabs(pole(i))
            b = b*b
            if(aimag(pole(i)).eq.0) gd = gd + (1+a*c)/(1+a*a+2*a*c)
            if(aimag(pole(i)).ne.0) gd = gd +
     1                        (2 + 4*a2 +6*a*c + 2*b*(c2 + a*c))/
     2                        (cabs((s+pole(i))*(s+conjg(pole(i))))**2)
 11         continue
         endif
         gd = gd/(2.*fn)
      endif
      return
      end
