      subroutine rooter(n, a, z, iok)
c$$$$calls guerre,qfact
c  Attempts the location of all the roots, real and complex, of
c     a(1) + a(2)*x + a(3)*x**2 + ... + a(n+1)*x**n
c  the coefficients  a  are all real.
c  The roots are returned in the complex array  z,  and
c  if a particular root is real, the imaginary part is exactly zero.
c  iok  is  1  if all roots are found,  0  if n=0, -1 if  a(n+1)=0,
c   -2  if routine fails to find all the roots.
c  The method uses Laguerre's method, alternating if necessary with
c  Bairstow's for complex roots.
      implicit double precision (a-h,o-z)
      complex z(n)
      common /saver/ g,dsq,p
      dimension a(*), c(250)
      data maxits/10/,iout/6/
c
      iok=1
      if (n.le.0)        iok=-1
      if (a(n+1).eq.0.0) iok=0
      if (iok.ne.1)      return
c  Extract all roots that are zero.
      do 1100 izero=1,n
      if (a(izero).ne. 0.0) go to 1200
 1100 z(izero)=0.0
      return
c  Copy remaining polynomial into working array  c.
 1200 m=n-izero+1
      m1=izero
      do 1300 i=1,m
      j=i+izero-1
 1300 c(i)=a(j)
      c(m+1)=a(n+1)
c
 2000 mdeg=m
      do 2800 ir=1,m
      if (mdeg.le.0) return
      izero=n-mdeg+1
c  Find a bound on the size of the root.
      bound=abs(c(1))
c     do 2100 i=2,mdeg
c2100 bound=abs(c(i)) + bound
c     bound=bound/abs(c(mdeg+1))
c  Starting on real axis with Laguerre's method, work inwards.  If
c  Lagurre's method indicates a complex root try Bairstow at that point.
      do 2200 its=1,maxits
      x=bound
      call guerre(-mdeg, c, x, ifound)
c  If a root has been found, polish it using original equation.
      if (ifound.eq.1) call guerre(m, a(m1), x, ifound)
      if (ifound.eq.1) go to 2500
c  Get termination data from /saver/ and find a start for qfact=Bairstow
      z(izero)=x - float(mdeg)/cmplx(g,sqrt(-dsq)/p)
      p=2.0*real(z(izero))
      el=-(real(z(izero))**2 + aimag(z(izero))**2)
      call qfact(-mdeg, c, p, el, ididit)
c  Polish the factors with a solution on the original coeffs.
      if (ididit.eq.1) call qfact(m, a(m1), p, el, ididit)
      if (ididit.eq.1) go to 2400
      bound=-0.6*bound
 2200 continue
c  Ran out of iterations - print message and surrender.
      ngood=izero - 1
      write(iout,220) ngood
 220  format(' rooter has failed to find all the roots'/
     +' the first',i3,'  are correct, however')
      iok=-2
      return
c  Compute complex roots from quadratic factor.
 2400 z(izero)=(p+sign(1.0,p)*csqrt(cmplx(p*p+4.0*el,0.0)))/2.0
      z(izero+1)=-el/z(izero)
      mdeg=mdeg-2
      go to 2800
 2500 z(izero)=x
      mdeg=mdeg-1
 2800 continue
      return
      end                                                               rooter
c_______________________________________________________________________
      subroutine qfact (n, a, r, s, ididit)
c$$$$$$ calls no other routines
c  Factorizes the polynomial
c     a(1) + a(2)*x + a(3)*x**2+ ... +  a(n+1)*x**n
c  into   q*(z**2 - r*z -s) ,  where q is a polynomial of degree n-2.
c  ididit=1  if successful, 0  if iterations fail to converge.
c  Uses Bairstow's method (see Wilkinson, The Algebraic Eigenvalue
c  Problem pp 447-449).
c  Upon entry  r, s  are approximations for starting iteration.
c  If  n  is negative a polynomial of degree  -n  is assumed, and the
c  factor  q  replaces the original polynomial  in  a .
c  Unless the dimension statement below is altered, iabs(n) .le. 24
      implicit double precision (a-h,o-z)
      common /qfact1/ q(250),t(250)
      common /qstart/ rel,it
      dimension a(*)
c      double precision t,q,p,el                                         double
      data tol/1.0d-9/,max/40/ ,dp,dl/2*1.0d-3/                         prime
c
      m=iabs(n)
      ididit=1
      if (m .eq.2)  go to 4000
      last=0
      m2=m + 1
      el=s
      p=r
      do 2500 it=1,max
      if (last.eq.1) go to 3000
c  Divide  current factor into  a.
      q(m+1)=a(m+1)
      q(m)=a(m) + p*q(m+1)
      do 1100 i=2,m
      is=m-i+1
 1100 q(is)=a(is) + p*q(is+1) + el*q(is+2)
      r=q(2)
      s=q(1)-p*q(2)
c  Test accuracy.  If not exact always go round again.
      q1=q(1)
      q2=q(2)
      if (q1.eq.0.0 .and. q2.eq.0.0) go to 3000
      c=q(3)*p
      if(abs(q1).le.tol*abs(a(1)).and.abs(q2).le.tol*(abs(a(2))+abs(c)))
     +  last=1
c  Divide current factor into  q.
      t(m-1)=q(m+1)
      t(m-2)=q(m) + p*t(m-1)
      do 1200 i=4,m
      is=m-i+1
 1200 t(is)=q(is+2) + p*t(is+1) + el*t(is+2)
c  Compute correction to coeffs of quadratic factor.
      em=el*t(2) + p*t(1)
      dd=t(1)*t(1) - em*t(2)
c  Double roots sometimes cause dd to vanish.  This fix may help.
      if (dd.eq.0.0) go to 1300
      dp=(t(2)*q(1) - t(1)*q(2))/dd
      dl=(em*q(2) - t(1)*q(1))/dd
 1300 p=p + dp
      el=el + dl
 2500 continue
c  Iteration has failed to converge.
      ididit=0
      return
c  Factor found successfully.  Copy into single-precision variables.
 3000 r=p
      s=el
      if (n.gt.0) return
c  Over-write old polynomial  a , with reduced one  q .
      do 3100 i=2,m
 3100 a(i-1)=q(i+1)
      return
 4000 r=-a(2)/a(3)
      s=-a(1)/a(3)
      if (n.lt.0) a(1)=a(3)
      return
      end                                                               qfact
c_______________________________________________________________________
      subroutine guerre(nn, c, x, ifound)
c$$$$ calls no other routines
c  Given the real polynomial
c    c(1) + c(2)*x + c(3)*x**2 + ... + c(nn+1)*x**nn
c  and a starting guess  x  performs the Laguerre iterative search for
c  a real root.  If successful,  x  contains answer and  ifound  is 1.
c  If a complex root is indicated, returns with ifound=0.  If the number
c  of iterations needed exceeds a reasonable limit, ifound=-1.
c  If  nn  is negative the degree is taken to be -nn,  and if a root is
c  found, it is divided out and reduced polynomial is returned in  c.
c  See Acton 1970, Numerical Methods That Work, p187.
      implicit double precision (a-h,o-z)
      dimension c(*),t(2)
      common /saver/ g,dsq,ps
c      double precision xx,p,pp,ppp,t(2)
      data tol/1.0d-7/, maxit/20/
c
      ifound=1
      xx=x
      n=iabs(nn)
      en=n
c  Apply Laguerre scheme  maxit  times.
      do 1500 its=1,maxit
c  Evaluate polynomial, 1st, 2nd derivative.
      p=c(n+1)
      pp=en*p
      ppp=(en-1.0)*pp
      do 1100 i=1,n
      j=n-i+1
      p=xx*p + c(j)
      if (j.gt.1) pp=xx*pp + (j-1.0)*c(j)
      if (j.gt.2) ppp=xx*ppp + (j-1.0)*(j-2.0)*c(j)
 1100 continue
c  Solution is exact, so return at once.
      if (p.eq.0.0) go to 3000
      g=pp/p
      dsq=(en-1.0)*((en-1.0)*pp**2 - en*ppp*p)
      ps=p
      if (dsq .lt. 0.0) go to 2000
      a=en/(g + sign(sqrt(dsq)/ps,g))
      xx=xx-a
      x=xx
      if (abs(a).le. tol*abs(x)) go to 3000
 1500 continue
c  Ran out of iterations.  Flag and return.
      ifound=0
c  Complex root indicated.  Flag and return.
 2000 ifound=ifound - 1
      return
c  If requested (nn.lt.0) divide out the root, reducing degree of  c  by 1
 3000 if (nn.ge.-1) return
      j=1
      t(j)=c(n)
      c(n)=c(n+1)
      do 3100 i=2,n
      k=n-i+1
      t(3-j)=c(k)
      c(k)=t(j) + xx*c(k+1)
 3100 j=3-j
      return
      end                                                               guerre
