/*** file listin.c ***/

#define TRUE		1
#define FALSE		0
#define MIN(P1,P2)	(P1 < P2 ? P1 : P2)
#define MAX(P1,P2)	(P1 > P2 ? P1 : P2)

/************************************************************************
 listin(list,np,icp) is a fortran callable function that reads in a list
 of integers (into array 'list') and, if np != -1, sorts them into 
 increasing order. if icp <= 0 the terminal is read until a value of 'icp'
 is encountered (icp will not be included in 'list'); if icp > 0 'icp'
 values will be read.
 a range of numbers may be indicated by input of the form 'nnn to mmm';
 all numbers from nnn to mmm inclusive will be put into 'list'.
 the number of values in the list is returned in 'np'.
	this function calls ffinc_
 listin does not return a value.

 coded 8/85 (dh) after PDP 11 fortran version
 modified for Sun by DCA 11/91
 mod to match new ffinc 6/05
*************************************************************************/

listin_(list,np,icp)
int list[],*np,*icp;			/* fortran passes pointers */
{
int n,ic;
int l,l2;
char not;
double ffinc_();
short cont;

n = *np; ic = *icp;

l=0;					/* index into list[] */
cont = TRUE;
while (cont) {				/* loop until all values grabbed */

	list[l] = (int)ffinc_(&not);
	switch (not) {

	case '0': if ((ic <= 0 && list[l] == ic) || 
		   (ic > 0 && l >= ic-1)) cont = FALSE;	/* got all values? */
		  else l++;
		  break;
	
	case 't': l2 = (int)ffinc_(&not);  	/* "t?" is good enough  */
		  l = fillin(l,l2,ic,list); 	/* fill in the range */
		  if ((ic > 0 && l >= ic-1) ||
		     (ic <= 0 && list[l] == ic)) cont=FALSE;
		  else l++;
		  break;
					/* illegal delimiter */
	default:  printf("undecodable character (%c, octal %o)\n",not,(int)not);
		  cont = FALSE;
		  if (l) l--;
		  break;

		  }; /* switch */
	
	}; /* while */

if (!((ic <= 0) && (list[l] == ic))) l++; /* l is now # of terms (NOT index) */
if (n != -1) sort(l,list);		/* sort only if n != -1 */
if (ic <= 0) *np = l;
else *np = MIN(l,ic);

} /* end of listin_() */


/*****************************************************************************
 local function fills in a list of integers of the form 'nnn to mmm'
*****************************************************************************/

int fillin(l,l2,ic,list)
int l,l2,ic,list[];
{
int l1,lb,lt,nt,j;

l--; 
l1 = list[l];
lb = MIN(l1,l2);
lt = MAX(l1,l2);
nt = lt - lb + 1;
if (ic > 0) nt = MIN(ic+1-l,nt);
if (ic <= 0 && lb <= 0 && lt >= 0 && lb <= ic) nt = ic + 1 -lb;

for (j=1; j <= nt; j++) 
	if (l1 == lb) list[l+j-1] = lb+j-1;
	else if (l1 == lt) list[l+j-1] = lt-j+1;

return(l+nt-1);				/* return last spot filled */

} /* end of fillin() */

/*************************************************************************
 shellsort n values in a list into increasing order
**************************************************************************/

/*static sort(n,list)*/
extern sort(n,list)
int n,list[];
{
int ig,l,lb,i,lt,nt;

ig = n;
while (ig > 1) {

	ig = ig/2;
	l = n-ig;
	lb = 1;
	while (lb > 0) {

		lb = 0;
		for (i=0; i < l; i++) {
			nt = i+ig;
			if (list[i] <= list[nt]) continue;
			lt = list[i];
			list[i] = list[nt];
			list[nt] = lt;
			lb++;
			};

		};
	
	};
return;

} /* end of sort() */

/*** end of file listinc.c ***/


