      subroutine elastd(grav,tilt,strain,ntw)
      save gdc,tnsdc,etdc,eldc,potdc,re,del,dim
      real k,l
      dimension tilt(2),strain(3),ntw(3),amrat(2),rbor(2),pa(2),p(3),
     1 pp(3),ppp(3),e(3),rkr(3),tltcor(2),del(3),dim(3)
c  computes the earth tides on an elastic earth, given the solar and lunar
c  positions and the place of observation.  degree 2 is used for the solar
c  tides, 2 through 4 for the lunar tides. the results are returned in
c  grav, tilt, and strain. ntw(1) gives the number of gravity tides
c  ntw(2) the number of tilt tides, ntw(3) the number of strain tides.
c  as the dimensioning shows, at most one gravity tide, two tilts, and
c  three strains may be computed.  if ntw(1) = -1, the program will
c  put the equilibrium potential height for a rigid spherical earth in
c  grav.  the units are m/s**2, radians, extension, and meters.
c  the sign convention is positive for upward potential
c  height, a decrease in g, a tilt in the azimuth given, and
c  extensional strain.
c  that part of the tidal signal which comes from the permanent
c  deformation is subtracted out, using the coefficient of .31455 m
c  found by cartwright and edden for the dc tide.
c
c   based (very closely) on j. berger earth tide program.
c   converted to subroutine by d agnew nov 1979.
c   computation of the potential and subtraction of dc tides added
c   by d agnew jan 1980.
c
c  common block with observatory information
      common/obs/cth,sth,clg,slg,dif,radn,gl
c  common block with love numbers
      common/love/h(3),k(3),l(3)
c  common block with strainmeter and tiltmeter azimuths
      common/azimut/azt(3,2),azs(3,3)
c  common block with lunar and solar colat and long, lunar sine parallax,
c  and solar distance
      common/bpos/coor(8),par,sdist
c  data for mean parallaxes, a times m over m(earth), equatorial radius.
      data rbor/1.65925e-2,4.2635e-5/,amrat/78452.,
     1  2.1237e12/,a/6.37816e6/
c      data rbor/1.6592496e-2,4.2635233e-5/,amrat/78451.25,
c     1  2.1235762e12/,a/6.37814e6/
      data g1/9.79828/,g2/9.82022/,ppp(1)/3./,iflag/0/
c  on first call compute factors for gravity and tilt, and dc tides
c  at the given latitude.
      if(iflag.eq.1) go to 3
      iflag=1
      do 1 i = 1,3
      del(i) = 1. + (2./(i+1.))*h(i) - ((i+2.)/(i+1.))*k(i)
 1    dim(i) = 1. + k(i) - h(i)
c  dc gravity tide is also known as the honkasalo correction
c  **note that the love numbers for an elastic earth are used
c    in computing the dc tide as well
      gdc = -3.0481e-7*(3*cth**2 - 1.)*del(1)*radn
      tnsdc = -9.1445e-7*cth*sth*dim(1)*radn/gl
      etdc = -1.555e-8*(h(1)*(3.*cth**2-1.) - 6.*l(1)*(2.*cth**2 -1.))
      eldc = -1.555e-8*(h(1)*(3.*cth**2-1.) - 6.*l(1)*cth**2)
      potdc = .0992064*(1.-3*cth**2)
      re = 1./(radn*a)
c  zero out arrays
 3    do 4 i = 1,2
      tilt(i) = 0.
      e(i) = 0.
 4    tltcor(i) = 0.
      e(3) = 0.
      grav = 0.
      gnth = 0.
c  compute normalized parallax
      pa(1) = par/3422.45
      pa(2) = 1./sdist
c  in outer loop, ii = 1 for moon, 2 for sun
      do 40 ii = 1,2
      id = 3
      if(ii.eq.2) id = 1
      ir = 4*(ii-1)
c  find cosine of zenith angle, potential constants, legendre polynomials
c  and their derivatives, and derivatives of the cosine of the zenith
c  angle.
      cll = clg*coor(ir+4) + slg*coor(ir+3)
      sll = slg*coor(ir+4) - clg*coor(ir+3)
      cz = coor(ir+2)
      sz = coor(ir+1)
      cu = cth*cz + sth*sz*cll
      xi = rbor(ii)*pa(ii)*radn
      cc = amrat(ii)*rbor(ii)*pa(ii)
      rkr(1) = cc*xi*xi
      rkr(2) = rkr(1)*xi
      rkr(3) = rkr(2)*xi
      p(1) = 0.5*(3*cu*cu - 1.)
      pp(1)=3*cu
      if(ii.eq.2) go to 5
      p(2) = .5*cu*(5.*cu*cu - 3.)
      p(3) = .25*(7.*cu*p(2) - 3.*p(1))
      pp(2) = 1.5*(5.*cu*cu - 1.)
      pp(3) = .25*(7.*p(2) + cu*pp(2)) - 3.*pp(1)
      ppp(2) = 15.*cu
      ppp(3) = 7.5*(7.*cu*cu - 1.)
 5    cut = -sth*cz + cth*sz*cll
      cutt = -cu
      cul = -sth*sz*sll
      cull = -sth*sz*cll
      cutl = -cth*sz*sll
      if(ntw(1).eq.0) go to 15
      do 10 j = 1,id
      if(ntw(1).eq.-1) grav = grav + rkr(j)*p(j)
 10   if(ntw(1).eq.1) grav = grav + del(j)*(j+1)*rkr(j)*p(j)*g1*re
      gnth = gnth - dim(1)*rkr(1)*pp(1)*g1*cut*re
 15   if(ntw(2).eq.0) go to 30
      do 25 kk = 1,ntw(2)
      do 20 j=1,id
 20   tilt(kk)=tilt(kk)-(dim(j)*rkr(j)*pp(j)*re)*(cut*azt(2,kk)
     $ - cul*azt(3,kk)/sth)
 25   tltcor(kk) = tltcor(kk) + del(1)*2.*rkr(1)*p(1)*re
 30   if(ntw(3).eq.0) go to 40
      do 35 j = 1,id
      e(1) = e(1) + rkr(j)*(l(j)*(pp(j)*cut*cth/sth + (ppp(j)*cul*cul
     1 + pp(j)*cull)/(sth*sth)) + h(j)*p(j))
      e(2) = e(2) + rkr(j)*(l(j)*(ppp(j)*cut*cut + pp(j)*cutt) +
     1  h(j)*p(j))
      e(3) = e(3) + 2*(rkr(j)*l(j)*(ppp(j)*cul*cut + pp(j)*cutl -
     1 cth*pp(j)*cul/sth))/sth
  35  continue
  40  continue
c  do ellipticity corrections, convert strains to strainmeter
c  azimuths, and remove dc terms.
      if(ntw(1).eq.1) grav = grav + gnth*dif - gdc
      if(ntw(1).eq.-1) grav = grav - potdc
      do 45 i =1,ntw(2)
 45   tilt(i) = tilt(i)*(g1/gl) -tltcor(i)*dif*azt(2,i) -tnsdc*azt(2,i)
      do 50 i = 1,ntw(3)
 50   strain(i) = (e(1)*azs(3,i)**2 - e(3)*azs(3,i)*azs(2,i)
     $+e(2)*azs(2,i)**2)*re*(g1/g2) -etdc*azs(2,i)**2 -eldc*azs(3,i)**2
      return
      end

