      subroutine ephem(t)
      double precision t
c   t is ephemeris time in julian centuries from 12 hr 0 jan 1900
c  (in ordinary civil reckoning this is greenwich noon on 31 december
c  1899). if the difference between ephemeris and unversal time is
c  not put in (see common tdiff below), t should be in universal time
c  which is (nearly) the time ordinarily kept by clocks.
c   computes positions of the sun and moon at time t, returning results
c  in common block bpos. the solar ephemeris uses the mean sun.
c    derived from j. levine's revision (after j. c. harrison)
c  of an earthtide program by j. berger and w. e. farrell, with small
c  alterations by d. c. agnew, partly after m. wimbush. present
c  subroutine version by d. c. agnew.
      double precision ts,hr,pi20,t2,ll,xxx,hs,ps,es,psig,chmp,shmp,
     $ ls,sz,cz,psis,hm,pm,nm,bl,bls,bf,bd,tlatm,tlongm,plx,sinmla,
     $ cosmla,sinmln,cosmln,at1,at2,ram,w,sinw,cosw,sls
c  common block bpos contains, in order:
c    sine and cosine of colatitude of sublunar point
c    sine and cosine of east longitude of sublunar point
c    sine and cosine of colatitude of subsolar point
c    sine and cosine of east longitude of subsolar point
c    the lunar sine parallax in arc seconds
c    the solar distance in astronomical units
      common/bpos/dsz,dcz,dsl,dcl,ssz,scz,ssl,scl,dpar,sdist
c
c  common block containing the difference ephemeris minus
c  universal time, in seconds. if this is not known it should
c  be set to zero, and the argument to the program should be
c  universal rather than ephemeris time.
c
      common/tdiff/etmutc
c  common block containing the instruction on which ephemeris to compute
c  moon =   0  - both sun and moon
c           1  - moon only
c           2  - sun only
      common/sunny/moon
      fcos(xxx) = cos(sngl(xxx))
      fsin(xxx) = sin(sngl(xxx))
      data pi20/62.8318530717958d0/
c   compute universal time in hours
      ts = 876600.d0*t -12.d0 - (etmutc/3600.d0)
      hr = dmod(ts,24.d0)
c   compute obliquity of the ecliptic
      w = .409319747d0 - .0002271107d0*t
      cosw = dcos(w)
      sinw = dsin(w)
      t2 = t*t
      if(moon.eq.1) go to 7
c
c     compute solar constants for given t
c
      hs = 4.881627982482d0 + 628.3319508731d0*t + 0.523598775578d-5*t2
      hs = dmod(dmod(hs,pi20)+pi20,pi20)
      ps = 4.908229466993d0+ 0.03000526416690d0*t + 0.790246300201d-5*t2
      es = 0.01675104d0 - 0.00004180d0*t - 0.000000126d0*t2
      psig = 0.2617993877971d0*(hr-12.) + hs
      chmp = dcos(hs-ps)
      shmp = dsin(hs-ps)
      ls = hs + shmp*es*(2.+2.5*es*chmp)
      sls = dsin(ls)
      cz = sinw*sls
      sz = dsqrt(1.-cz**2)
      psis = datan2(cosw*sls,dcos(ls))
      rbarr = 1. + es*(chmp + es*(chmp-shmp)*(chmp+shmp))
      ll = psis - psig
      scz = cz
      ssz = sz
      ssl = sin(ll)
      scl = cos(ll)
      sdist = 1./rbarr
c
c     compute lunar constants for given t
c
 7    if(moon.eq.2) return
      hm=4.7199666d0+8399.7091449d0*t-.0000198d0*t2
      pm=5.83515154d0+71.01804120839d0*t-.180205d-3*t2
      nm=4.523601515d0-33.75714624d0*t+.3626406335d-4*t2
c.......  bl bls bf bd are the fundamental arguments of browns theory
      bl=hm-pm
      bls=hs-ps
      bf=hm-nm
      bd=hm-hs
c.........lunar lat long and parallax from brown.  latter two from
c......improved lunar ephemeris, latitude from ras paper of 1908...
      tlongm=hm+.10976*fsin(bl)-.02224*fsin(bl-2.*bd)+0.01149*fsin(2.*bd
     1)+0.00373*fsin(2.*bl)-.00324*fsin(bls)-.00200*fsin(2.*bf)-0.00103*
     2fsin(2.*bl-2.*bd)-.00100*fsin(bl+bls-2.*bd)+0.00093*fsin(bl+2.*bd)
     3-.00080*fsin(bls-2.*bd)+.00072*fsin(bl-bls)-.00061*fsin(bd)-.00053
     4*fsin(bl+bls)
      tlatm=.08950*fsin(bf)+.00490*fsin(bl+bf)-.00485*fsin(bf-bl)-.00303
     1 *fsin(bf-2.*bd)+.00097*fsin(2.*bd+bf-bl)-.00081*fsin(bl+bf-2.*bd)
     2+.00057*fsin(bf+2.*bd)
      plx=(3422.45+186.54*fcos(bl)+34.31*fcos(bl-2.*bd)+28.23*fcos(2.*bd
     1)+10.17*fcos(2.*bl)+3.09*fcos(bl+2.*bd)+1.92*fcos(bls-2.*bd)+1.44*
     2fcos(bl+bls-2.*bd)+1.15*fcos(bl-bls)-0.98*fcos(bd)-0.95*fcos(bl+
     3bls)-0.71*fcos(bl-2.*bf)+0.62*fcos(3.*bl)+0.60*fcos(bl-4.*bd))
      sinmla=dsin(tlatm)
      cosmla=dcos(tlatm)
      sinmln=dsin(tlongm)
      cosmln=dcos(tlongm)
c...convert from celestial lat and long according to explan suppl of
c......na and le page 26
      cz=cosmla*sinmln*sinw+sinmla*cosw
      sz=dsqrt(1.-cz**2)
      at1=cosmla*sinmln*cosw-sinmla*sinw
      at2=cosmla*cosmln
      ram=datan2(at1,at2)
      ll=ram-psig
      dcz = cz
      dsz = sz
      dsl = sin(ll)
      dcl = cos(ll)
      dpar = plx
      return
      end

