      subroutine tdfrph(idood,freq,phase)
      double precision freq,phase
c
c   Given the Doodson number of a tidal constituent (in idood), returns
c  the frequency and phase.  Phase is returned in degrees and frequency
c  in cycles/day.
c
c   Note that phases must be decreased by 90 degrees if the sum of the order
c  and the species number is odd (as for the 2nd degree diurnals, and 3rd
c  degree low-frequency and semidiurnals).
c   These phases may need further adjustment to allow for the spherical
c  harmonic normalization used; e.g. for that used for the potential by
c  Cartwright and Tayler, 180 degrees must be added for (species,order)
c  = (1,2), (1,3), or (3,3).
c
c  Common block date stores time information, in UT
c
c    calls toymd, leap, juldat, etutc (all for timekeeping)
c
      save itmsave,d,dd
      double precision f1,f2,f3,f4,f5
      double precision fd1,fd2,fd3,fd4,fd5
      double precision d(6),dd(6)
      double precision dayfr,djd,t
      dimension idood(6),itm2(6),itmsave(5)
      common/date/itm(5)
      data itmsave/5*0/
c
c  test to see if time has changed; if so, set the phases and frequencies
c   for each of the Doodson arguments
c
      intial=0
      do i=1,5
        if(itm(i).ne.itmsave(i)) initial=1
      enddo
      if(initial.eq.1) then
        do i=1,5
           itmsave(i) = itm(i)
        enddo
c
c convert times to Julian days (UT) then to Julian centuries from J2000.00
c   (ET)
c
        call toymd(itm,itm2)
        jd = juldat(itm2)
        dayfr=  itm(3)/24.d0 + itm(4)/1440.d0 + itm(5)/84600.d0
        year=itm(1)+(itm(2)+dayfr)/(365+leap(itm(1)))
        call etutc(year,delta)
        djd= jd - 0.5d0 + dayfr
        t = (djd - 2451545.0d0 + delta/86400.d0)/36525.d0
c
c IERS expressions for the Delauney arguments, in degrees
c
        f1 =     134.9634025100d0 +
     1    t*( 477198.8675605000d0 +
     1    t*(      0.0088553333d0 +
     1    t*(      0.0000143431d0 +
     1    t*(     -0.0000000680d0 ))))
        f2 =     357.5291091806d0 +
     1    t*(  35999.0502911389d0 +
     1    t*(     -0.0001536667d0 +
     1    t*(      0.0000000378d0 +
     1    t*(     -0.0000000032d0 ))))
        f3 =      93.2720906200d0 +
     1    t*( 483202.0174577222d0 +
     1    t*(     -0.0035420000d0 +
     1    t*(     -0.0000002881d0 +
     1    t*(      0.0000000012d0 ))))
        f4 =     297.8501954694d0 +
     1    t*( 445267.1114469445d0 +
     1    t*(     -0.0017696111d0 +
     1    t*(      0.0000018314d0 +
     1    t*(     -0.0000000088d0 ))))
        f5 =     125.0445550100d0 +
     1    t*(  -1934.1362619722d0 +
     1    t*(      0.0020756111d0 +
     1    t*(      0.0000021394d0 +
     1    t*(     -0.0000000165d0 ))))
c
c  convert to Doodson (Darwin) variables
c
        d(1) = 360.d0*dayfr - f4
        d(2) = f3 + f5
	d(3) = d(2) - f4
        d(4) = d(2) - f1
        d(5) = -f5
        d(6) = d(3) - f2 
c
c   find frequencies of Delauney variables (in cycles/day), and from these
c    the same for the Doodson arguments
c 
        fd1 =  0.0362916471 + 0.0000000013*t
        fd2 =  0.0027377786
        fd3 =  0.0367481951 - 0.0000000005*t
        fd4 =  0.0338631920 - 0.0000000003*t
        fd5 = -0.0001470938 + 0.0000000003*t
        dd(1) = 1.d0 - fd4
        dd(2) = fd3 + fd5
        dd(3) = dd(2) - fd4
        dd(4) = dd(2) - fd1
        dd(5) = -fd5
        dd(6) = dd(3) - fd2 
      endif
c
c   end of intialization (likely to be called only once)
c
c  compute phase and frequency of the given tidal constituent
c
      freq=0.d0
      phase=0.d0
      do i = 1,6
         freq =   freq + idood(i)*dd(i)
         phase = phase + idood(i)*d(i)
      enddo
c adjust phases to fall in the positive range 0 to 360
      phase = dmod(phase,360.d0)
      if(phase.lt.0.d0) phase = phase + 360.d0
      return
      end
