c   program autoff
c
c      Produces edits files that can be used to remove abrupt steps
c   in a data series, by doing linear fits (separated by some amount)
c   to two patches of data, and deciding if there is a step by looking
c   at the amount of offset needed to make these match up, and the ratio
c   of the variance (around a single linear fit) before and after doing
c   such a shift.
c      Original data are not altered--only an edits file is produced.
c   Optionally, the estimated step and rms change at each point can be
c   written to files, to be inspected along with the data (useful for
c   deciding on parameters).
c      If linear fits are done using least squares, the computations
c   involved are just convolutions with an appropriate set of filter
c   weights.
c
c
      character*1 not
      dimension ix(100000),istp(100000),wt(200)
      dimension rmsrat(100000)
      dimension igap(2,300),ioff(300)
      write(6,100)
 100  format(' File of data; ',$)
      ifin=ffin(1,not)
      len=longis(ifin)
      write(6,102)
 102  format(' Step-filter parameters (1/2 length of fit and gap); ',$)
      nf=ffin(1,not)
      mf=ffin(1,not)
      write(6,104)
 104  format(' Files for steps, rms changes (0 0 to skip); ',$)
      ifout=ffin(1,not)
      ifout2=ffin(1,not)
      write(6,106)
 106  format(' Lower limit for step estimate, rms change; ',$)
      stplim=ffin(1,not)
      rm1lim=ffin(1,not)
      write(6,108)
 108  format(' Round steps to nearest multiple of?; ',$)
      stpmul=ffin(1,not)
      if(stpmul.lt.1) stpmul=1
c
c  compute filter weights
c
      sc=1./(2*nf+1)
      do k=1,2*nf+1
         i=k
         wt(k)=sc*(1+(3.*(nf+mf+1.)/(nf*(nf+1.))*(i-nf-1.)))
      enddo
      do k=2*nf+2,2*nf+1+2*mf+1
         wt(k)=0.
      enddo
      do k=2*nf+1+2*mf+1+1,2*nf+1+2*mf+1+2*nf+1
         i=k-(2*nf+1+2*mf+1)
         wt(k)=-sc*(1-(3.*(nf+mf+1.)/(nf*(nf+1.))*(i-nf-1.)))
      enddo
      lf=2*nf+1+2*mf+1+2*nf+1
      lfh=2*nf+mf+1
c
c   read in data, padding both ends with the start (end) value
c
      call intio(ix(lfh+1),1,len,ifin,0)
      do i=1,lfh
        ix(i)=ix(lfh+1)
        ix(len+lfh+i)=ix(len+lfh)
      enddo
c
c  convolve the data with the weights to give a series of estimated offsets
c     index the result to match the data as stored in memory, though when
c     written out it will match the data as stored on disk
c
      do k=1,len
        istp(lfh+k)=0.  
        stp=0.  
        do j=-lfh,lfh
          in=j+lfh+1
          stp = stp + wt(in)*ix(lfh+k+j)
        enddo
        istp(lfh+k) = nint(stp)
      enddo
c
c produce a file of the rms misfit (to a constant value) with and without
c   the step as fit for. Use array wt as storage
c
      do k=1,len
        do j=-lfh,lfh
          in=j+lfh+1
          wt(in)=ix(lfh+k+j)
        enddo
        call rmss(wt,lf,rm1,sl1,var1,varl1)
c offset the data and repeat the estimate
        do j=1,lfh
          in=j+lfh+1
          wt(in)=wt(in)+istp(k+lfh)
        enddo
        call rmss(wt,lf,rm2,sl2,var2,varl2)
        rmsrat(k+lfh)=varl1/varl2
      enddo
c
c    write out file of steps
c
      if(ifout.ne.0) call intio(istp(lfh+1),1,len,ifout,1)
      if(ifout2.ne.0) call realio(rmsrat(lfh+1),1,len,ifout2,1)
C      ifl=0
C      do k=1,len
C       if(abs(istp(k+lfh)).lt.stplim.and.ifl.eq.1) then
C          write(6,*) 'step lim ',stplim,' exceeded over ',irl,k
C          ifl=0
C       endif
C       if(abs(istp(k+lfh)).ge.stplim.and.ifl.eq.0) then
C          irl=k
C          ifl=1
C       endif
C      enddo
      ifl=0
      ned=0
      do k=1,len
       if(rmsrat(k+lfh).lt.rm1lim.and.ifl.eq.1) then
          rmax=rmsrat(irl+lfh-1)
          do jj=irl-1,k+1
            if(rmsrat(jj+lfh).gt.rmax) then
               kc=jj
               rmax=rmsrat(jj+lfh)
               is=istp(jj+lfh)
            endif
          enddo
          if(abs(is).ge.stplim) then
           ned=ned+1
           igap(1,ned)=kc-mf
           igap(2,ned)=kc+mf
           if(stpmul.eq.1) ioff(ned)=is
           if(stpmul.gt.1) then
             if(is.ge.0) ioff(ned)=stpmul*nint(is/stpmul)
             if(is.lt.0) ioff(ned)=stpmul*nint(is/stpmul)
           endif
           write(6,*) 'location and size ',kc,ioff(ned)
           ifl=0
          endif
       endif
       if(rmsrat(k+lfh).ge.rm1lim.and.ifl.eq.0) then
          irl=k
          ifl=1
       endif
      enddo
      write(6,114)
 114  format(' File for edits; ',$)
      ifed=ffin(1,not)
      call edcon3(ifed,ned,igap,ioff,-1,0)
      stop
      end
      subroutine rmss(x,n,rm,sl,var,varl)
c for an array x of (odd) length n, finds the mean rm, the slope sl,
c and the two variances about the mean and about a best-fit slop
      dimension x(n)
      rm = 0.
      var = 0.
      sl=0.
      varl = 0.
      k=(n-1)/2
      sc=k*(k+1)*(2*k+1)/3.
      sc=1./sc
      do i=1,n
        rm = rm + x(i)
      enddo
      rm = rm/n
      do i=1,n
        var = var + (x(i)-rm)**2
      enddo
      var = var/(n-1.)
      do i=1,n
        sl=sl+x(i)*sc*(i-k-1)
      enddo
      do i=1,n
        p=rm+sl*(i-k-1)
        varl = varl + (x(i)-p)**2
      enddo
      return
      end
