c
c  program borecouple -- finds the coupling coefficients for a borehole
c   strainmeter, assuming 3 "layers" (strainmeter, grout, and rock), with
c   an inside material with a specified bulk modulus (for the hydraulic
c   strainmeter case).
c
c    D. C. Agnew, May 1992
c 
      character*1 not
      dimension e(9),gnu(9),r(9)
 3    write(6,100)
 100  format(' Number of cylindrical layers (h for help); ',$)
      n=ffin(1,not)
      if(not.eq.'h') then
        write(6,102)
 102    format(/,'Number of layers includes outermost material ',/,
     1  ' so for usual case of grouted-in pipe, number is 3.',/,
     2  ' Moduli and radii should be in consistent units',//)
        go to 3
      endif
      write(6,106)
 106  format('  Going from inside to out:')
c  read in, from inside to out, Young's modulus, Poisson's ratio, and
c  the inside radius.
      do i=1,n
         if(i.ne.n) write(6,108) i
 108     format(' Youngs modulus, Poissons ratio, inside radius'
     1  ,' of layer ',i2,'; ',$)
         if(i.eq.n) write(6,110)
 110     format(' Youngs modulus, Poissons ratio, inside radius'
     1  ,' of material; ',$)
         e(i)=ffin(1,not)
         gnu(i)=ffin(1,not)
         r(i)=ffin(1,not)
      enddo
c  read in the bulk modulus of the inside
      write(6,112)
 112  format(' Bulk modulus of inside of instrument; ',$)
      bk=ffin(1,not)
      call borcpl(e,gnu,r,n,bk,del,sh)
      write(6,120) del,sh
 120  format(' Areal and shear strain coupling is ',2f9.3)
      stop
      end
      subroutine borcpl(ec,gnu,rc,n,bkc,del,sh)
c
c   For a series of cylinders of elastic material, computes the
c   plane-stress coupling coefficients. There are assumed to be n-1
c   cylinders, of inside radius r, Young's modulus e, and Poisson's
c   ratio gnu; rc(n), ec(n), and gnu(n) apply to the medium in which
c   the cylinders are embedded. bkc is the bulk modulus of the material
c   filling the innermost volume (assumed fluid). Uses propagator matrices.
c
c   del and sh are returned as the coupling constants for areal strain
c   and pure shear
c
c      calls mult, invert
c
c    D. C. Agnew, May 1992
c 
      dimension ec(n),gnu(n),rc(n)
      dimension e(9),r(9)
      dimension ramat(2,2),oamat(2,2),qamat(2,2),pamat(2,2),
     1  bamat(2,2),f(2),g(2),h(2)
      dimension rsmat(4,4),osmat(4,4),qsmat(4,4),psmat(4,4),
     1  bsmat(4,4),f1(4),hs(4),f2(4)
      dimension x(4)
      do 3 j=1,4
      do 1 i=1,4
      if(i.le.2.and.j.le.2) then
         qamat(i,j) = 0.
         if(i.eq.j) qamat(i,j) = 1.
      endif
      qsmat(i,j) = 0.
      if(i.eq.j) qsmat(i,j) = 1.
 1    continue
 3    continue
c  normalize the Young's modulus and radius to the outmost medium
      bk = bkc/ec(n)
      do 11 i=1,n
      e(i) = ec(i)/ec(n)
      r(i) = rc(i)/rc(n)
 11   continue
c
c       Areal strain case
c
      do 21 i=n,2,-1
c  from the outside in, construct the layer matrices and multiply them
c  to get the propagator
      call cpl1(ramat,e(i-1),r(i),gnu(i-1))
      call cpl1(oamat,e(i),r(i),gnu(i))
      call invert(ramat,2,det)
      call mult(ramat,oamat,pamat,2,2,2)
      call mult(pamat,qamat,ramat,2,2,2)
      call matcop(ramat,qamat,2)
 21   continue
c  now have the propagator; construct the additional matrices
      f(1) = 1./(r(1)*r(1))
      f(2) = 2.
      g(1) = 0.
      g(2) = 4*(1-gnu(n))/e(n)
      h(1) = -2.*(1+gnu(1))/(e(1)*r(1)*r(1))
      h(2) = 4*(1-gnu(1))/e(1)
      f(1) = f(1) - bk*h(1)
      f(2) = f(2) - bk*h(2) + bk*4*gnu(1)/e(1)
      call mult(f,qamat,x,1,2,2)
      bamat(1,1) = x(1)
      bamat(1,2) = x(2)
      bamat(2,1) = g(1)
      bamat(2,2) = g(2)
      call invert(bamat,2,det)
      call mult(qamat,bamat,pamat,2,2,2)
      call mult(h,pamat,x,1,2,2)
      del = x(2)
c
c       Shear strain case
c
      do 31 i=n,2,-1
c  from the outside in, construct the layer matrices and multiply them
c  to get the propagator
      call cpl2(rsmat,e(i-1),r(i),gnu(i-1))
      call cpl2(osmat,e(i),r(i),gnu(i))
      call invert(rsmat,4,det)
      call mult(rsmat,osmat,psmat,4,4,4)
      call mult(psmat,qsmat,rsmat,4,4,4)
      call matcop(rsmat,qsmat,4)
 31   continue
c  now have the propagator; construct the additional matrices
      f1(1) = -6/(r(1)**4)
      f1(2) = -4/(r(1)*r(1))
      f1(3) = -2
      f1(4) =  0
      f2(1) =  6/(r(1)**4)
      f2(2) =  2/(r(1)*r(1))
      f2(3) = -2
      f2(4) = -6*r(1)*r(1)
      call mult(f1,qsmat,x,1,4,4)
      bsmat(1,1) = x(1)
      bsmat(1,2) = x(2)
      bsmat(1,3) = x(3)
      bsmat(1,4) = x(4)
      call mult(f2,qsmat,x,1,4,4)
      bsmat(2,1) = x(1)
      bsmat(2,2) = x(2)
      bsmat(2,3) = x(3)
      bsmat(2,4) = x(4)
      bsmat(3,1) = 0
      bsmat(3,2) = 0
      bsmat(3,3) = 0
      bsmat(3,4) = 1
      bsmat(4,1) = 0
      bsmat(4,2) = 0
      bsmat(4,3) = -2*(1+gnu(n))/e(n)
      bsmat(4,4) = 0
      hs(1) =  2*(1+gnu(1))/(e(1)*(r(1)**4))
      hs(2) =  4/(e(1)*r(1)*r(1))
      hs(3) = -2*(1+gnu(1))/e(1)
      hs(4) = -4*gnu(1)*(r(1)**2)/e(1)
      call invert(bsmat,4,det)
      call mult(qsmat,bsmat,psmat,4,4,4)
      call mult(hs,psmat,x,1,4,4)
      sh = x(4)
      return
      end
      subroutine cpl1(c,e,r,g)
      dimension c(2,2)
c  computes terms in the 2x2 areal-strain coupling matrices
      c(1,1) = -(1+g)/(r*e)
      c(1,2) = 2*(1-g)*r/e
      c(2,1) = 1./(r*r)
      c(2,2) = 2.
      return
      end
      subroutine cpl2(c,e,r,g)
      dimension c(4,4)
c  computes terms in the 4x4 shear-strain coupling matrices
      c(1,1) =  2*(1+g)/(e*(r**3))
      c(1,2) =  4/(e*r)
      c(1,3) = -2*(1+g)*r/e
      c(1,4) = -4*g*(r**3)/e
      c(2,1) = -2*(1+g)/(e*(r**3))
      c(2,2) =  2*(1-g)/(e*r)
      c(2,3) = -2*(1+g)*r/e
      c(2,4) = -2*(3+g)*(r**3)/e
      c(3,1) = -6/(r**4)
      c(3,2) = -4/(r*r)
      c(3,3) = -2
      c(3,4) =  0
      c(4,1) =  6/(r**4)
      c(4,2) =  2/(r*r)
      c(4,3) = -2
      c(4,4) = -6*r*r
      return
      end
      subroutine matcop(a,b,n)
c   copies n-by-n matrix a to b
      dimension a(n,n),b(n,n)
      do 3 i=1,n
      do 1 j=1,n
      b(i,j) = a(i,j)
 1    continue
 3    continue
      return
      end
      subroutine mult(a,b,c,n,m,l)    
c$$$$$ calls no other routines 
c  routine multiplies matrices a and b, putting the answer into c
c  a  is n by m  nad b is m by l in dimension
      dimension a(*),b(*),c(*) 
      lm=l*m     
      l1=0
      do 1 j=1,lm,m     
      do 1 i=1,n 
      k1=i
      k2=j
      x=0.
      do 2 k=1,m 
      x=x+a(k1)*b(k2)   
      k1=k1+n    
 2    k2=k2+1    
      l1=l1+1    
 1    c(l1)=x    
      return     
      end
      subroutine invert(a,n,determ)
c$$$$$ calls no other routines
c  modified version of ucsd matinv,  to invert square matrix  a  in place.      
c  the array  a  must be n by n close packed, with n.le.30,  determinant
c  appears in  determ. singular matrices are detected by zero determinant
      dimension a(n,n),ipivot(30),index(30,2),pivot(30)
      equivalence(irow,jrow),(icolum,jcolum),(amax,t,swap)
      determ=1.0
      do 20 j=1,n
   20 ipivot(j)=0
      do 550 i=1,n
      amax=0.0
      do 105 j=1,n
      if(ipivot(j)-1)60,105,60
   60 do 100 k=1,n
      if(ipivot(k)-1)80,100,740
   80 if(abs(amax)-abs(a(j,k)))85,83,83
 83   if (.not.(amax.eq.0.0 .and. a(j,k).eq.0.0)) go to 100
   85 irow=j
      icolum=k
      amax=a(j,k)
  100 continue
  105 continue
      ipivot(icolum)=ipivot(icolum)+1
      if(irow-icolum)140,260,140
  140 determ=-determ
      do 200 l=1,n
      swap=a(irow,l)
      a(irow,l)=a(icolum,l)
  200 a(icolum,l)=swap
  260 index(i,1)=irow
      index(i,2)=icolum
      pivot(i)=a(icolum,icolum)
      determ=determ*pivot(i)
      if (determ.eq.0) return
      a(icolum,icolum)=1.0
      do 350 l=1,n
  350 a(icolum,l)=a(icolum,l)/pivot(i)
      do 550 l1=1,n
      if(l1-icolum)400,550,400
  400 t=a(l1,icolum)
      a(l1,icolum)=0.0
      do 450 l=1,n
  450 a(l1,l)=a(l1,l)-a(icolum,l)*t
  550 continue
      do 710 i=1,n
      l=n+1-i
      if(index(l,1)-index(l,2))630,710,630
  630 jrow=index(l,1)
      jcolum=index(l,2)
      do 705 k=1,n
      swap=a(k,jrow)
      a(k,jrow)=a(k,jcolum)
      a(k,jcolum)=swap
  705 continue
  710 continue
  740 return
      end
