      program ddrift
c   program for fitting cubic splines to a data series at preselected
c  points, so as to fit and remove "drift".  points may be entered from
c  the keyboard or by using the edit feature of credit (in which case
c  they are taken to be the leading edge of each edit).  the first and
c  last points of the file are always fitted, but the spline
c  (or residual) is only computed over the range asked for.
c  a triangular average around the selected points may be taken, to 
c  minimize aliasing effects.
c
c    **calls regap,spline,eval,iolib
c
      dimension d(120),id(120),x(400),u(400),s(400),a(400)
      data npt/370/,w/0./,a(1),x(1)/2*1./
      write(6,100)
 100  format(' data file and length; ',$)
      ifd = ffin(1,not)
      rntt = ffin(1,not)
      if(rntt.le.0) rntt=longis(ifd)
      write(6,102)
 102  format(' start and end terms; ',$)
      st = ffin(1,not)
      en = ffin(1,not)
      if(en.le.0) en=rntt
      write(6,104)
 104  format(' edits file (-n to enter n points from terminal; ',$)
      ifed = ffin(1,not)
      if (ifed.lt.0) go to 3
      call regap(ifed,u,x(2),npt,rntt)
      npt = npt + 1
      go to 7
 3    npt = -ifed
      write(6,106)
 106  format(' enter points to be fit; ',$)
      do 5 i = 1,npt
 5    x(i+1) = ffin(1,not)
      npt = npt + 2
      x(npt) = rntt
 7    write(6,108)
 108  format(' averaging width; ',$)
      iw = ffin(1,not)
c  make sure we average over an odd number of points
      iw = iw/2
      np = 2*iw + 1
      if(iw.eq.0) go to 10
c  generate a weights file
      do 9 i = 1,np
      ri = i - iw + 1
      a(i) = 1. - abs(ri)/iw
 9    w = w + a(i)
 10   if(iw.eq.0) w = 1
c  find the data values except at first and last points
      do 13 i = 2,npt-1
      ixi = x(i)
      call realio(d,ixi-iw,ixi+iw,ifd,0)
      u(i) = 0.
      do 11 j = 1,np
 11   u(i) = u(i) + d(j)*a(j)
 13   u(i) = u(i)/w
c  do first and last points without averaging
      call realio(u(1),1,1,ifd,0)
      irntt = rntt
      call realio(u(npt),irntt,irntt,ifd,0)
c  generate spline
      call spline(npt,x,u,s,a)
      write(6,110)
 110  format(' type 0 for residual, 1 for fitting function; ',$)
      icon = ffin(1,not)
      if(icon.ne.0) icon = 1
      write(6,112)
 112  format(' output file (real or integer; ',$)
      ifou = ffin(1,not)
      ifed = ifou
c     np = (en-st+60)/60
      if(ifed.gt.0) call intio(id,1,1,ifou,1)
      if(ifed.lt.0) call realio(d,1,1,ifou,1)
      rl = st
 19   rh = amin1(rl+119.,en)
      rlo = rl - st + 1
      rho = rh - st + 1
      np = rh - rl + 1
      do 21 i = 1,np
      d(i) = eval(rl+i-1,npt,x,u,s)
 21   if(ifed.gt.0) id(i) = d(i)
      if(icon.eq.1) go to 25
      call realio(a,int(rl),int(rh),ifd,0)
      do 23 i = 1,np
      if(ifed.gt.0) id(i) = a(i) - d(i)
 23   d(i) = a(i) - d(i)
 25   irlo = rlo
      irho = rho
      if(ifed.gt.0) call intio(id,irlo,irho,ifou,-1)
      if(ifed.lt.0) call realio(d,irlo,irho,ifou,-1)
      rl = rh + 1.
      if(rl.le.en) go to 19
      write(6,114) en-st+1, ifed
 114  format(2x,f7.0,' terms written to file ',i6)
      stop
      end

