c Program edjoy
c
c 3/85 by N.E. King.
c Converted to VAX UNIX 1/86 by N.E. King.
c Modified 9/86 by N.E. King:
c          Input headers must be in ascending order by start time.
c          Output header file is no longer optional.
c          User specifies desired channels, as in PINJOY.
c Many mods 4/98 by D Agnew
c
c Program constructs edits files, and a header, for joined files.
c
c Input (from terminal):      Sampling interval of input series, sec.
c                             Channel numbers of files to be joined (<= 16).
c                             Header numbers for input edits files (<= maxhd).
c                             Output header file number. 
c                             Start time of joined files, yr-da-hr-mn-sc.
c                             Sample interval of joined files, sec.
c                      
c Input (from header files):  For each header:
c                                 file numbers of edits files;
c                                 start time, yr-da-hr-mn-sc;
c                                 number of channels;
c                                 channel numbers.
c
c Output:                     Edits files for joined files.
c                             Header for output header files.
c
c Internal subroutines:       chktime,hedin,
c                             hedout,newfil,rdin,rdout,resed (adapted from
c                             shred)
c                             (These subroutines follow main program in
c                             alphabetical order.)
c
c External subroutines:       ffin,headco,edcon3,scansp(function).
c
c Libraries required:         libgen,libterm,libhed,libnio.
c
c See comments in subroutines for description of variables.
c
      character*1 mode
      character*4 stc1
      character*8 chnid1
c
      dimension chnid1(16),igap(2,6000),ichanl(16),ihead(51),
     $          ioff(2,6000),it1(5,50),it2(5),num(16,50)
c
      data maxed/6000/,mode/'d'/,maxhd/50/,ifl/20000000/
c
c Read input sample interval, channel numbers, and input header file numbers.
c
      call rdin(maxhd,isptin,nch,ichanl,nhead,ihead)
c
c Retrieve and store info from each header.
c Also store station code and channel names from 1st header,
c for use in constructing header for joined edits files.
c
      call hedin(nhead,ihead,nch,ichanl,it1,num,stc1,chnid1)
c
c Read output header file number, output start time, and output sampling 
c interval.
c
      call rdout(nhead,isptin,ihout,isptout,it2,leno)
c
c For each channel, construct new edits file and write to disk.
c
      iflag = 0
c
      do 100 i=1,nch
      call newfil(ifl,i,nhead,num,it1,it2,isptin,
     $            isptout,leno,maxed,mode,igap,ioff,nedits)
      iflag = iflag + nedits
      call edcon3(ihout+i,nedits,igap,ioff,0,0)
C      if(nedits.ne.0) call edcon3(ihout+i,nedits,igap,ioff,0,0)
100   continue
c
c Create header for joined edits files.
c
      if(iflag.ne.0)
     $  call hedout(nch,it2,ihout,ichanl,stc1,chnid1)
c
      if(iflag.eq.0) write(6,3001)
3001  format(' No edits or header files created')
c
      stop
      end
      subroutine chktime(nhead,it1,iflag)
c
c Subroutine checks that headers are in chronological order.
c
c Input from hedin:  nhead   Number of headers.
c                    it1     2-d array containing start times, in format
c                                 yr-da-hr-mn-sc.
c 
c Output to hedin:   iflag   index in array ihead of a header not in
c                                 chronological order, zero otherwise.
c                            
      dimension it1(5,1)
c
      iflag = 0
      time1 = it1(1,1) + float(it1(2,1))/367.
c
      do 200 i=2,nhead
      time2 = it1(1,i) + float(it1(2,i))/367.
      if(time2.gt.time1) go to 100
c
c Execute this block upon finding 1st header not in chronological order.
c
      iflag = i
      return
c
100   time1 = time2
200   continue
c
      return
      end
      subroutine hedin(nhead,ihead,nch,ichanl,it1,num,stc1,chnid1)
c
c Subroutine retrieves and stores data from headers.
c
c Input from main:     nhead    number of headers.
c                      ihead    array of header file numbers.
c                      nch      number of channels to be joined.
c                      ichanl   array of channel numbers to be joined.
c
c Output to main:      it1      2-d array containing start times, in format
c                                     yr-da-hr-mn-sc.
c                      num      2-d array containing edits file numbers;
c                                     num(j,i) refers to jth edits file
c                                     of ith header.
c                      stc1     station code from last header.
c                      chnid1   channel ids from last header.
c
c Internal variables:  iflag    used in 2 separate tests:
c                               (1) error count, number of times a channel in
c                                   array ichanl is not found in a header;
c                               (2) index in array ihead of 1st header not in
c                                   chronological order, zero otherwise.
c                      nchann   number of channels in header being unpacked.
c
c Calls headco, chktime.
c
      double precision scans
      character*4 stc,stc1,commen
      character*8 source,chanid,chnid1
c
      common/head/scans,calc(4),clkc(2),sampin,nchann,iblock,istart(5),
     1     iend(5),nchf(17),ichnm(16),ncal(4),source,chanid(16),stc,
     2     commen(200)
c
      dimension chnid1(1),ihead(1),ichanl(1),it1(5,20),num(16,20)
c
c Unpack header files. 
c Fill array index for each header, displaying message and quitting if any
c channel in array ichanl is not found in all of the headers of array ihead.
c Save channel ids and station code from last header, for user in output header.
c
      iflag = 0
c
      do 130 i=1,nhead
      call headco(ihead(i),1)
      do 100 j=1,5
100   it1(j,i) = istart(j)
c now load the array of file numbers, checking that the channel is
c present; if the 
      do 120 j=1,nch
      num(j,i) = 0
      ifnd=0
      do 110 k=1,nchann
      if(ichanl(j).eq.ichnm(k)) then
	 ifnd=1
         num(j,i) = nchf(k)
         if(i.eq.nhead) chnid1(j) = chanid(k)
      endif
110   continue
      if(ifnd.eq.0) then
         write(6,*) ichanl(j)
         write(6,4001) ichanl(j), ihead(i)
4001     format(' Channel ',i9,' not found in header ',i5)
         stop
      endif
120   continue
      write(6,5000) ihead(i)
 5000 format(/,'Edits files for header ',i6,' :',1x,$)
      write(6,5001) (num(j,i),j=1,nch)
 5001 format(13i6)
      if(i.eq.nhead) stc1 = stc
130   continue
c
c Check that input headers are in chronological order.
c Display message and stop if not.
c
      call chktime(nhead,it1,iflag)
      if(iflag.eq.0) return
c
      write(6,4002) 
4002  format(/' Header(s) out of order:'/)
      write(6,4003) (ihead(k),it1(1,k),it1(2,k),it1(3,k),it1(4,k),
     $                       it1(5,k),k=1,nhead)
4003  format(1x,i5,10x,5i5)
      stop
c
      end
      subroutine hedout(nch,it2,ihout,ichanl,stc1,chnid1)
c
c Subroutine creates header for new edits files
c
c Input from main:    nch      number of channels.
c                     it2      array containing start time of joined files,
c                              yr-da-hr-mn-sc.
c                     ihout    file number of header to be created.
c                     ichanl   array containing channel numbers.
c                     stc1     station code.
c                     chnid1   array containing channel ids.
c
c Calls headco.
c
      double precision scans
      character*4 stc,stc1,commen,slash
      character*8 source,chanid,chnid1,blank
c
      common/head/scans,calc(4),clkc(2),sampin,nchann,iblock,istart(5),
     1     iend(5),nchf(17),ichnm(16),ncal(4),source,chanid(16),stc,
     2     commen(200)
c
      dimension chnid1(1),ichanl(1),it2(1)
      data slash/'////'/
      data blank/'        '/
c
      commen(1) = slash
      nchann = nch
      sampin = 0.
      stc = stc1
      scans = 1441.d0
c
      do 101 i=1,5
      istart(i) = it2(i)
101   iend(i) = it2(i)
c
      do 103 i=1,nchann
      chanid(i) = chnid1(i)
      ichnm(i) = ichanl(i)      
103   nchf(i) = ihout + i
c
      if(nchann.eq.16) go to 105
      do 104 i=nchann+1,16
      chanid(i) = blank
      ichnm(i) = 0
104   nchf(i) = 0
105   nchf(17) = ihout
c
      write(6,3001)
3001  format(' Source tape name for new header;; ',$)
      read(5,1001) source
1001  format(a8)
      call headco(ihout,0)
c
      return
      end
      subroutine newfil(ifl,ich,nhead,num,it1,it2,isptin,
     $                  isptout,leno,maxed,mode,igap,ioff,nedits)
c
c Subroutine constructs edits file for one channel.
c
c Input from main:       ifl       maximum edits file length.
c                        ich       channel order
c                        nhead     number of input headers.
c                        num       array containing file numbers of
c                                        edits files; num(j,i) is number
c                                        of file for jth channel in ith header.
c                        it1       array of start times.
c                        it2       output start time.
c                        isptin     input sampling interval.
c                        isptout    output sampling interval.
c                        leno      length of output series
c                        maxed     max number edits.
c                        mode      start time format (d).
c              
c Output to main:        igap,ioff  edits arrays.
c                        nedits    number of edits.
c
c Internal variables:    i1        start time of each segment.
c                        indout    index of arrays for combined edits file.
c                        itotal     cumulative number of terms.
c                        kount     number of non-zero-length edits from
c                                  each segment.
c
c Calls scansp, resed.
c
      character*1 mode
      dimension num(16,1),it1(5,1),it2(1),i1(5),
     $          igap1(2,6000),ioff1(6000),igap(2,1),ioff(1)
c
      indout = 0
      itotal = 0
      nedits = 0
      do 50 i=1,5
50    i1(i) = it2(i)
c
c Loop on headers (ie. segments, in order from earliest).
c
      do 300 i=1,nhead
      kount = 0
      call resed(num(ich,i),maxed,ifl,leno,it1(1,i),
     $           isptin,i1,isptout,mode,igap1,ioff1,ned)
      if(ned.eq.0) go to 225
c  strip out zero-length edits
      do 200 j=1,ned
      if(igap1(2,j)-igap1(1,j).eq.1.) go to 200
      kount = kount + 1
      indout = indout + 1
      do 100 k=1,2
100   igap(k,indout) = igap1(k,j) + itotal
      ioff(indout) = ioff1(j)
200   continue
      if(kount.eq.0) then
        write(6,122) num(ich,i)
 122    format(' No edits in file ',i5)
      endif
      nedits = nedits + kount
225   if(i.eq.nhead) go to 300
c
      itotal = iscnsp(it2,it1(1,i+1),isptout,mode)
      do 250 j=1,5
250   i1(j) = it1(j,i+1)
c
300   continue
c
      return
      end
      subroutine rdin(maxhd,isptin,nch,ichanl,nhead,ihead)
c
c Subroutine reads and checks sample interval, channel numbers, and input
c header numbers.
c
c Input from main:       maxhd     maximum number of headers allowed.
c
c Output to main:        isptin     input sample interval, sec.
c                        nch       number of channels to be joined.
c                        ichanl    array of channel numbers to be joined.
c                        nhead     number of headers (<=maxhd).
c                        ihead     array of header numbers.
c
c Calls ffin.
c
      character*1 not
      dimension ihead(1),ichanl(1)
c
c *****************************************************************************
c
c Read and check sample interval.
c
100   write(6,3001)
      isptin = ffin(1,not)
      if(isptin.gt.0) go to 110
c
      write(6,4001)
      go to 100
c
c *****************************************************************************
c
c READ CHANNEL NUMBERS TO BE JOINED.
c
110   write(6,3002)
      nch = 0
c
120   itemp = ffin(1,not)
      if(itemp.eq.0) go to 200
      nch = nch + 1
      if(nch.gt.16) go to 130
      ichanl(nch) = itemp
      go to 120
c
c Execute this block if more than 16 channel numbers were entered.
c Display message and give user option to continue with 1st 16 channels.
c
130   idum = ffin(1,not)
      if(idum.ne.0) go to 130
c
140   write(6,4002)
      idum = ffin(1,not)
      if(not.eq.'n') stop
      if(not.ne.'y') go to 140
      nch = 16
c
c *****************************************************************************
c
c READ FILE NUMBERS OF HEADERS CONTAINING INPUT EDITS FILES, UNTIL USER
c ENTERS ZERO
c
200   nhead = 0
c 
      do 300 i=1,maxhd+1
      write(6,3003)
      itemp = ffin(1,not)  
      if(itemp.eq.0) return
      nhead = nhead + 1
300   ihead(nhead) = itemp
c
c Execute this block if more than maxhd header numbers have been entered.
c Give user option to continue with the first maxhd headers.
c
400   write(6,4003) maxhd,maxhd
      idum = ffin(1,not)
      if(not.eq.'n') stop
      if(not.ne.'y') go to 400
      nhead = maxhd
      return
c
c *****************************************************************************
c
3001  format(' Input series sample interval, sec; ',$)
3002  format(' Channel numbers (0 to end); ',$)
3003  format(' Header for edits files; ',$)
4001  format(' Invalid sample interval')
4002  format(' Number of channels cannot exceed 16'/
     $       ' Do you want to continue with the 1st 16 (y/n)?; ',$)
4003  format(' Number of headers must be in [1,',i2,']'/
     $       ' Do you want to continue with the 1st ',i2,' (y/n)?; ',$)
      end
      subroutine rdout(nhead,isptin,ihout,isptout,it2,leno)
c
c Subroutine reads and checks output parameters--file number of output header,
c output sampling interval, and output start time.
c
c Input from main:    nhead    number of input headers.
c                     isptin    input sampling interval, sec.
c
c Output to main:     ihout    output header file number;
c                              output edits files are numbered from ihout.
c                     isptout   output sampling interval, sec.
c                     it2      array containing output start time, in
c                                    format yr-da-hr-mn-sc.
c                     leno     length of output series
c
c Internal variables: i1,i2,   integers used to check that output sample
c                     itest          interval is a multiple of the 
c                                    input interval, and whether output
c                                    start year is a leap year.
c                     l1,l2    arrays containing lower and upper bounds
c                                    for output start time.
c
c Calls ffin.
c
      character*1 not
      dimension it2(1),l1(5),l2(5)
      data l1/1900,1,0,0,0/
      data l2/2100,365,23,59,59/
c
c Read output header file number.
c
      write(6,3001) 
      ihout = ffin(1,not)
c
c Read output sample interval.
c Check that output sample interval is a positive multiple of the input 
c sample interval.
c
200   write(6,3003)
      isptout = ffin(1,not)
      i1 = isptin
      i2 = isptout
      itest = isptout/isptin
      if(isptout.gt.0.and.itest*isptin.eq.isptout) go to 300
c
      write(6,4003) isptin
      go to 200
c
c Read and check output start time.
c
300   write(6,3004)
      do 400 i=1,5
400   it2(i) = ffin(1,not)
c
      itest = it2(1)/4
      if(4*itest.eq.it2(1).and.it2(1).ne.1900) then 
          l2(2) = 366
      else
          l2(2) = 365
          end if
c
      do 500 i=1,5
      if(it2(i).ge.l1(i).and.it2(i).le.l2(i)) go to 500
      write(6,4004)
      if(i.eq.1) write(6,4005)
      go to 300
500   continue
      write(6,3005)
      leno = ffin(1,not)
      return
c
3001  format(' Output header number                  :; ',$)
3003  format(' Output sample interval (sec)          :; ',$)
3004  format(' Output series start time (yr-d-h-m-s) :; ',$)
3005  format(' Output series length                  :; ',$)
4003  format(' Output interval must be positive multiple of input',
     $       ' interval ',i6)
4004  format(' Invalid output start time')
4005  format(' Year must be in our lifetimes')
      end
c  subroutine resed (adapted from shred)

c    Takes edits files and 
c     1. rewrites them to apply to a series with different start time 
c        and sample interval.  the offsets are used where edits are kept.
c        more than one edits file may be processed, and the results combined
c        later using merged
c
      subroutine resed(ifed,maxed,ifl,iofl,isti,isampi,isto,isampo,
     $                 mode,igapo,ioff,ned)
c
      character*1 mode
      dimension ioff(6000),igap(2,6000),isti(6),isto(5),igapo(2,6000),
     1 itim(6),ig(2)
      ned = maxed
      ifile = ifed
      call edcon3(ifed,ned,igap,ioff,1,ifl)
      newed = 0
      do 21 j=1,ned
c  Find new gap positions corresponding to the same times
      do 19 k=1,2
      call timfnd(isti,igap(k,j)-1,isampi,itim,mode)
 19   ig(k) = iscnsp(isto,itim,isampo,mode) + 1
c  Ignore edits that fall before start or after end of output series,
c  or would be less than one term long
      if(ig(2).lt.1.or.ig(1).gt.iofl.or.ig(2)-ig(1).le.1) go to 21
      newed = newed + 1
c  Boundary-crossing edits are truncated to the start and end
      if(ig(1).lt.1) ig(1) = 1
      if(ig(2).gt.iofl) ig(2) = iofl
c  Put new gap numbers into output array
      do 20 k=1,2
 20   igapo(k,newed) = ig(k)
      ioff(newed) = ioff(j)
 21   continue
      ned=newed
      if(ned.eq.0) then
        write(6,122) ifile
 122    format(' no edits in file ',i7)
        return
      endif
c  Now go through list and combine edits which have less than one term of
c  good data between them, also converting gap edges to integers using the
c  nearest integer rule
      if(ned.eq.1) return
      j=1
 24   if(igapo(1,j+1).ge.igapo(2,j)+1) go to 26
      igapo(2,j) = igapo(2,j+1)
      ioff(j) = ioff(j) + ioff(j+1)
      ned = ned - 1
      if(ned.eq.0) then
         write(6,122) ifile
         return
      endif
      if(j.eq.ned) return
      do 25 l=j+1,ned
      do 25 k=1,2
      ioff(l) = ioff(l+1)
 25   igapo(k,l) = igapo(k,l+1)
      go to 24
 26   j = j + 1
      if(j.lt.ned) go to 24
      return
      end
