c   driver program for parks-mcclellan filter design routine
c
c   calls pmcfir,effwat,iolib,firres,atrig
c
      character*1 not
      double precision ad,dev,x,y,pi2
      dimension des(1648),wt(1648),grid(1648)
      dimension iext(103),alpha(103),ad(103),x(103),y(103)
      dimension h(103),edge(20),fx(10),wtx(10)
      common des,wt,alpha,iext,nfcns,ngrid,pi2,ad,dev,x,y,grid
      common/wtsc/h,edge,fx,wtx,ipexp
      data pi2/6.283185307179586d0/,nmx/202/,ispset/0/
      padn = sqrt(2000.)
      write(6,98)
 98   format(' Value of nyquist frequency; ',$)
      fny = ffin(1,not)
      write(6,100) 
 100  format(' Filter type (1-bandpass,2-diff,3-h.t.); ',$)
      jt = ffin(1,not)
 3    write(6,102) 
 102  format(' Number of bands; ',$)
      nb = ffin(1,not)
      if(nb.gt.10) go to 3
      write(6,104) 
 104  format(' For each band enter the frequencies of',
     1' the upper and lower edges',/,' the value (or slope) wanted',
     2' and the weighting value',/)
      do 5 i = 1,nb
      write(6,106)  i
 106  format(' Band',i3,'; ',$)
      edge(2*i-1) = ffin(1,not)/(2.*fny)
      edge(2*i) = ffin(1,not)/(2.*fny)
      fx(i) = ffin(1,not)
      wtx(i) = ffin(1,not)
 5    if(wtx(i).lt.0) ispset = 1
 7    write(6,108) 
 108  format(' Filter length (may be 0 for lowpass); ',$)
      nfil = ffin(1,not)
      if((nfil.eq.0.and.jt.ne.1).or.nfil.gt.nmx) go to 7
      if(nfil.ne.0) go to 9
      write(6,110) 
 110  format(' Passband ripple and stopband attenuation; ',$)
      rip = ffin(1,not)
      att = ffin(1,not)
 9    write(6,112) 
 112  format(' Grid density (le 32, 0 for default); ',$)
      lg = ffin(1,not)
      if(ispset.eq.0) go to 10
      write(6,113) 
 113  format(' Exponent for weight dependence; ',$)
      ipexp = ffin(1,not)
 10   call pmcfir(jt,nb,nfil,rip,att,lg)
c  weights are returned in h, elements 1 through nfcns. need
c  them reversed so as to be 'center to right'.
      do 12 i = 1,nfcns
 12   alpha(i) = h(nfcns-i+1)
      do 13 i = 1,1648
 13   des(i) = 0.
 15   write(6,114) 
 114  format(' Type start, end, interval frequencies; ',$)
      sf = ffin(1,not)
      ef = ffin(1,not)
      fi = ffin(1,not)
      if(fi.eq.0) go to 19
      nf = (ef-sf)/fi + 1
      do 17 i = 1,nf
      f = sf + fi*(i-1)
      rip = f/(2.*fny)
      it = 0
      if(2*(nfil/2).ne.nfil) it = 1
      if(jt.eq.1) call firres(alpha,des,nfcns,0,it,rip,att)
      if(jt.ne.1) call firres(des,alpha,nfcns,1,it,rip,att)
      rip = 20.*alog10(att+1.e-25)
      write(6,116)  f,att,rip
 116  format(2x,g13.5,g15.7,f7.1)
 17   continue
      go to 15
 19   write(6,118) 
 118  format(' Output file; ',$)
      ifl = ffin(1,not)
      write(6,120) 
 120  format(' 1 for weights, 0 for them padded to 2000; ',$)
      ic = ffin(1,not)
      do 21 i = 1,nfcns
      wt(i) = h(i)
      wt(nfil-i+1) = h(i)
 21   if(jt.ne.1) wt(nfil-i+1) = -wt(nfil-i+1)
      if(jt.ne.1.and.it.eq.1) wt(nfcns+1) = 0.
      if(ic.eq.0) go to 23
      nb = 1
      call realio(wt,1,nfil,ifl,nb)
      write(6,122)  nfil, ifl
 122  format(i5,' points written to file ',i5)
      stop
 23   do 24 i=1,1000
 24   des(i) = 0.
      call realio(des,1,1000,ifl,1)
      call realio(des,1001,2000,ifl,-1)
      do 25 i = 1,nfil
 25   wt(i) = padn*wt(i)
      call realio(wt,1001-nfcns,1000-nfcns+nfil,ifl,-1)
      write(6,122) 2000,ifl
      stop
      end
      subroutine pmcfir(jtype,nbands,nfilt,ripple,attn,lgrid)
c     program for the design of linear phase finite impulse
c     response (fir) filters using the remez exchange algorithm   
c     jim mcclellan, rice university, april 13, 1973
c     see ieee trans. on audio and electroacoustics,
c     vol. au-21, no. 6, december 1973.
c     three types of filters are included--bandpass filters
c     differentiators, and hilbert transform filters
c  
c  
c  the input data are:
c
c    jwrite(6,- filter type (1=bandpass, 2=differentiator, 3=
c            hilbert transformer)
c    nbands - the number of bands (le. 10) in which the
c            frequency response will be specified
c    nfilt - the total filter length, which may
c            be even or odd. (it must be odd for the filter
c            to be zero phase shift).
c    ripple, attn - if the filter is a lowpass and the length
c            is input as zero, the length necessary to give
c            the required ripple (in the passband) and attenuation
c            (in the stopband) will be calculated using the
c            formulae given in l. r. rabiner (1973),
c            approximate design relationships for
c            lowpass digital filters, ieee trans. on audio
c            and electroacoustics, vol. au-21, no.5
c    lgrid - the density of the interpolating grid. the default
c            value is 16, which will be used if lgrid.le.0.
c    edge -  the edges of each band, given in units such that the
c            nyquist is .5, and in increasing order.
c    fx   -  desired value (or desired slope if a differentiator)
c            for each band.
c    wtx  -  the weight function for each band.  for a differentiator,
c            this weight function is inversely proportional to f.
c
c  the outputs are:
c
c    nfilt - if this has been put in as zero, it will be returned
c            with the filter length.
c    h     - the array of weights, given from left to center.
c            if nfilt is even, the nfilt/2 weights are 
c            returned; if it is odd, nfilt/2 + 1.
c    dev   - the deviation of the response from the one requested.
c
c
      common des,wt,alpha,iext,nfcns,ngrid,pi2,ad,dev,x,y,grid    
      common/wtsc/h,edge,fx,wtx,ipexp
      dimension iext(103),ad(103),alpha(103),x(103),y(103)      
      dimension h(103)   
      dimension des(1648),grid(1648),wt(1648)
C      dimension iext(66),ad(66),alpha(66),x(66),y(66)      
C      dimension h(66)   
C      dimension des(1045),grid(1045),wt(1045)
      dimension edge(20),fx(10),wtx(10),deviat(10)  
      double precision pi2,pi  
      double precision ad,dev,x,y     
C      data pi/3.141592653589793d0/,nfmax/128/
      data pi/3.141592653589793d0/,nfmax/202/
c  
c     the program is set up for a maximum length of 128, but      
c     this upper limit can be changed by redimensioning the
c     arrays iext, ad, alpha, x, y, h to be nfmax/2 + 2.   
c     the arrays des, grid, and wt must dimensioned 
c     16(-fmax/2 + 2).  
c  
      if(nfilt.gt.nfmax.or.nfilt.eq.3) call error   
      if(nbands.le.0) nbands=1 
c  
c     grid density is assumed to be 16 unless specified    
c     otherwise  
c  
      if(lgrid.le.0) lgrid=16  
      jb=2*nbands
      if(jtype.eq.0) call error
      if(nfilt.eq.0) go to 10  
      go to 11   
c     section for finding filter length given ripple and attenuation
 10   ripple=10.**(ripple/20)   
      attn=10.**(-attn/20)      
      ripple= (ripple-1)/(ripple+1)    
      ripple=alog10(ripple)    
      attn=alog10(attn) 
      svrip=ripple      
      ripple=(0.005309*(ripple**2)+0.07114*ripple-0.4761)*attn    
     1+(-0.00266*(ripple**2)-0.5941*ripple-0.4278)  
      attn=11.01217+0.51244*svrip-0.51244*attn      
      deltaf=edge(3)-edge(2)   
      nfilt=ripple/deltaf-attn*(deltaf**2)+1 
c     force number of filter weights to be odd
      nfilt = 2*(nfilt/2) + 1
      if(nfilt.gt.nfmax) call error
   11 neg=1      
      if(jtype.eq.1) neg=0     
      nodd=nfilt/2      
      nodd=nfilt-2*nodd 
      nfcns=nfilt/2     
      if(nodd.eq.1.and.neg.eq.0) nfcns=nfcns+1      
c  
c     set up the dense grid.  the number of points in the grid    
c     is (filter length + 1)*grid density/2  
c  
      grid(1)=edge(1)   
      delf=lgrid*nfcns  
      delf=0.5/delf     
      if(neg.eq.0) go to 135   
      if(edge(1).lt.delf) grid(1)=delf
  135 continue   
      j=1 
      l=1 
      lband=1    
  140 fup=edge(l+1)     
  145 temp=grid(j)      
c  
c     calculate the desired magnitude response and the weight     
c     function on the grid     
c  
      des(j)=eff(temp,fx,wtx,lband,jtype)    
      wt(j)=wate(temp,fx,wtx,lband,jtype,ipexp)
      j=j+1      
      grid(j)=temp+delf 
      if(grid(j).gt.fup) go to 150    
      go to 145  
  150 grid(j-1)=fup     
      des(j-1)=eff(fup,fx,wtx,lband,jtype)   
      wt(j-1)=wate(fup,fx,wtx,lband,jtype,ipexp)   
      lband=lband+1     
      l=l+2      
      if(lband.gt.nbands) go to 160   
      grid(j)=edge(l)   
      go to 140  
  160 ngrid=j-1  
      if(neg.ne.nodd) go to 165
      if(grid(ngrid).gt.(0.5-delf)) ngrid=ngrid-1   
  165 continue   
c  
c     set up a new approximation problem which is equivalent      
c     to the original problem  
c  
      if(neg) 170,170,180      
  170 if(nodd.eq.1) go to 200  
      do 175 j=1,ngrid  
      change=dcos(pi*grid(j))  
      des(j)=des(j)/change     
  175 wt(j)=wt(j)*change
      go to 200  
  180 if(nodd.eq.1) go to 190  
      do 185 j=1,ngrid  
      change=dsin(pi*grid(j))  
      des(j)=des(j)/change     
  185 wt(j)=wt(j)*change
      go to 200  
  190 do 195 j=1,ngrid  
      change=dsin(pi2*grid(j)) 
      des(j)=des(j)/change     
  195 wt(j)=wt(j)*change
c  
c     initial guess for the extremal frequencies--equally  
c     spaced along the grid    
c  
  200 temp=float(ngrid-1)/float(nfcns)
      do 210 j=1,nfcns  
  210 iext(j)=(j-1)*temp+1     
      iext(nfcns+1)=ngrid      
      nm1=nfcns-1
      nz=nfcns+1 
c  
c     call the remez exchange algorithm to do the approximation   
c     problem    
c  
      call remez(edge,nbands)  
c  
c     calculate the impulse response. 
c  
      if(neg) 300,300,320      
  300 if(nodd.eq.0) go to 310  
      do 305 j=1,nm1    
  305 h(j)=0.5*alpha(nz-j)     
      h(nfcns)=alpha(1) 
      go to 350  
  310 h(1)=0.25*alpha(nfcns)   
      do 315 j=2,nm1    
  315 h(j)=0.25*(alpha(nz-j)+alpha(nfcns+2-j))      
      h(nfcns)=0.5*alpha(1)+0.25*alpha(2)    
      go to 350  
  320 if(nodd.eq.0) go to 330  
      h(1)=0.25*alpha(nfcns)   
      h(2)=0.25*alpha(nm1)     
      do 325 j=3,nm1    
  325 h(j)=0.25*(alpha(nz-j)-alpha(nfcns+3-j))      
      h(nfcns)=0.5*alpha(1)-0.25*alpha(3)    
      h(nz)=0.0  
      go to 350  
  330 h(1)=0.25*alpha(nfcns)   
      do 335 j=2,nm1    
  335 h(j)=0.25*(alpha(nz-j)-alpha(nfcns+2-j))      
      h(nfcns)=0.5*alpha(1)-0.25*alpha(2)    
  350 return
      end
      subroutine remez(edge,nbands)
c  
c     this subroutine implements the remez exchange algorithm     
c     for the weighted chebychev approximation of a continous     
c     function with a sum of cosines.  inputs to the subroutine   
c     are a dense grid which replaces the frequency axis, the     
c     desired function on this grid, the weight function on the   
c     grid, the number of cosines, and an initial guess of the    
c     extremal frequencies.  the program minimizes the chebychev  
c     error by determining the best location of the extremal      
c     frequencies (points of maximimum error) and then calculates   
c     the coefficients of the best approximation.   
c  
      common des,wt,alpha,iext,nfcns,ngrid,pi2,ad,dev,x,y,grid    
      dimension edge(20)
      dimension iext(103),ad(103),alpha(103),x(103),y(103)      
      dimension des(1648),grid(1648),wt(1648)
      dimension a(103),p(103),q(103)  
      double precision pi2,dnum,dden,dtemp,a,p,q  
      double precision ad,dev,x,y  
      double precision d,gee  
c  
c     the program allows a maximum number of iterations of 25     
c  
      itrmax=25  
      devl=-1.0  
      nz=nfcns+1 
      nzz=nfcns+2
      niter=0    
  100 continue   
      iext(nzz)=ngrid+1 
      niter=niter+1     
      if(niter.gt.itrmax) go to 400
      do 110 j=1,nz     
      k=iext(j)  
      dtemp=grid(k)     
      dtemp=dcos(dtemp*pi2)    
  110 x(j)=dtemp 
      jet=(nfcns-1)/15+1
      do 120 j=1,nz     
  120 ad(j)=d(j,nz,jet) 
      dnum=0.0   
      dden=0.0   
      k=1 
      do 130 j=1,nz     
      l=iext(j)  
      dtemp=ad(j)*des(l)
      dnum=dnum+dtemp   
      dtemp=k*ad(j)/wt(l)      
      dden=dden+dtemp   
  130 k=-k
      dev=dnum/dden     
      nu=1
      if(dev.gt.0.0) nu=-1     
      dev=-nu*dev
      k=nu
      do 140 j=1,nz     
      l=iext(j)  
      dtemp=k*dev/wt(l) 
      y(j)=des(l)+dtemp 
  140 k=-k
      write(6,141) dev,devl
 141  format(' dev=',f16.10,1x,'devl=',f16.10)  
      if(dev.ge.devl) go to 150
      call ouch  
      go to 400  
  150 devl=dev   
      jchnge=0   
      k1=iext(1) 
      knz=iext(nz)      
      klow=0     
      nut=-nu    
      j=1 
c  
c     search for the extremal frequencies of the best      
c     approximation     
c  
  200 if(j.eq.nzz) ynz=comp    
      if(j.ge.nzz) go to 300   
      kup=iext(j+1)     
      l=iext(j)+1
      nut=-nut   
      if(j.eq.2) y1=comp
      comp=dev   
      if(l.ge.kup) go to 220   
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.le.0.0) go to 220      
      comp=nut*err      
  210 l=l+1      
      if(l.ge.kup) go to 215   
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.le.0.0) go to 215      
      comp=nut*err      
      go to 210  
  215 iext(j)=l-1
      j=j+1      
      klow=l-1   
      jchnge=jchnge+1   
      go to 200  
  220 l=l-1      
  225 l=l-1      
      if(l.le.klow) go to 250  
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.gt.0.0) go to 230      
      if(jchnge.le.0) go to 225
      go to 260  
  230 comp=nut*err      
  235 l=l-1      
      if(l.le.klow) go to 240  
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.le.0.0) go to 240      
      comp=nut*err      
      go to 235  
  240 klow=iext(j)      
      iext(j)=l+1
      j=j+1      
      jchnge=jchnge+1   
      go to 200  
  250 l=iext(j)+1
      if(jchnge.gt.0) go to 215
  255 l=l+1      
      if(l.ge.kup) go to 260   
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.le.0.0) go to 255      
      comp=nut*err      
      go to 210  
  260 klow=iext(j)      
      j=j+1      
      go to 200  
  300 if(j.gt.nzz) go to 320   
      if(k1.gt.iext(1)) k1=iext(1)    
      if(knz.lt.iext(nz)) knz=iext(nz)  
      nut1=nut   
      nut=-nu    
      l=0 
      kup=k1     
      comp=ynz*(1.00001)
      luck=1     
  310 l=l+1      
      if(l.ge.kup) go to 315   
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.le.0.0) go to 310      
      comp=nut*err      
      j=nzz      
      go to 210  
  315 luck=6     
      go to 325  
  320 if(luck.gt.9) go to 350  
      if(comp.gt.y1) y1=comp   
      k1=iext(nzz)      
  325 l=ngrid+1  
      klow=knz   
      nut=-nut1  
      comp=y1*(1.00001) 
  330 l=l-1      
      if(l.le.klow) go to 340  
      err=gee(l,nz)     
      err=(err-des(l))*wt(l)   
      dtemp=nut*err-comp
      if(dtemp.le.0.0) go to 330      
      j=nzz      
      comp=nut*err      
      luck=luck+10      
      go to 235  
  340 if(luck.eq.6) go to 370  
      do 345 j=1,nfcns  
  345 iext(nzz-j)=iext(nz-j)   
      iext(1)=k1 
      go to 100  
  350 kn=iext(nzz)      
      do 360 j=1,nfcns  
  360 iext(j)=iext(j+1) 
      iext(nz)=kn
      go to 100  
  370 if(jchnge.gt.0) go to 100
c   calculation of the coefficients of the best approximation   
c     using the inverse discrete fourier transform  
  400 continue   
      nm1=nfcns-1
      fsh=1.0e-06
      gtemp=grid(1)     
      x(nzz)=-2.0
      cn=2*nfcns-1      
      delf=1.0/cn
      l=1 
      kkk=0      
      if(edge(1).eq.0.5.and.edge(2*nbands).eq.0.5) kkk=1   
      if(nfcns.le.3) kkk=1     
      if(kkk.eq.1) go to 405   
      dtemp=dcos(pi2*grid(1))  
      dnum=dcos(pi2*grid(ngrid))      
      aa=2.0/(dtemp-dnum)      
      bb=-(dtemp+dnum)/(dtemp-dnum)  
  405 continue   
      do 430 j=1,nfcns  
      ft=(j-1)*delf     
      xt=dcos(pi2*ft)   
      if(kkk.eq.1) go to 410   
      xt=(xt-bb)/aa     
      ft=acos(xt)/pi2   
  410 xe=x(l)    
      if(xt.gt.xe) go to 420   
      if((xe-xt).lt.fsh) go to 415  
      l=l+1      
      go to 410  
  415 a(j)=y(l)  
      go to 425  
  420 if((xt-xe).lt.fsh) go to 415  
      grid(1)=ft 
      a(j)=gee(1,nz)    
  425 continue   
      if(l.gt.1) l=l-1  
  430 continue   
      grid(1)=gtemp     
      dden=pi2/cn
      do 510 j=1,nfcns  
      dtemp=0.0  
      dnum=(j-1)*dden   
      if(nm1.lt.1) go to 505   
      do 500 k=1,nm1    
  500 dtemp=dtemp+a(k+1)*dcos(dnum*k)  
  505 dtemp=2.0*dtemp+a(1)     
  510 alpha(j)=dtemp    
      do 550 j=2,nfcns  
  550 alpha(j)=2*alpha(j)/cn   
      alpha(1)=alpha(1)/cn     
      if(kkk.eq.1) go to 545   
      p(1)=2.0*alpha(nfcns)*bb+alpha(nm1)  
      p(2)=2.0*aa*alpha(nfcns) 
      q(1)=alpha(nfcns-2)-alpha(nfcns)  
      do 540 j=2,nm1    
      if(j.lt.nm1) go to 515   
      aa=0.5*aa  
      bb=0.5*bb  
  515 continue   
      p(j+1)=0.0 
      do 520 k=1,j      
      a(k)=p(k)  
  520 p(k)=2.0*bb*a(k)  
      p(2)=p(2)+a(1)*2.0*aa    
      jm1=j-1    
      do 525 k=1,jm1    
  525 p(k)=p(k)+q(k)+aa*a(k+1) 
      jp1=j+1    
      do 530 k=3,jp1    
  530 p(k)=p(k)+aa*a(k-1)      
      if(j.eq.nm1) go to 540   
      do 535 k=1,j      
  535 q(k)=-a(k) 
      q(1)=q(1)+alpha(nfcns-1-j)      
  540 continue   
      do 543 j=1,nfcns  
  543 alpha(j)=p(j)     
  545 continue   
      if(nfcns.gt.3) return    
      alpha(nfcns+1)=0.0
      alpha(nfcns+2)=0.0
      return     
      end 
      subroutine ouch   
      write(6,100)    
 100  format(' ************ failure to converge ************'/    
     1' probable cause is machine rounding error'/  
     2' the impulse response may be correct'   /
     3' check with a frequency response'  /)
      return    
      end
      subroutine error  
      write(6,100)
 100  format(' ************ error in input data **********')      
      stop
      end 
      double precision function d(k,n,m)
c     function to calculate the lagrange interpolation     
c     coefficients for use in the function gee.    
      common des,wt,alpha,iext,nfcns,ngrid,pi2,ad,dev,x,y,grid    
      dimension iext(103),ad(103),alpha(103),x(103),y(103)      
      dimension des(1648),grid(1648),wt(1648)   
      double precision ad,dev,x,y   
      double precision q
      double precision pi2     
      d=1.0      
      q=x(k)     
      do 3 l=1,m 
      do 2 j=l,n,m      
      if(j-k)1,2,1      
    1 d=2.0*d*(q-x(j))  
    2 continue   
    3 continue   
      d=1.0/d    
      return     
      end 
      double precision function gee(k,n)
c     function to evaluate the frequency response using the
c     lagrange interpolation formula in the barycentric form      
      common des,wt,alpha,iext,nfcns,ngrid,pi2,ad,dev,x,y,grid    
      dimension iext(103),ad(103),alpha(103),x(103),y(103)      
      dimension des(1648),grid(1648),wt(1648)  
      double precision p,c,d,xf
      double precision pi2     
      double precision ad,dev,x,y    
      p=0.0      
      xf=grid(k) 
      xf=dcos(pi2*xf)   
      d=0.0      
      do 1 j=1,n 
      c=xf-x(j)  
      c=ad(j)/c  
      d=d+c      
    1 p=p+c*y(j) 
      gee=p/d    
      return     
      end
c
c  magnitude and weighting function subroutines for the parks-
c  mcclellan filter program, kept separate so that other ones 
c  may be easily compiled and used.  an added feature is that 
c  if the weight is negative, the weight output is inversely
c  proportional to frequency, and if the functional value is
c  negative, a value is returned that is propotional to 
c  frequency. (this allows construction of whitening filters
c  for red noise).
c 
c 
      function eff(temp,fx,wtx,lband,jtype) 
c 
c  function to calculate the desired magnitude response 
c  as a function of frequency.
c 
      dimension fx(5),wtx(5)
      eff = fx(lband) 
      if(jtype.eq.2) eff = eff*temp 
      if(eff.lt.0) eff = -eff*temp
      return
      end 
      function wate(temp,fx,wtx,lband,jtype,ip) 
c  
c  function to calculate the weight function as a function
c  of frequency.
c 
      dimension fx(5),wtx(5)
      wate=wtx(lband) 
      if(jtype.eq.2.and.fx(lband).gt.0.0001) wate = wate/temp 
      if(wate.lt.0) wate = -wate*(temp**ip) 
      return
      end 
