c   program for computing the insolation. three options
c   are available:
c
c      1. insolation on a horizontal surface, uncorrected for
c         atmospheric absorption or scattering.  this
c         is identical to the 'radiational potential' of
c         munk and cartwright, except that it depends on the
c         square of the parallax.
c
c      2. insolation on an arbitrarily oriented surface, corrected
c         for atmospheric effects using the 'desert insolation'
c         formula in meinel & meinel, "applied solar energy".
c         insolation on the back of the surface is computed as
c         negative (except that at night it is of course zero).
c
c      3. the munk-cartwright "radiational potential" harmonic coefficients,
c         output in complex files
c
c   units of output are watts/m**2.
c
c   revised from the pdp-11 version of ertid by d. agnew, may-june 1979,
c                                                         april 1983
c
c     calls tidlib,realio
c
      character*1 kopt,not
      double precision tt,ts,te,d,zhr,yhr,t
      real iyr,jyr,iday,jday,lamda
      dimension serout(60,5),iget(5),aa(10),ispc(3),it1(5),it2(5)
      common/bpos/coor(8),par,sdist
      common/obs/ctheta,stheta,clg,slg,dif,radn,gl
      common/tdiff/etmut
      common/sunny/moon
      data irl/1/,iflag/0/,sk/1353./,moon/2/,dr/.01745329/,
     1 iget/5*0/,ispc/3*0/,ntotl/1/,ihed/0/
      write(6,701)
701   format(' start time- - -  (year,day,hour) [nnnh for header] ;',
     1 t45,$)
      iyr=ffin(1,not)
      if(not.eq.'h') then
         ihed=nint(iyr)
         call hedtim(ihed,it1,it2,irnt,samp)
         iyr=it1(1)-1900
         iday=it1(2)
         zhr=it1(3)
         jyr=it2(1)-1900
         jday=it2(2)
         yhr=it2(3)
         d=samp/3600.
      endif
      if(ihed.eq.0) then
        iday=ffin(1,not)
        zhr=ffin(1,not)
        iyr=iyr-1900
        write(6,702)
702     format(' end time- - -  (year,day,hour) ;',t45,$)
        jyr=ffin(1,not)
        jday=ffin(1,not)
        yhr=ffin(1,not)
        jyr=jyr-1900
        write(6,703)
703     format(' time interval (hours)-   ;  ',$)
        d=ffin(1,not)
      endif
c   find times in hours from 0 hr, 1 jan 1900
      ts=zhr+24.d0*(iday-1)+8760.d0*iyr+24.d0*int((iyr-1)/4)
      te=yhr+24.d0*(jday-1)+8760.d0*jyr+24.d0*int((jyr-1)/4)
      terms=(te-ts)/d
      write(6,7035)
7035  format(' type i for insolation, r for radpot; ',$)
      idum = ffin(1,kopt)
      if(kopt.eq.'i') go to 9
 6    write(6,7036)
 7036 format(' type 1 for order 1, 2 for 2, 3 for both ',$)
      iord=ffin(1,not)
      if(iord.lt.1.or.iord.gt.3) go to 6
      nspc = min0(iord+1,3)
      write(6,7037)
 7037 format(' type 1 to get long-period tides ',$)
      ispc(1) = ffin(1,not)
      write(6,7038)
 7038 format(' type 1 to get diurnal tides ',$)
      ispc(2) = ffin(1,not)
      if(nspc.eq.3) write(6,7039)
 7039 format(' type 1 to get semidiurnal tides ',$)
      if(nspc.eq.3) ispc(3) = ffin(1,not)
      do 20 i=1,nspc
      if(ispc(i).le.0) go to 20
      if((iord.eq.1.or.iord.eq.3).and.i.le.2) iget(i)=1
      if(iord.ge.2) iget(i+2) = 1
 20   continue
      ntotl = 0
      do 21 i=1,5
 21   if(iget(i).eq.1) ntotl=ntotl+1
      go to 741
 9    write(6,704)
704   format(' north latitude-  ;   ',$)
      theta=ffin(1,not)
      write(6,705)
705   format(' east longitude-  ;   ',$)
      lamda=ffin(1,not)
 33   write(6,100)
 100  format(' option wanted (-1 for information):; ',$)
      iopt = ffin(1,not)
      if(iopt.ge.0) go to 55
      write(6,102)
 102  format(' type 0 to get the usual radiational potential',/
     1 ,' type 1 to get desert insolation on an arbitrary surface,'
     2,/,' corrected for atmospheric effects')
      go to 33
 55   if(iopt.eq.0) go to 741
      write(6,104)
 104  format(' angle of surface to the horizontal; ',$)
      sth = ffin(1,not)
      cth = cos(dr*sth)
      sth = sin(dr*sth)
      write(6,106)
 106  format(' azimuth of surface; ',$)
      a = ffin(1,not)
      sa = sin(dr*a)
      ca = cos(dr*a)
      write(6,108)
 108  format(' altitude (meters); ',$)
      a = ffin(1,not)
      a = a/1000.
      t1 = sk*(1.-.14*a)
      t2 = sk*.14*a
 741  write(6,709)
 709  format(' number of first output file; ',$)
      ifil = ffin(1,not)
      ifil = iabs(ifil)
c
c                  begin execution
c
      i = 1
      tt = ts
      call sph(theta,lamda,a)
      etmut = 41.184 + iyr - 70
c   begin loop through times
 10   if(tt.gt.te) go to 749
      t = (tt+12.d0 + (etmut/3600.d0))/876600.d0
c   t is ephemeris time in julian centuries from 12 hr 0 jan 1900
      call ephem(t)
      if(kopt.eq.'r') go to 80
c    calculate cosine of zenith angle
      cll = clg*coor(8) + slg*coor(7)
      sll = slg*coor(8) - clg*coor(7)
      sz = coor(5)
      cz = coor(6)
      cu = ctheta*coor(6) + stheta*coor(5)*cll
      rbarr = 1./sdist
      if(cu.le.0) go to 74
      serout(i,1) = sk*cu*rbarr**2
      if(iopt.eq.0) go to 3
c  option 1 - include atmospheric effects and geometrical for
c  arbitrarily oriented surface.
      rimm = (t1*exp(-.357*((1./cu)**.678)) + t2)*rbarr**2
      serout(i,1) = rimm*(cth*(ctheta*cz + stheta*sz*cll) +
     1 sth*ca*((stheta*sz*cll + cz*stheta**2)/stheta)
     2 + sa*sz*sll)
      go to 3
c   mu lt 0 means it is night
 74   serout(i,1) = 0.
      go to 3
 80   call mccor(aa,aa(6))
      idd = 0
      do 81 kk=1,5
      if(iget(kk).eq.0) go to 81
      idd = idd + 1
      serout(i,idd) = aa(kk)
      serout(i+1,idd) = aa(5+kk)
 81   continue
      i = i + 1
 3    i = i + 1
      tt = tt + d
      if(i.eq.61) go to 750
      go to 10
c   have reached end time.
749   iflag=1
      if(i.eq.1) go to 770
c
c      the following pertains to the use of an i/o program written
c      for pdp 11/40.
c
 750  irh = irl + i - 2
      do 751 kk =1,ntotl
      nnn = -(ifil+kk-1)
 751  call realio(serout(1,kk),irl,irh,nnn,1)
      irl = irh + 1
      i = 1
      if(iflag.eq.0) go to 10
770   write(6,771) irh
771   format(1x,i8,' terms in output ')
      if(kopt.eq.'r') go to 780
      stop
 780  idd = -ifil + 1
      do 781 i=1,5
      if(iget(i).eq.0) go to 781
      idd = idd - 1
      j = mod(i-1,5)
      iord = (j+1)/3 + 1
      nspc = j - 2*(iord/2)
      write(6,782) iord,nspc,idd
 782  format(' rad(',i1,',',i1,') in file ',i5)
 781  continue
      end
