c  program intercal
c
c  given a list of calendar times and values associated with these,
c  produces a time series of linearly interpolated data between these
c  times. The input file may also contain values for the slope (in units/day)
c  before and after the first and last dates, in case the series extends
c  beyond these
c
c  calls linter (niolib), realio (niolib), ffin (termlib), timsubs (genlib)
c
      character*1 not,mode,ished,type,inter
      character*80 dvfile,line
c  limited to 10000 values
      dimension isc(10000),val(10000),x(10000),s(10000),a(10000)
      dimension d(600)
      dimension it1(5),itc(5),itc2(5)
      data mode/'d'/
      data bslope/0./,aslope/0./
      write(6,100)
 100  format(' Start time of file to correct (nnnh for header); ',$)
      it1(1)=ffin(1,ished)
      if(ished.ne.'h') then
         do i=2,5
           it1(i)=ffin(1,not)
         enddo
         write(6,102)
 102     format(' Sample interval; ',$)
         samp=ffin(1,not)
         write(6,104)
 104     format(' File length; ',$)
         nterm = ffin(1,not)
      endif
      if(ished.eq.'h') then
         nhfl=it1(1)
         call hedtim(nhfl,it1,nterm,samp)
      endif
      isamp=samp
      write(6,106)
 106  format(' Number of file (real) for output series; ',$)
      ifo = ffin(1,not)
      write(6,108)
 108  format(' Name of file with date-value information; ',$)
      read(5,110) dvfile
      write(6,109)
 109  format(' Interpolation (l for linear, s for spline); ',$)
      idum = ffin(1,inter)
      lu=2
      open(lu,file=dvfile)
c  read in information, skipping lines with # as the first character
      nv=1
 9    read(lu,110,end=13) line
 110  format(a80)
      if(line(1:1).ne.' ') go to 9
c  read in times and convert them to sample numbers
      itc(5) = 0
      read(line,112) type
 112  format(1x,a1)
      if(type.ne.'a'.and.type.ne.'v'.and.type.ne.'b') then
          write(6,113) line,dvfile
 113      format('Line',/,a80,/,'from file',/,a80,/,'is unreadable',
     1    '  All lines must have data in the format ',/,
     2    ' b sssssssss',/,
     3    ' v yyyy ddd hh mm ss  xxxxxxxxx',/,
     4    ' a sssssssss',/,
     5    '  where xxx is a value, and sss is a slope (units/day)',/,
     6    '  or line must begin with a nonblank character')
          stop
      endif
      if(type.eq.'b') read(line,114) type,bslope
 114  format(1x,a1,1x,f9.0)
      if(type.eq.'a') read(line,114) type,aslope
      if(type.eq.'v') read(line,116) type,(itc(i),i=1,5),val(nv)
 116  format(1x,a1,1x,i4,i4,i3,i3,i3,2x,f9.0)
      if(type.eq.'v') then
        isc(nv) = iscnsp(it1,itc,isamp,mode) + 1
        if(isc(1).gt.nterm) then
          call timfnd(it1,nterm-1,isamp,itc2,mode)
          write(6,118) itc,itc2
 118      format('First value in ASCII file, at ',2i4,3i3,/,
     1           'falls after end of data, at   ',2i4,3i3,/,
     2           '  Program quits')
          stop
        endif
        if(nv.gt.1.and.isc(nv).lt.isc(nv-1)) then
          write(6,119) itc
 119      format('Value in ASCII file, at ',2i4,3i3,/,
     1           ' is out of chronological order -- Program quits')
          stop
        endif
c  if we are going to a large sample interval, the values may
c  fall on the same point--separate them by moving the last point back
        if(isc(nv).eq.isc(nv-1)) isc(nv-1)=isc(nv-1)-1
        nv=nv+1
        if(nv.eq.10001) then
          write(6,120) dvfile
 120      format('Too many values in',/,a80,/,' program quits')
        endif
      endif
      go to 9
 13   nv=nv-1
      if(isc(nv).lt.1) then
        call timfnd(it1,isc(nv)-1,isamp,itc2,mode)
        write(6,122) itc,it1
 122    format('  Last value in ASCII file, at ',2i4,3i3,/,
     1         'falls before start of data, at ',2i4,3i3,/,
     2         '  Program quits')
        stop
       endif
c convert slopes to value/sample
      bslope=bslope*(samp/86400.)
      aslope=aslope*(samp/86400.)
c  if necessary, create imputed values at start and end of series,
c   using the slopes
      if(isc(1).gt.1) then
         valimp=val(1)-bslope*(isc(1)-1)
         do i=nv,1,-1
           val(i+1)=val(i)
           isc(i+1)=isc(i)
         enddo
         val(1)=valimp
         isc(1)=1
         indst=1
         nv=nv+1
      endif
      if(isc(nv).le.nterm) then
         val(nv+1)=val(nv)+aslope*(nterm-isc(nv))
         isc(nv+1)=nterm
         nv=nv+1
         inden=nv
      endif
c  if we will be doing linear inerpolation, create imputed values at
c  start and end of series, using the values given "outside" the time
c  span of the series (not needed for spline fit)
      if(inter.eq.'l') then
        ic=1
 17     if(isc(ic).lt.1.and.isc(ic+1).gt.1) then
           slope=(val(ic+1)-val(ic))/(isc(ic+1)-isc(ic))
           valimp=val(ic+1)-slope*(isc(ic+1)-1)
           do i=nv,ic+1,-1
             val(i+1)=val(i)
             isc(i+1)=isc(i)
           enddo
           val(ic+1)=valimp
           isc(ic+1)=1
           indst=ic+1
           nv=nv+1
        endif
        if(isc(ic).lt.nterm.and.isc(ic+1).gt.nterm) then
           slope=(val(ic+1)-val(ic))/(isc(ic+1)-isc(ic))
           valimp=val(ic+1)-slope*(isc(ic+1)-nterm)
           do i=nv,ic+1,-1
             val(i+1)=val(i)
             isc(i+1)=isc(i)
           enddo
           val(ic+1)=valimp
           isc(ic+1)=nterm
           inden=ic+1
           nv=nv+1
        endif
        ic=ic+1
        if(ic.lt.nv) go to 17
c  now write these specific values to a file
        do ic=indst,inden
          call realio(val(ic),isc(ic),isc(ic),ifo,1)
        enddo
c  and linearly interpolate between them
        do ic=indst,inden-1
          call linter(ifo,isc(ic),isc(ic+1))
        enddo
c
c  write out slopes
c
        do ic=indst,inden-1
           slope=(val(ic+1)-val(ic))/(isc(ic+1)-isc(ic))
           slope= slope*(86400/samp)
           call timfnd(it1,isc(ic)-1,isamp,itc,mode)
           call timfnd(it1,isc(ic+1)-1,isamp,itc2,mode)
           write(6,130) (itc(i),i=1,5),(itc2(i),i=1,5),slope
 130       format('From ',i4,i4,3i3,' to ',i4,i4,3i3,
     1          ' slope is ',g12.4,' units/day')
        enddo
      endif
      if(inter.eq.'s') then
        do l=1,nv
          x(l)=isc(l)
        enddo
        call spline(nv,x,val,s,a)
        irl = 1
 19     irh = min(irl+599,nterm)
        np = irh - irl + 1
        do i = 1,np
          y=irl+i-1.
          d(i) = eval(y,nv,x,val,s)
        enddo
        call realio(d,irl,irh,ifo,1)
        irl = irh + 1
        if(irl.le.nterm) go to 19
      endif
      stop
      end
      subroutine hedtim(nfl,ist,irnt,samp)
      character*8 source,chanid
      character*4 stc,commen
      double precision scans
      dimension ist(5)
      common/head/scans,calc(4),clkc(2),sampin,nchann,iblock,istart(5),
     1     iend(5),nchf(17),ichnm(16),ncal(4),source,chanid(16),stc,
     2     commen(200)
      call headco(nfl,1)
      do 3 i=1,5
 3    ist(i) = istart(i)
      samp = sampin
      irnt=nint(scans)
      return
      end
