c
c   program minlowf  -- lowpass 1-2 days of 1-Hz data to produce one day
c     or less of 300-s data, using the minimum-phase lowpass-and-decimate
c     filter method
c
c
      character*70 dumm
      character*70 filfil
      character*8 source,chanid
      character*4 stc,commen
      character*1 mode
      double precision scans
      dimension its(5),ith1(5),ith2(5)
      dimension ich1(16),ich2(16)
      dimension x(100000),y(100000),wts(100),ix(100000)
c  
c   arrays of filter weights and decimation factors (up to 9 allowed)
c
      dimension fw(9,100),nw(9),nd(9)
      common/head/scans,calc(4),clkc(2),sampin,nchann,iblock,istart(5),
     1     iend(5),nchf(17),ichnm(16),ncal(4),source,chanid(16),stc,
     2     commen(200)
      data luf/1/,mode/'d'/,len1/0/,kst/0/
      data sc/1./
      if(iargc().lt.9) then
         write(6,100)
 100     format('Usage: minlowf hed1 hed2 hed3 y d h m s filfil [sc]',/,
     1  'where  hed1 and hed2 are header files for the previous and',
     2  ' current days',/,
     3'          (hed1=0 if there is no data)',/,
     4  '       hed3 is the header file for the decimated data',/,
     5  '       y d h m s is the time for the first output point',/,
     6  '           (if this cannot be produced, the next one that',/,
     7  '            is a multiple of the new interval will be used',/,
     8  '       filfil is the ASCII file giving all the filters and ',
     9          'decimation factors',/,
     9  '       sc is the factor to scale the output by (default 1)',/,
     9  '         (if >1 output files will be long int;',/,
     9  '          if <0 output files will be real, unscaled)')
         stop
      endif
c
c  get the information from the command line
c
      call getarg(1,dumm)
      read(dumm,102) ih1
 102  format(i6)
      idum = longis(ih1)
      if(idum.le.0) ih1=0
      call getarg(2,dumm)
      read(dumm,102) ih2
      call getarg(3,dumm)
      read(dumm,102) ih3
      call getarg(4,dumm)
      read(dumm,102) its(1)
      call getarg(5,dumm)
      read(dumm,102) its(2)
      call getarg(6,dumm)
      read(dumm,102) its(3)
      call getarg(7,dumm)
      read(dumm,102) its(4)
      call getarg(8,dumm)
      read(dumm,102) its(5)
      call getarg(9,filfil)
      if(iargc().gt.9) then
        call getarg(10,dumm)
        read(dumm,*) sc
      endif
c
c  get information from the first two headers
c
      if(ih1.ne.0) then
        call headco(ih1,1)
        do i=1,5
          ith1(i)=istart(i)
        enddo
        len1=nint(scans)
        isam1=nint(sampin)
        nch1=nchann
        do i=1,nchann
          ich1(i)=nchf(i)
        enddo
      endif
c
      call headco(ih2,1)
      do i=1,5
        ith2(i)=istart(i)
      enddo
      len2=nint(scans)
      isam2=nint(sampin)
      nch2=nchann
      do i=1,nchann
        ich2(i)=nchf(i)
      enddo
c
c  check for consistency
c
      if(ih1.ne.0.and.isam1.ne.isam2) then
         write(6,104) isam1,isam2
 104     format(' Different sample intervals (',i5,' and ',i5)
         stop
      endif
      if(ih1.ne.0.and.nch1.ne.nch2) then
         write(6,106) nch1,nch2
 106     format(' Different numbers of chans (',i5,' and ',i5)
         stop
      endif
c
c read the filter file
c
      open(luf,file=filfil)
      nstage=1
 5    read(luf,102,end=7) ndt
      nd(nstage)=ndt
      read(luf,102) nw(nstage)
      do i=1,nw(nstage)
        read(luf,108) fw(nstage,i)
 108    format(f9.8)
      enddo
      nstage=nstage+1
      go to 5
 7    nstage=nstage-1
c
c   figure out the total decimation and the offset
c
      ndec=1
      npad=0
      do i=1,nstage
        npad=npad+ndec*(nw(i)-1)
        ndec=ndec*nd(i)
      enddo
c
c   make decisions about where to start in the series, after finding the
c    locations (relative to the requested start time) of the start and end
c    of both data files; test for a gap between them
c
c  find filter start point relative to start points of each section
c   if the previous day file is missing, or the start point would
c   fall after the end of it, set this to -1 as a flag
      if(ih1.eq.0) ifst1=-1
      if(ih1.ne.0) then
          ifst1=iscnsp(ith1,its,isam1,mode)-npad+1
          if(ifst1.gt.len1) ifst1=-1
      endif
      ifst2=iscnsp(ith2,its,isam2,mode)-npad+1
c  case in which earlier file ends too soon, and start time is before
c  start of later file--adjust the start time
      if(ifst2.lt.1.and.ifst1.lt.1) then
        nst=-ifst2/ndec+1
        ifst2=ifst2+nst*ndec
        kst=ifst2+npad
        call timfnd(ith2,kst-1,isam2,its,mode)
      endif
c
c   perform the filtering
c
      do k=1,nch2
        call getx(ifst1,len1,ifst2,len2,ich1(k),ich2(k),x,n)
        do i=1,nstage
          do l=1,nw(i)
            wts(l)=fw(i,l)
          enddo
          call conv(x,y,n,nd(i),nw(i),wts)
          do l=1,n
            x(l) = y(l)
          enddo
        enddo
        if(sc.lt.0.and.ich2(k).gt.0) ifout=-(ich2(k)+ih3-ih2)
        if(sc.lt.0.and.ich2(k).lt.0) ifout=ich2(k)+ih2-ih3
        if(sc.gt.0) ifout=ich2(k)+ih3-ih2
        if(sc.gt.1) ifout=ifout+1000
        do l=1,n
          ix(l)=nint(sc*x(l))
        enddo
        if(sc.gt.0) call intio(ix,1,n,ifout,1)
        if(sc.lt.0) call realio(x,1,n,ifout,1)
        nchf(k)=ifout
        call retnio(ifout)
      enddo
      scans=n
      isam2=isam2*ndec
      sampin=isam2
      do l=1,5
        istart(l)=its(l)
      enddo
      call timfnd(istart,n-1,isam2,iend,mode)
      nchf(17)=ih3
      call headco(ih3,0)
      stop
      end
      subroutine conv(xin,xout,n,nd,nw,wts)
c
c  convolves nw weights in weights with n terms in xin, decimating by nd,
c  with results in xout--n is returned as the number there. The decimation
c  is assumed to start with term 1 of xin
c
      dimension xin(*),xout(*),wts(*)
      if(n.lt.nw) then
        write(6,100) n,nw
 100    format(' Series with ',i7,' terms is less than filter of ',i5,
     1  /,' ***Program quits')
        stop
      endif
      nout=(n-nw)/nd+1
      do j=1,nout
        xout(j)=0.
        ist=1+nd*(j-1)
        do l=1,nw
          xout(j) = xout(j) + wts(nw-l+1)*xin(l+ist-1)
        enddo
      enddo
      n=nout
      return
      end
      subroutine getx(is1,l1,is2,l2,ic1,ic2,x,n)
c
c    gets data from files numbered ic1 and ic2, with start terms for the
c    filtering of is1 and is2, and total numbers of terms l1 and l2. Puts
c    the data into x, returning the total length as n
c
      dimension x(*)
      n1=0
c usual case: start is in previous file, before it ends
      if(is1.ge.1) then
         call realio(x,is1,l1,ic1,0)
         n1=l1-is1+1
         n2=2-is2
         call realio(x(n2),1,l2,ic2,0)
         call retnio(ic1)
         call retnio(ic2)
         n=n2+l2-1
c interpolate if we need to
         if(n2.ne.n1+1) then
           ni=n2-n1
           dy=(x(n2)-x(n1))/ni
           do i=n1+1,n2-1
             x(i)=x(n1)+dy*(i-n1)
           enddo
         endif
      endif
c next case: start is in current file (is1 was set negative to flag)
      if(is1.le.0) then
         call realio(x,is2,l2,ic2,0)
         n=l2-is2+1
         call retnio(ic1)
      endif
      return
      end
