       program mulsum
c
c mulsum stands for sum of multiple scaled inputs.
c mulsum will take in up to 10 data files, scale each one then add
c them up point by point and put the result in an output file.
c the length of this output file is determined from the entries of
c the first input file.
c the start term of all input files does not have to be 1, as long
c as the user can be certain that the rest of the data file is at
c least as long as the length of output file, and this is calculated
c from the start and end term of the first input file. this length 
c requirement does, of course, apply to inputs with start term of 1's.
c
c about the program :
c mulsum does it by looping through all inputs length/600 times, picking
c out the same # of terms (at most 600 terms) from each file, dependent
c of its start term, scaling it and adding it to an accumulated array.
c at the end of each loop an offset value is added to this array before
c it is written to the output file.
c since only 2 files are open at any given time, the number of input files
c can be increased significantly without causing the program to crash.
c
c
c            feb-16-1983  nguyen-phan-trung
c
c
      character*1 not
      dimension sum(600),tem(600),item(600),ifi(10),ist(10),iet(10),
     * sfac(10)
      data iterm2/0/
60    write(6,500)
500   format(' # of input files (10 maximum) ; ',$)
      nfi=ffin(1,not)
      if(nfi.gt.10.or.nfi.le.0)go to 60
      write(6,520)
520   format(' file #,start term, end term, and scale factor of'
     *,' file 1 ; ',$)
      ifi(1)=ffin(1,not)
      ist(1)=ffin(1,not)
      iet(1)=ffin(1,not)
      if(iet(1).le.ist(1)) iet(1) = longis(ifi(1))
      sfac(1)=ffin(1,not)
      nlen=iet(1)-ist(1)+1
      if(nfi.eq.1)go to 80
      do 70 j=2,nfi
      write(6,540)j
540   format(' file #, start term and scale factor of input # ',i2,
     *' ; ',$)
      ifi(j)=ffin(1,not)
      ist(j)=ffin(1,not)
      iet(j)=ist(j)+nlen-1
      sfac(j)=ffin(1,not)
70    continue
80    write(6,560)
560   format(' ouput file # ; ',$)
      ioufi=ffin(1,not)
      do 90 j=1,nfi
      if(ioufi.eq.ifi(j))write(6,580)
580   format(' output must be different from input.')
      if(ioufi.eq.ifi(j))go to 80
90    continue
      write(6,600)
600   format(' offset of output file ; ',$)
      offset=ffin(1,not)
c     nrec=nint((filen+59.0)/60.0)
      if(ioufi.lt.0)call realio(0.0,1,1,ioufi,1)
      if(ioufi.gt.0)call intio(0,1,1,ioufi,1)
c idrl and idrh are control variables.
      idrl=ist(1)
100   idrh=min(idrl+599,iet(1))
      length=idrh-idrl+1.0
c clearing the accumulated array before each loop.
      do 110 j=1,length
      sum(j)=0.0
110   continue
c looping through all inputs.
      do 150 i=1,nfi
      irl=ist(i)
      irh=min0(irl+599,iet(i))
      length=irh-irl+1
      call realio(tem,irl,irh,ifi(i),0)
      call retnio(ifi(i))
c scaling input and adding it to the accumulated array.
      do 140 j=1,length
      tem(j)=tem(j)*sfac(i)
      sum(j)=sum(j)+tem(j)
c integer overflow check.
      if(abs(tem(j)).gt.2147483647.0.and.ioufi.gt.0.or.
     *   abs(sum(j)).gt.2147483647.0.and.ioufi.gt.0)go to 190
140   continue
c updating the start term.
      ist(i)=irh+1
150   continue
      do 160 j=1,length
      sum(j)=sum(j)+offset
160   continue
c writting to output file.
      iterm1=iterm2+1
      iterm2=iterm1+length-1
      if(ioufi.lt.0)call realio(sum,iterm1,iterm2,ioufi,-1)
      if(ioufi.lt.0)go to 180
      do 170 j=1,length
      item(j)=nint(sum(j))
170   continue
      call intio(item,iterm1,iterm2,ioufi,-1)
c checking control variables for new loop.
180   idrl=idrh+1
      if(idrl.le.iet(1))go to 100
c mulsum is done.
      write(6,620)ioufi,nlen
620   format(' mulsum completed, ouput file ',i7,' has ',i8,' terms.')
      stop
c printing out integer overflow error message.
190   write(6,640)irl+j-1.0,ifi(i),ioufi,iterm2
640   format(' *** term ',f7.0,' of file ',i7,' will cause integer
     * overflow.',/,5x,'(output will be out of range +-2147483647)',/,
     *5x,'mulsum output file ',i7,' only has ',i8,' terms.')
      stop
      end
