          program pcorr
c
c    using pole-zero expression for transfer function, generates corrections
c    needed to correct a power spectrum from least counts squared per
c    bandwidth, to physical units squared per hertz. corrections may be
c    made to an existing file or output as a separate file.
c     an option also allows correcting a power spectrum for first-
c    differencing done in the time domain.
c
c***************************************************************
c
c    order of pole-zero data on a disk file:
c        leading (normalization) coefficient
c        number of poles
c        number of zeroes
c        zeroes, given as alternating real and imaginary parts, and
c                only one of a complex conjugate pair included
c        poles, given as for zeroes
c
c************************************************************
c
c     ** calls pzresp, iolib
c
      complex poles,zeroes,resp
      character*1 more,kind
      dimension cor(4002),deg(3),poles(20),zeroes(20)
      pcor(d1,d2) = -10.*alog10((d1**2)*(d2))
      data np,nz/0,0/,kind/'a'/
      write(6,98)
 98   format(' option (0 for transfer function, ',
     $ '1 for differencing); ',$)
      ifl = ffin(1,not)
      if(ifl.eq.1) go to 300
      write(6,100)
 100  format(' number of pole-zero file (0 if none); ',$)
      ifile = ffin(1,not)
      if(ifile.eq.0) write(6,101)
 101  format(' gain in lc/(physical unit), or if < 0, assumed',
     1 ' to be value of lc; ',$)
      if(ifile.eq.0) go to 3
      call realio(deg,1,3,ifile,0)
      cff = deg(1)
      nz = deg(2)
      np = deg(3)
      call realio(zeroes,4,int(3+2*deg(2)),ifile,0)
      call realio(poles,int(4+2*deg(2)),int(3+2*deg(2)+2*deg(3)),
     $ ifile,0)
      call retnio(ifile)
      write(6,104)
 104  format(' normalizing frequency, in hz ',
     $  '(.01 for chz, .15955 for rads/sec); ',$)
      ff = ffin(1,not)
      write(6,105)
 105  format(1x,'absolute gain (in counts/whatever) at frequency',/,
     1  ' for which polynomial gives a gain of one: ',/,
     2 ' (if <0, will be taken to be value of l.c.); ',$)
 3    dc = ffin(1,not)
      if(dc.eq.0) stop
      if(dc.lt.0) dc = -1./dc
      write(6,106)
 106  format(1x,'length of section in transform; ',$)
      n = ffin(1,not)
      npt = n/2 + 1
      write(6,107)
 107  format(1x,'sample interval in seconds; ',$)
      dt = ffin(1,not)
      cf = 10.*alog10(2.*dt)
 5    write(6,108)
 108  format(' file to be corrected (0 to get corrections ',
     $ 'separately, -1 for freq list); ',$)
      ifl = ffin(1,not)
      if(ifl.eq.-1) then
 7      write(6,120)
 120    format(' Frequency in Hz; ',$)
        f=ffin(1,not)
        if(f.lt.0) stop
        call pzresp(kind,f,ff,np,poles,nz,zeroes,cff,resp,del)
         amp = cabs(resp)**2 +1.e-20
         c = cf + pcor(dc,amp)
         write(6,122) f,c
 122     format('fr ',f12.7,' corr ',f8.2,' dB')
        go to 7
      endif
      if(ifl.eq.0.and.np.eq.0.and.nz.eq.0) go to 140
      write(6,109)
 109  format(' output file; ',$)
      if2 = ffin(1,not)
c    correction factor to go to db relative to lc**2/hz
 140  continue
      if(ifl.ne.0) go to 200
c   generate output file of corrections - if there is no frequency variation
c   simply print out the correction
      if(np.eq.0.and.nz.eq.0) go to 160
      do 150 i = 1,npt
      f = (i-1.)/(n*dt)
      call pzresp(kind,f,ff,np,poles,nz,zeroes,cff,resp,del)
      amp = cabs(resp)**2 +1.e-20
 150  cor(i) = cf + pcor(dc,amp)
      call realio(cor,1,npt, if2,1)
      stop
 160  f = cf + pcor(dc,1.)
      write(6,111) f
 111  format(' add ',f9.2,' to the spectrum')
      stop
c   correct an existing file
 200  call realio(cor,1,npt,ifl,0)
      if(np.eq.0.and.nz.eq.0) go to 260
      do 250 i = 1,npt
      f = (i-1.)/(n*dt)
      call pzresp(kind,f,ff,np,poles,nz,zeroes,cff,resp,del)
      amp = cabs(resp)**2 + 1.e-20
 250  cor(i) = cor(i) + cf + pcor(dc,amp)
      call realio(cor,1,npt,if2,1)
      call retnio(if1)
      call retnio(if2)
      write(6,114)
 114  format(' do another file?; ',$)
      idum = ffin(1,more)
      if (more .eq. 'y') goto 5
      stop
 260  cf = cf + pcor(dc,1.)
      do 270 i=1,npt
 270  cor(i) = cor(i) + cf
      call realio(cor,1,npt,if2,1)
      stop
c  correct for differencing, making zero-frequency term 
c  equal to the next higher one
 300  write(6,106)
      n = ffin(1,not)
      npt = n/2 + 1
      write(6,113)
 113  format(' file to be corrected; ',$)
      ifl = ffin(1,not)
      write(6,109)
      if2 = ffin(1,not)
      call realio(cor,1,npt,ifl,0)
      cf = 3.14159265/n
      do 303 i = 2,npt
 303  cor(i) = cor(i) - 20.*alog10(2.*sin((i-1)*cf))
      cor(1) = cor(2)
      call realio(cor,1,npt,if2,1)
      end

