c     program  power
c
c      finds power spectrum by forming periodograms of short parts of a
c      series and then averaging them.  because of limited memory,
c      stores accumulated sum on disk as each periodogram is found.
c
c
c         see IEEE Trans. Aud. Electroac. AU-15 no. 2 p. 70 (1967)
c
c     calls fft, realtr, nfac, dtren, hann, prol, iolib
c
      character*1 not,iflg
      dimension x(20004),wwt(4)
      alog12(dfg) = 10.*alog10(abs(dfg) + 1.e-38)
      wind(j,l) = 1. - abs((2.*j-l-1.)/(l+1.))
      data wwt/1.,.3334,1.50025,1./,pi/3.1415926535/
      write(6,100)
 100  format(' Number of data file; ',$)
      inp = ffin(1,not)
      write(6,102)
 102  format(' Locations of first and last points ; ',$)
      irlow = ffin(1,not)
      irhi = ffin(1,not)
      if(irhi.lt.0) irhi=longis(inp)
 1    write(6,104)
 104  format(' Length of sections (le 20000) ; ',$)
      leng = ffin(1,not)
      leng = iabs(leng)
      lenok = 2*nfac(leng/2)
      if(leng.eq.lenok) go to 3
      write(6,105) leng,lenok
 105  format(' A length of',i5,' is unacceptable. The next smallest',
     1 ' acceptable length is',i5)
      go to 1
 3    write(6,106)
 106  format(' Remove mean and trend from each section? (y/n); ',$)
      dfg = ffin(1,iflg)
      if(iflg.ne.'y'.and.iflg.ne.'n') go to 3
 5    write(6,108)
 108  format(' Type of windowing (-1 for more information); ',$)
      itw = ffin(1,not)
      if ((itw.gt.0.and.itw.le.6).or.(itw.eq.0.and.not.eq.'0'))
     1  go to 7
      write(6,110)
 110  format(' Available windows are ',/,3x,'0 - none',/,3x,
     1    '1 - triangular taper',/,3x,'2 - 1 + cos (hann) taper'
     2  ,/,3x,'3 - 4pi prolate spheroidal taper',/,3x,
     3  '4 - triangular, 50% overlap',/,3x,'5 - hann, ',
     4  '50% overlap',/,3x,'6 - 4pi prolate, 70%',
     5  ' overlap',/)
      go to 5
 7    iol = (itw/4) + 1
      if(itw.gt.3) itw = itw - 3
c      compute sum of squares of window weights
      wn = wwt(itw+1)*leng
c      zero fill disk file
 9    write(6,112)
 112  format(' Number of output file; ',$)
      iout = ffin(1,not)
      do 11 i = 1,leng
 11   x(i) = 0.
      call realio(x,1,leng/2+1,iout,1)
      nsec = 0
      incr = leng/iol
      if(iol.eq.2.and.itw.eq.3) incr = .3*leng
c   incr is the amount we move along the series each time
      irl = irlow
      irh = irl + leng - 1
      if(irh.gt.irhi) stop
 13   call realio(x,irl,irh,inp,0)
      if(iflg.eq.'y') call dtren(x,leng)
      if (itw.ne.1) go to 17
      do 15 i = 1,leng
 15   x(i) = x(i)*wind(i,leng)
 17   if(itw.eq.2) call hann(x,leng)
      if(itw.eq.3) call prol(x,leng)
      call fft(x,x(2),leng/2,leng/2,leng/2,2,ierr)
      if(ierr.ne.0) stop
      call realtr(x,x(2),leng/2,2)
c  sum squares of cos and sin coefficients and pack result into lower half
c  of array x, normalizing by window factor and dividing by 4 to get
c  fourier coefficients
c      put results of previous summing in upper half(from disk) and
c  add the two.
      do 19 i = 1,leng+1,2
 19   x(i) = (x(i)**2 + x(i+1)**2)/(4.*wn)
      do 21 i = 1,(leng/2) + 1
 21   x(i) = x(2*i - 1)
      call realio(x(leng/2 +2),1,leng/2+1,iout,0)
      do 23 i = 1,leng/2 + 1
 23   x(i) = x(i) + x(leng/2+1+i)
      nsec = nsec + 1
      irl = irlow + incr*nsec
      irh = irl + leng - 1
      if(irh.gt.irhi) go to 25
      call realio(x,1,leng/2+1,iout,-1)
      go to 13
c  average and convert to db
 25   cf = pi/leng
      do 27 i = 1,leng/2 + 1
 27   x(i) = alog12(x(i)/nsec)
      call realio(x,1,leng/2+1,iout,-1)
      if(itw.eq.3) go to 29
c   compute 95 percent comfidence limits for single frequency estimates
c      deg is number of degrees of freedom.  expressions used are
c   modified approximations to inverse chi-square
      deg = 2.*((irhi-irlow+1)/leng)
      if (iol.eq.2) deg = 1.4*deg
      ep = 2./(9.*deg)
      chi = -30.*alog10(1.-ep-(1.96-.243*ep)*sqrt(ep)) -
     1    (2.38/((deg-1.)**2))
      clo = -30.*alog10(1.-ep + (1.96+.054*ep)*sqrt(ep))
      write(6,114) deg, chi, clo
 114  format(1x,f4.0,' degrees of freedom.  95% confidence limits are',
     1    f5.1,' and',f5.1,' db.')
 29   write(6,116) leng/2 + 1, iout
 116  format(i7,' points in file ',i7)
      stop
      end
