c  program runmed
c   does a running median over a series.  the beginning is padded
c  with an appropriate number of values so that the output and
c  input will line up.
c
c      $$$calls perflt (in same source),iolib
c
      character*1 not
      dimension ix(4500),x(4500)
      data irlo/1./ifrst/0/
      write(6,100)
 100  format(' Input file, start and end terms; ',$)
      ifi = ffin(1,not)
      irli = ffin(1,not)
      irnt = ffin(1,not)
      if(irnt.le.0) irnt=longis(ifi)
      write(6,102)
 102  format(' Output file; ',$)
      ifo = ffin(1,not)
      write(6,104)
 104  format(' Block length (odd, le 1001); ',$)
      med = ffin(1,not)
      khf = med/2
c  khf is number of values to pad (for first buffer)
 1    write(6,106)
 106  format(' Averaging method (1-5, h for help); ',$)
      imeth = ffin(1,not)
      if(imeth.ge.1.and.imeth.le.5) go to 3
      write(6,108)
 108  format(' methods are:',/,
     1 '  1 - average min and max',/,
     2 '  2 - median',/,
     3 '  3 - average n-th percentile values',/,
     4 '  4 - average all values within n-th percentile (trimmed mean)'
     5 ,/,'  5 - take only n-th percentile value'
     6 ,//)
      go to 1
 3    if(imeth.eq.3.or.imeth.eq.4) then
	 write(6,110)
 110     format(' Percentile value (0-50, other one symmetric); ',$)
         perc = ffin(1,not)
      endif
      if(imeth.eq.5) then
         write(6,111)
 111     format(' Percentile value;  ',$)
         perc = ffin(1,not)
      endif
      ilast=0
      ifrst=0
      if(ifo.gt.0) call intio(ix,1,1,ifo,1)
      if(ifo.lt.0) call realio(x,1,1,ifo,1)
      ibufl = 2999 - khf
 5    irhi = min(irli+ibufl,irnt)
      if(irhi.ge.irnt) ilast=1
      call realio(x,irli,irhi,ifi,0)
      np=irhi-irli+1
c  for the first buffer, leave some space at the front which is padded
c  out so that start times will line up (this makes the immediately
c  previous call to intio unneccessary, but it is simplest to leave it in)
      if(ifrst.ne.0) go to 13
      ifrst = 1
      ibufl = 2999
      call realio(x(khf+1),irli,irhi,ifi,0)
      np = irhi-irli + khf + 1.
      do 11 i=1,khf
 11   x(i) = x(khf+1)
 13   if(ilast.eq.1) then
         do 12 i=1, khf
         x(np+i)=x(np)
 12      continue
         np=np+khf
      endif
      call perflt(x,np,med,imeth,perc)
c  np is returned as the number of points filtered
      irho = irlo + np -1
      if(ifo.lt.0) call realio(x,irlo,irho,ifo,-1)
      if(ifo.lt.0) go to 17
      do 15 i=1,np
 15   ix(i) = nint(x(i))
      call intio(ix,irlo,irho,ifo,-1)
 17   irlo = irho + 1
      irli = irhi + 2 - med
      if(irhi.lt.irnt) go to 5
      write(6,120) irho
 120  format(i8,' points written out')
      stop
      end
      subroutine perflt(ar,len,m,method,perc)
c  filters data in array ar, of length len, by some operation on
c  m-length blocks of sorted data. m must be odd and between 3 and 1001.
c  data in ar are overwritten. len is returned as the number of points
c  in the output array.  the first filtered point is put in ar(1).
c
c  for meth =1, the value returned is halfway between the min and max.
c  if it is 2, the value is the median. meth=3 and 4 are generalizations
c  of these to percentile levels perc and 100-perc; 3 gives the average
c  of the values at these percentiles, 4 the mean of the values within
c  them.  for all of these the sorting is done (after the first block)
c  by updating the sort array rather than redoing the sort each time.
c
c  original algorithim by k. r. anderson, rewritten by j. r. evans;
c  this coding by d.c. agnew, after suggestions by f. wyatt.
c  usgs openfile report 81-1091 gives details of the running median case.
c
c     $$$calls valfnd (included in this source file)
c
      dimension ar(1),sort(3001),isb(3001)
      n = (m-1)/2
      len = len - 2*n
      iold = 0
      inew = m
      io = 1
c  test for errors
      if(m.ge.3.and.m.le.1001.and.m.le.len.and.mod(m,2).ne.0) go to 10
      write(6,100) m
 100  format(1x,i4,' is not a suitable length for the median')
      stop
c  begin filtering. subscripts used are:
c      io   - subscript of ar
c      i    - general use, including oldest value seach
c      iold - oldest element of ar still in sort; isb is searched
c             for it.
c      inew - subscript in ar of element to be added to sort and isb
c
c  at start, fill sorting array and bubble sort
c
 10   do 20 i=1,m
      isb(i) = i
 20   sort(i) = ar(i)
 30   key = 0
      do 40 i=1,m-1
      if(sort(i).le.sort(i+1)) go to 40
      s = sort(i)
      sort(i) = sort(i+1)
      sort(i+1) = s
      is = isb(i)
      isb(i) = isb(i+1)
      isb(i+1) = is
      key = 1
 40   continue
      if(key.ne.0) go to 30
c  now we step along; note that at any time sort contains all elements
c  needed, so overwriting is safe
 50   ar(io) = valfnd(sort,m,method,perc)
      io = io + 1
      if(io.gt.len) return
      inew = inew + 1
      iold = iold + 1
c  find old subscript
      i = 0
 60   i = i + 1
      if(isb(i).gt.iold) go to 60
c  decide which way to move new entry
      s = ar(inew)
      if(i.eq.1) go to 70
      if(sort(i-1)-s) 70,80,77
c  move new entry up until it fits
 70   if(i.eq.m.or.sort(i+1).ge.s) go to 80
      sort(i) = sort(i+1)
      isb(i) = isb(i+1)
      i = i + 1
      go to 70
c  move new entry down until it fits
 75   if(sort(i-1).le.s) go to 80
 77   sort(i) = sort(i-1)
      isb(i) = isb(i-1)
      i = i-1
      if(i.gt.1) go to 75
c  insert new entry
 80   sort(i) = s
      isb(i) = inew
      go to 50
      end
      function valfnd(sort,m,meth,perc)
c  returns a value computed from the sorted array sort; for descriptions
c  of meth and perc see the comments to the calling routine perflt (above)
c
      dimension sort(m)
      if(meth.ge.3) go to 5
      if(meth.eq.1) valfnd = (sort(1) + sort(m))/2
      if(meth.eq.2) valfnd = sort((m+1)/2)
      return
 5    nl =(m*(perc/100.001))+1
      nu = (m - nl) +1
      if(meth.eq.3) valfnd= (sort(nl) + sort(nu))/2
      if(meth.eq.3) return
      if(meth.eq.5) valfnd= sort(nl) 
      if(meth.eq.5) return
      valfnd = 0
      do 7 i=nl,nu
 7    valfnd = valfnd + sort(i)
      valfnd = valfnd/(nu-nl+1.)
      return
      end
