       program spcmat
c program does cross-spectrum between series on files
c  inp  and  iout , finds phase admittance and coherence.
c  may start at any point in either series.  works in chunks
c  of length up to 1024, and averages over sections.
c  writes out results in real data files.  admittance can be
c  output as complex multiplexed series and relative error
c  saved on a file for least-squares fitting.
c
c  tapers available: 0  no tapering
c                    1  10% cosine taper on each section
c                    2  full triangular taper on each
c                        section, overlapping by 50%
c
c  actual cross-spectrum done with pcrosp
c $$$$$$$ link with  pcrosp,ffttwo,iolib
c
c
c
      character*1 ihel
      dimension x(16000),sp(16000),scra(8000),cf(1)
      common/ownpcr/np,nh,qn,n,nc,ktaper,np1
      alog11(dm) = alog10((2.*dm) + 1.e-38)
      formul(cof,s)=sqrt(abs(cof*(1.-s)/(s+1.0e-10))) + 1.0e-6
      write(6,1)
 1    format(' File number for input series ; ',$)
      inp = ffin(1,not)
      write(6,2)
 2    format(' File number for output series; ',$)
      iout = ffin(1,not)
      write(6,3)
 3    format(' Start and end term of input series ; ',$)
      low1=ffin(1,not)
      ihi1 = ffin(1,not)
      if(ihi1.le.0) ihi1=longis(inp)
      len = ihi1 - low1 + 1
      write(6,41)
 41   format(' Start term of output series ; ',$)
      low2=ffin(1,not)
      is1=low1
      is2=low2
      write(6,9)
 9    format(' Length of sections (composite le 8000) ; ',$)
      lens=ffin(1,not)
      write(6,7)
 7    format(' Sample interval in seconds  ; ',$)
      dt = ffin(1,not)
 5    write(6,4)
 4    format (' Enter taper, or help; ',$)
      kt = ffin(1,ihel)
      if(kt.ge.0.and.kt.le.2.and.ihel.eq.'0') go to 6
      if(kt.lt.0.or.kt.gt.2.or.ihel.ne.'h') go to 5
      write(6,51)
 51   format(' available tapers are:',/,12x,
     1 ' 0 - no tapering',/,12x,
     2 ' 1 - 10% cosine taper',/,12x,
     3 ' 2 - full triangular taper, 50% overlap')
      go to 5
 6    ip=1
      nsec=len/lens
      kmax = lens/2 + 1
      lot = lens + 1
      dt = lens*dt
      df = 1000./dt
c  put input series into 1st half of x, output in 2nd
      call realio(x,is1,is1+lens-1,inp,0)
      call realio(x(lot),is2,is2+lens-1,iout,0)
      lend = 2*lens
      if(nsec.gt.1) go to 100
      p = pcrosp(x,lens,2,-1,kmax,1,sp,scra,cf,kt,3,0)
c  p=0 unless  pcrosp  detects an error
      if (p.ne.0) go to 666
      go to 300
c  complications arise since  pcrosp works
c  differently when section is 1st,last or both
 100  p = pcrosp(x,lens,2,-1,kmax,1,sp,scra,cf,kt,1,0)
      if (p.ne.0) go to 666
      if (nsec.eq.2.and.kt.ne.2) go to 150
      if (kt.eq.2) go to 400
      nn = nsec - 2
      ll = lens
      go to 410
c  set up for 50% overlap
 400  nn = 2*nsec-2
      ll = lens/2
c  do middle sections
 410  do 200 l = 1,nn
      is1=is1+ll
      is2=is2+ll
      call realio(x,is1,is1+lens-1,inp,0)
      call realio(x(lot),is2,is2+lens-1,iout,0)
      p = pcrosp(x,lens,2,-1,kmax,1,sp,scra,cf,kt,0,0)
      if (p.ne.0) go to 666
 200  continue
c  do last sectiom
 150  is1 = ((nsec-1)*lens) + low1
      is2=((nsec-1)*lens) + low2
      call realio(x,is1,is1+lens-1,inp,0)
      call realio(x(lot),is2,is2+lens-1,iout,0)
      call retnio(inp)
      call retnio(iout)
      p = pcrosp(x,lens,2,-1,kmax,1,sp,scra,cf,kt,2,0)
      if (p.ne.0) go to 666
c  rework cross-specral matrix into meaningful form
 300  km2 = 2*kmax
      km3 = 3*kmax
      conv = 180./3.1415927
      rmax = kmax
      do 301 i = 1,kmax
      scra(i) = (sp(i+kmax))**2 + (sp(i+km2))**2
c  find phase
      sp(km2+i) = atan2(sp(km2+i),sp(kmax+i))
      sp(km2+i) = sp(km2+i)*conv
c  find squared coherence
      sp(kmax+i) = scra(i)/(sp(i)*sp(km3+i) + 1.e-38)
c  find admittance
      scra(i) = sqrt(scra(i))/(sp(i) + 1.e-38)
c  put input,output power into decibels
      sp(i) = 10.*alog11(sp(i))
 301  sp(km3+i) = 10.*alog11(sp(km3+i))
      if(ip.eq.0) go to 350
c  find coeffs for 95% confidence calculations
      deg=2.*nsec
      if (kt.eq.2) deg=2.8*nsec
      deg=amax1(deg-2.,3.)
      cof=(2./deg)*(3.+(9./(deg-.95))+(13./deg**2))
c
c
      write(6,999)
 999  format(/)
 64   write(6,1002)
 1002 format(' enter start, end, spacing, bands'
     &,' (0,0,0 for no printing); ',$)
      i1=ffin(1,not)
      i2=ffin(1,not)
      i3=ffin(1,not)
      if(i1.eq.0) go to 65
      write(6,999)
      write(6,1000)
 1000 format(' band',3x,'frequency',3x,'input-power-output',
     1   2x,'coher**2',5x,'admitt',6x,'phase',3x,'rel err')
c  print out results as requested, a line at a time
      do 77 i=i1,i2,i3
      rel=formul(cof,sp(kmax+i))
 77   write(6,1001)i,(i-1)*df,sp(i),sp(km3+i),sp(kmax+i),scra(i),
     & sp(km2+i),rel
 1001 format(i4,g13.4,3x,2g10.4,f8.5,g13.4,f9.2,f10.4)
      go to 64
 65   write(6,651)
 651  format(' enter 1 for file to complex admittance, 2 for'
     &,' powers, coherence etc; ',$)
      igo=ffin(1,not)
      go to (8000,6500),igo
 6500    write(6,15)
 15    format(' enter file numbers for input and output power ',/,
     1 ' coherence, phase, and amplitude (0 for unwanted); ',$)
      if1 = ffin(1,not)
      if2 = ffin(1,not)
      if3 = ffin(1,not)
      if4 = ffin(1,not)
      if5 = ffin(1,not)
 350  if(if1.ne.0) call realio(sp,1,kmax,if1,1)
      if(if1.ne.0) call retnio(if1)
      if(if2.ne.0) call realio(sp(km3+1),1,kmax,if2,1)
      if(if2.ne.0) call retnio(if2)
      if(if3.ne.0) call realio(sp(kmax+1),1,kmax,if3,1)
      if(if3.ne.0) call retnio(if3)
      if(if4.ne.0) call realio(sp(km2+1),1,kmax,if4,1)
      if(if4.ne.0) call retnio(if4)
      if(if5.ne.0) call realio(scra,1,kmax,if5,1)
      if(if5.ne.0) call retnio(if5)
      write(6,10)if1,if2,if3,if4,if5,rmax
 10   format(' input and output power in ',i7,' and ',i7,/,
     1 ' coherence squared in ',i7,/,' phase and admittance in ',
     2 i7,' and ',i7,/,5x,'all with',f7.0,' terms')
      go to 700
 666  write(6,55)p
 55   format (1x,'error, p = ',f3.1)
 700  stop
c
c  output complex admittance and relative error files
c
 8000 write(6,800)
 800  format(' enter files for admittance and rel error; ',$)
      if1=ffin(1,not)
      if2=ffin(1,not)
      max=rmax+.01
      do 8100 i=1,max
      phi=sp(km2+i)/conv
      sp(2*i-1)=scra(i)*cos(phi)
      sp(2*i)=scra(i)*sin(phi)
      x(i)=formul(cof,sp(kmax+i))
 8100 continue
      call realio(sp,1,2*kmax,if1,1)
      call retnio(if1)
      call realio(x,1,kmax,if2,1)
      call retnio(if2)
      write(6,810) if1,2*rmax,if2,rmax
 810  format(' complex admittance in',i7,' with',f10.0,
     &' terms'/' rel error in',i7,' with',f10.0,' terms')
      stop
      end
