c   program for computing times of sunrise and sunset
c
c     uses solar ephemeris from ertid, and steps along by minutes,
c   noting times of sunrise and sunset
c
c     calls tidlib, genlib (for date conversion)
c
      character*80 dumm
      character*1 md,mm
      character*3 name
      double precision tt,ts,te,d,zhr,yhr,t
      real iyr,jyr,iday,jday,lamda
      dimension it1(5),it2(5)
      dimension it1m(6),it2m(6)
      common/bpos/coor(8),par,sdist
      common/obs/ctheta,stheta,clg,slg,dif,radn,gl
      common/tdiff/etmut
      common/sunny/moon
      data moon/2/
      data it1/5*0/,it1m/6*0/,md/'d'/,mm/'m'/,isamp/60/,icount/0/,
     1    cuold/0./,in/-1/
c dip in minutes, from Suppl. Astron. Eph.
      data dip/50/,pi/3.14159265/
      if(iargc().ne.5) then
         write(6,100)
 100     format('Usage: suntimes yr day days lat long',//,
     2  'where  year and day give the start time (0h UT) ',/,
     3  '       days is the number of days to compute for',/,
     4  '       lat and long are the locations; e.g. ',//,
     5  '    suntimes 2007 204 1 33.609 -116.455 ',//,
     6  '       will print ',//,
     7  ' set  2007 204  2 53   ( 7 23) 204.120  Mon',/,
     8  ' rise 2007 204 12 53   ( 7 23) 204.537  Mon',/,
     9  ' noon 2007 204 19 53   ( 7 23) 204.828  Mon',//,
     1  '       where all the times are UTC',/)
         stop
      endif
      cdip=cos(pi/2+(pi/180)*(dip/60.))
      call getarg(1,dumm)
      read(dumm,102) i
 102  format(i6)
      iyr=i
      it1(1)=i
      it1m(1)=i
      call getarg(2,dumm)
      read(dumm,102) i
      iday=i
      it1(2)=i
      call julmon(it1,it1m)
      zhr=0.
c time is saved in array for use with scans/date routines
      call getarg(3,dumm)
      read(dumm,102) i
      is=86400
      call timfnd(it1,i,is,it2,md)
      iyr=iyr-1900
      jyr=it2(1)-1900
      jday=it2(2)
      yhr=0
      d=1.d0/60.d0
      tz=0.
c   find times in hours from 0 hr, 1 jan 1900
      ts=zhr+24.d0*(iday-1)+8760.d0*iyr+24.d0*int((iyr-1)/4)
      te=yhr+24.d0*(jday-1)+8760.d0*jyr+24.d0*int((jyr-1)/4)
      terms=(te-ts)/d
      call getarg(4,dumm)
      read(dumm,104) theta
 104  format(f7.0)
      call getarg(5,dumm)
      read(dumm,104) lamda
c
c                  begin execution
c
      i = 1
      tt = ts
      call sph(theta,lamda,0.)
      etmut = 41.184 + iyr - 70
c   begin loop through times
 10   if(tt.gt.te) stop
      t = (tt+12.d0 + (etmut/3600.d0))/876600.d0
c   t is ephemeris time in julian centuries from 12 hr 0 jan 1900
      call ephem(t)
c    calculate cosine of zenith angle
      cll = clg*coor(8) + slg*coor(7)
      sll = slg*coor(8) - clg*coor(7)
      sz = coor(5)
      cz = coor(6)
      cu = ctheta*coor(6) + stheta*coor(5)*cll
      if(cu.le.cdip.and.cuold.gt.cdip) then
        call timfnd(it1,icount,isamp,it2,md)
        call timfnd(it1m,icount,isamp,it2m,mm)
        call weekday(it2,name,md)
        day=it2(2)+it2(3)/24.+it2(4)/1440.
        write(6,110) (it2(i),i=1,4),(it2m(i),i=2,3),day,name
 110    format('set  ',i4,1x,i3,1x,i2,1x,i2,3x,'(',i2,1x,i2,')',
     1      f8.3,2x,a3)
        in=0
      endif
      if(cu.gt.0.and.cuold.gt.cu.and.in.eq.0) then
        in=1
        call timfnd(it1,icount,isamp,it2,md)
        call timfnd(it1m,icount,isamp,it2m,mm)
        call weekday(it2,name,md)
        day=it2(2)+it2(3)/24.+it2(4)/1440.
        write(6,111) (it2(i),i=1,4),(it2m(i),i=2,3),day,name
 111    format('noon ',i4,1x,i3,1x,i2,1x,i2,3x,'(',i2,1x,i2,')',
     1      f8.3,2x,a3)
      endif
      if(cu.ge.cdip.and.cuold.lt.cdip) then
        call timfnd(it1,icount,isamp,it2,md)
        call timfnd(it1m,icount,isamp,it2m,mm)
        call weekday(it2,name,md)
        day=it2(2)+it2(3)/24.+it2(4)/1440.
        write(6,112) (it2(i),i=1,4),(it2m(i),i=2,3),day,name
 112    format('rise ',i4,1x,i3,1x,i2,1x,i2,3x,'(',i2,1x,i2,')',
     1      f8.3,2x,a3)
      endif
      cuold=cu
      tt = tt + d
      tz = tz + d
      icount=icount+1
      go to 10
      end
