c      program tidspc
c
c  does cross-spectrum between two tidal series, the input being
c  presumed to be a theoretical tide (and hence noise-free) and
c  both it and the output being sampled at 3-hour intervals.
c  cross-spectral parameters are found for each tidal group
c  (cycle/month separation), first averaging over sections
c  (which are windowed with a cosine bell, and overlapped 50%),
c  and then over harmonics.  because a slow fourier transform is
c  used the section length is arbitrary, but a good one is 1093
c  points, and averaging over 3 harmonics. (this puts the
c  tidal groups on harmonics).
c
c    calls sft,dtren,realio
c
      character*1 not
      dimension x(2840),cspc(7,18),qspc(7,18),asi(7,18),aso(7,18),
     1 psi(18),pso(18),csq(18),cohp(18),uncp(18),radm(18),urad(18),
     2 sig(18),tpow(7,18),rc(7,18,2),rs(7,18,2)
      dbz(dum) = 10.*alog10(dum+1.e-9)
      data pi/3.1415926536/,cspc,qspc,tpow,asi,aso/126*0.,
     1 126*0.,126*0.,126*0.,126*0./,psi,pso,csq,cohp,uncp,radm
     2 /18*0.,18*0.,18*0.,18*0.,18*0.,18*0./
      write(6,100)
 100  format(' input (noise-free) file,start and end term ; ',$)
      ifi = ffin(1,not)
      irli = ffin(1,not)
      irhi = ffin(1,not)
      write(6,102)
 102  format(' output file and start term                 ; ',$)
      ifo = ffin(1,not)
      irlo = ffin(1,not)
      rnt = irhi-irli+1.
      write(6,105)
 105  format(' type 1 for overlap and taper, 0 for neither; ',$)
      iove = ffin(1,not)
      if(iove.le.0) iove = 0
      if(iove.ge.1) iove = 1
      write(6,106)
 106  format(' transform length                          ; ',$)
      tl = ffin(1,not)
      itl = tl
      write(6,107)
 107  format(' type 1 for spectrum in physical units     ; ',$)
      iphz = ffin(1,not)
      pcor = 0.
      if(iphz.ne.1) go to 3
      write(6,120)
 120  format(' units per l.c.                              ',$)
      gain = ffin(1,not)
      pcor = 43.445 + 20.*alog10(gain)
 3    nsec = (2.*rnt)/tl -1.
      if(iove.eq.0) nsec = rnt/tl
      write(6,108)
 108  format(' number of harmonics to average over (odd)  ; ',$)
      nhav = ffin(1,not)
c  find the harmonics at which the tidal groups are centered, for
c  3 hour data and the given section length
      cf = (.966136708*tl)/8.
      nch = cf + .5
      fbg = (.0366011023*tl)/8.
      nhg = fbg + .5
      if(nsec.lt.1) stop
      do 25 ii = 1,nsec
      do 20 kk = 1,2
      irhi = irli + tl -1
      irho = irlo + tl -1.
      if(kk.eq.1) call realio(x,irli,irhi,ifi,0)
      if(kk.eq.2) call realio(x,irlo,irho,ifo,0)
      if(iove.eq.0) go to 7
      call dtren(x,itl)
      do 5 i = 1,itl
 5    x(i) = x(i)*(1. - cos((2.*pi*(i-1.))/tl))
 7    do 10 jj = 1,18
      om = nhg*(mod(jj-1,9)-4) + nch - (nhav/2)
      if(jj.gt.9) om = om + nch
      dom = 1./tl
      om = om/tl
      call sft(x,itl,om,dom,nhav,rs(1,jj,kk),rc(1,jj,kk))
 10   continue
 20   continue
      irli = irli + int(tl/2)*iove + tl*(1-iove)
      irlo = irlo + int(tl/2)*iove + tl*(1-iove)
      do 21 k = 1,18
      do 21 j = 1,nhav
      cspc(j,k) = cspc(j,k) + rs(j,k,1)*rs(j,k,2) 
     1                      + rc(j,k,1)*rc(j,k,2)
c  sign convention for quadrature spectrum is -i (delay gives
c  a phase lag)
      qspc(j,k) = qspc(j,k) - rs(j,k,1)*rc(j,k,2) 
     1                      + rc(j,k,1)*rs(j,k,2)
      sp = rc(j,k,1)**2 + rs(j,k,1)**2
      sp2 = rc(j,k,2)**2 + rs(j,k,2)**2
      asi(j,k) = asi(j,k) + sp
      aso(j,k) = aso(j,k) + sp2
 21   tpow(j,k) = tpow(j,k) + sqrt(sp*sp2)
 25   continue
      do 35 j = 1,18
      do 35 i = 1,nhav
      psi(j) = psi(j) + asi(i,j)
      pso(j) = pso(j) + aso(i,j)
      cohp(j) = cohp(j) + cspc(i,j)
      uncp(j) = uncp(j) + qspc(i,j)
 35   radm(j) = radm(j)  + tpow(i,j)
      p = nhav*int(1.4*rnt/tl)
      if(iove.eq.0) p = .75*nhav*nsec
      df = 2.*p
      rnt = 1.5*tl*nsec*nhav
      if(iove.eq.0) rnt = tl*nsec*nhav
c  compute various cross-spectral parameters
      do 40 i = 1,18
      csq(i) = (cohp(i)**2+uncp(i)**2)/(psi(i)*pso(i))
      radm(i) = cohp(i)/psi(i)
      urad(i) = uncp(i)/psi(i)
      x(i) = sqrt(radm(i)**2+urad(i)**2)
      x(i+30) = (180./pi)*atan2(urad(i),radm(i))
      csq(i) = (p*csq(i)-1.)/(p-1.)
      if(csq(i).le.0.) csq(i) = 1.e-4
      cohp(i) = dbz(csq(i)*pso(i)/rnt)+ pcor
      uncp(i) = dbz(amax1((1.-csq(i))*pso(i)/rnt,0.))
     1  + pcor
      psi(i) = dbz(psi(i)/rnt)+ pcor
      pso(i) = dbz(pso(i)/rnt) + pcor
 40   sig(i) = sqrt(amax1(((1./csq(i)) -1.)/df,0.))
      write(6,109) abs(cf-nch), abs(fbg-nhg)
 109  format(' position error ',g13.5,' space error ',g13.5//)
      write(6,110)
 110  format(1x,'group',3x,'input',3x,'output',3x,'coh**2',
     1 5x,'incoh',3x,'coher',3x,'real',12x,'imag',9x,
     2 'error',10x,'amp',6x,'phase',//)
      write(6,114) (i/10+1,mod(i-1,9)-4,psi(i),pso(i),csq(i),
     1 uncp(i),cohp(i),radm(i),urad(i),sig(i),x(i),x(i+30),
     2 i = 1,9)
 114  format(1x,i1,i3,3x,f7.2,1x,f7.2,1x,f7.5,2x,f6.1,2x,
     1 f6.1,1x,g13.5,2x,g13.5,2x,g13.5,2x,g13.5,2x,f7.2)
      write(6,115)
 115  format(1x,/)
      write(6,114) (i/10+1,mod(i-1,9)-4,psi(i),pso(i),csq(i),
     1 uncp(i),cohp(i),radm(i),urad(i),sig(i),x(i),x(i+30),
     2 i = 10,18)
      write(6,118)
 118  format(///)
      stop
      end
