c    program for generating different sorts of random noise,
c      including white noise (with a variety of amplitude 
c      distributions), a random binary signal, brownian noise,      
c      and 1/f noise     
c   
c  calls iolib, and ran1 (random number generator from Numerical Recipes)    
c   
      character*1 not
      dimension x(600),xp(32)  
      data nt,ir/1,-2/,x0/0./,irl/1/,maxopt/8/,inte/1/
      x(1)=ran1(ir)
      write(6,100)      
 100  format(' File number (negative) and length; ',$)   
      ifo = ffin(1,not)
      irnt = ffin(1,not) 
      call realio(x,1,1,ifo,1)
 3    write(6,102)      
 102  format(' Type of output (h for list); ',$)     
      ity = ffin(1,not) 
      if(not.eq.'h') go to 4
      if(ity.le.0.or.ity.ge.maxopt) go to 3
      go to 5
 4    write(6,104)      
 104  format(3x,' 1 - gaussian**n white',/4x,'2 - uniform**n white',/,  
     1 4x,'3 - random binary',/4x,'4 - brownian (f**-2)',/,4x,    
     2 '5 - 1/f noise',/4x,'6 - cauchy white',/,4x,
     3 '7 - logistic white') 
      go to 3    
 5    if(ity.eq.1) write(6,1001)
 1001 format(' Square root of variance; ',$)
      if(ity.eq.2) write(6,1002)
 1002 format(' Maximum + and - value; ',$)
      if(ity.eq.3) write(6,1003)
 1003 format(' Amplitude (values will be 0 or this); ',$)
      if(ity.eq.4) write(6,1004)
 1004 format(' Frequency for spectrum at 0 db (nyquists); ',$)
      if(ity.ge.5) write(6,1005)
 1005 format(' Scaling factor; ',$)
      fac1 = ffin(1,not)
      if(ity.eq.6) write(6,1002)
      if(ity.eq.6) fac2 = ffin(1,not)
      if(ity.eq.1.or.ity.eq.2) write(6,1055)
 1055 format(' Power to raise distribution to (integer); ',$)
      if(ity.eq.3) write(6,1006)
 1006 format(' Interpolating factor (must divide 600); ',$)
      if(ity.eq.3.or.ity.eq.2.or.ity.eq.1) inte = ffin(1,not) 
      nrt = 600/inte    
      nsec = int((irnt-1.)/600.) + 1.  
      if(ity.ne.5) go to 9     
c   do setup for 1/f generator  
      do 7 i =1,32
 7    xp(i) = ran1(ir)
 9    do 1000 k=1,nsec  
      go to (11,21,31,41,51,61,71) ity      
c    white noise, gaussian distribution (Box-Muller transformation)
 11   do 15 j=1,600     
 15   x(j) = (fac1*grand(ir))**inte
      go to 900  
c    white noise, uniform distribution 
 21   do 23 j=1,600     
      xd=ran1(ir)
 23   x(j) = fac1*((2.*xd-1.)**inte)
      go to 900  
c    random binary signal
 31   do 35 j=1,nrt     
      xd = int(ran1(ir) + .5)
      do 33 i=1,inte    
      ind = (j-1)*inte + i     
 33   x(ind) = fac1*xd  
 35   continue   
      go to 900  
c    brownian noise (gaussian steps)   
 41   sc = 9.*(fac1**2) 
      do 45 j=1,600     
      xd = x0 + fac1*grand(ir)
      x0 = xd    
 45   x(j) = xd  
      go to 900  
c    1/f noise    
 51   sc = fac1  
      do 57 j=1,600     
      x(j) = 0.  
      do 53 l = 1,32    
 53   x(j) = x(j) + xp(l)      
      x(j) = .75*sc*(x(j)-16.)  
      l = 2      
      nt = nt + 1
      n = nt     
      xp(1) = ran1(ir)
 55   m = n/2    
      if(2*m.ne.n) go to 57    
      xp(l) = ran1(ir)
      l = l + 1  
      n = m      
      go to 55   
 57   continue   
      go to 900
c    white noise, cauchy distribution
 61   do 65 j=1,600     
      xd2 = grand(ir)
      xd = grand(ir)     
      x(j) = fac1*(xd/xd2)      
      if(x(j).gt.abs(fac2)) x(j) = abs(fac2)
 65   if(x(j).lt.-abs(fac2)) x(j) = -abs(fac2)
      go to 900  
c    white noise, logistic distribution
 71   do 75 j=1,600     
      xd = ran1(ir)
 75   x(j) = fac1*alog(xd/(1-xd))
      go to 900  
 900  irh = min0(irl+599,irnt)  
      call realio(x,irl,irh,ifo,-1)      
 1000 irl = irh + 1
      write(6,130) irnt,ifo     
 130  format(2x,i8,' terms written to file',i6)   
      stop
      end 
      function grand(ir)
c    gaussian distribution (Box-Muller transformation) - after Numerical Recipes
      data iset/0/
      i=0
      if(iset.eq.0) then
 13      v1=2.*ran1(ir)-1
         v2=2.*ran1(ir)-1
         i=i+1
         if(i.eq.50) stop
         rr = v1**2 + v2**2
         if(rr.ge.1) go to 13
         xx = sqrt(-2.*alog(rr)/rr)
         gsave= v1*xx
         grand = v2*xx
         iset = 1
      else
         grand = gsave
         iset = 0
      endif
      return
      end
      function ran1(idum)
      parameter (ia=16807,im=2147483647,am=1./im,iq=127773, ir=2836,
     1     ntab=32,ndiv=1+(im-1)/ntab,eps=1.2e-7,rnmx=1.-eps)
      dimension iv(ntab)
      save iv,iy
      data iv/ntab*0/, iy/0/
      if(idum.le.0.or.iy.eq.0) then
        idum=max(-idum,1)
        do 11 j=ntab+8,1,-1
          k=idum/iq
          idum=ia*(idum-k*iq)-ir*k
          if(idum.lt.0) idum=dum+im
          if(j.le.ntab) iv(j)=idum
 11     continue
        iy=iv(1)
      endif
      k=idum/iq
      idum=ia*(idum-k*iq)-ir*k
      if(idum.lt.0) idum=idum+im
      j=1+iy/ndiv
      iy=iv(j)
      iv(j)=idum
      ran1=min(am*iy,rnmx)
      return
      end
